@@ -37,10 +37,10 @@
 #include "option.h"
 #include "types.h"
 #include "game-cmd.h"
-#include "object/constants.h"
-#include "object/types.h"
+#include "object/obj-flag.h"
 #include "object/object.h"
-#include "monster/types.h"
+#include "monster/constants.h"
+#include "monster/monster.h"
 #include "player/types.h"
 #include "player/player.h"
 #include "store.h"
