@@ -22,7 +22,7 @@
 #include "cmds.h"
 #include "effects.h"
 #include "game-cmd.h"
-#include "object/object.h"
+#include "object/inventory.h"
 #include "object/tvalsval.h"
 #include "spells.h"
 #include "target.h"
@@ -35,16 +35,16 @@
 static int check_devices(object_type *o_ptr)
 {
 	int fail;
-	const char *msg;
+	const char *action;
 	const char *what = NULL;
 
 	/* Get the right string */
 	switch (o_ptr->tval)
 	{
-		case TV_ROD:   msg = "zap the rod";   break;
-		case TV_WAND:  msg = "use the wand";  what = "wand";  break;
-		case TV_STAFF: msg = "use the staff"; what = "staff"; break;
-		default:       msg = "activate it";  break;
+		case TV_ROD:   action = "zap the rod";   break;
+		case TV_WAND:  action = "use the wand";  what = "wand";  break;
+		case TV_STAFF: action = "use the staff"; what = "staff"; break;
+		default:       action = "activate it";  break;
 	}
 
 	/* Figure out how hard the item is to use */
@@ -53,16 +53,16 @@ static int check_devices(object_type *o_ptr)
 	/* Roll for usage */
 	if (randint1(1000) < fail)
 	{
-		if (OPT(flush_failure)) flush();
-		msg_format("You failed to %s properly.", msg);
+		flush();
+		msg("You failed to %s properly.", action);
 		return FALSE;
 	}
 
 	/* Notice empty staffs */
-	if (what && o_ptr->pval <= 0)
+	if (what && o_ptr->pval[DEFAULT_PVAL] <= 0)
 	{
-		if (OPT(flush_failure)) flush();
-		msg_format("The %s has no charges left.", what);
+		flush();
+		msg("The %s has no charges left.", what);
 		o_ptr->ident |= (IDENT_EMPTY);
 		return FALSE;
 	}
@@ -151,7 +151,7 @@ static void activation_message(object_type *o_ptr, const char *message)
 				end += object_desc(buf, 1024, o_ptr, ODESC_PREFIX | ODESC_BASE); 
 				break;
 			case ART_TAG_KIND:
-				object_kind_name(&buf[end], 1024-end, o_ptr->k_idx, TRUE);
+				object_kind_name(&buf[end], 1024-end, o_ptr->kind, TRUE);
 				end += strlen(&buf[end]);
 				break;
 			case ART_TAG_VERB:
@@ -175,7 +175,7 @@ static void activation_message(object_type *o_ptr, const char *message)
 	}
 	strnfcat(buf, 1024, &end, in_cursor);
 
-	msg_print(buf);
+	msg("%s", buf);
 }
 
 
@@ -188,7 +188,7 @@ void do_cmd_uninscribe(cmd_code code, cmd_arg args[])
 	object_type *o_ptr = object_from_item_idx(args[0].item);
 
 	if (obj_has_inscrip(o_ptr))
-		msg_print("Inscription removed.");
+		msg("Inscription removed.");
 
 	o_ptr->note = 0;
 
@@ -207,47 +207,6 @@ void do_cmd_inscribe(cmd_code code, cmd_arg args[])
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
 }
 
-void textui_obj_inscribe(object_type *o_ptr, int item)
-{
-	char o_name[80];
-	char tmp[80] = "";
-
-	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
-	msg_format("Inscribing %s.", o_name);
-	message_flush();
-
-	/* Use old inscription */
-	if (o_ptr->note)
-		strnfmt(tmp, sizeof(tmp), "%s", quark_str(o_ptr->note));
-
-	/* Get a new inscription (possibly empty) */
-	if (get_string("Inscription: ", tmp, sizeof(tmp)))
-	{
-		cmd_insert(CMD_INSCRIBE);
-		cmd_set_arg_item(cmd_get_top(), 0, item);
-		cmd_set_arg_string(cmd_get_top(), 1, tmp);
-	}
-}
-
-
-/*** Examination ***/
-void textui_obj_examine(object_type *o_ptr, int item)
-{
-	char header[120];
-
-	textblock *tb;
-	region area = { 0, 0, 0, 0 };
-
-	track_object(item);
-
-	tb = object_info(o_ptr, OINFO_NONE);
-	object_desc(header, sizeof(header), o_ptr, ODESC_PREFIX | ODESC_FULL);
-
-	textui_textblock_show(tb, area, format("%^s", header));
-	textblock_free(tb);
-}
-
-
 
 /*** Taking off/putting on ***/
 
@@ -258,13 +217,13 @@ void do_cmd_takeoff(cmd_code code, cmd_arg args[])
 
 	if (!item_is_available(item, NULL, USE_EQUIP))
 	{
-		msg_print("You are not wielding that item.");
+		msg("You are not wielding that item.");
 		return;
 	}
 
 	if (!obj_can_takeoff(object_from_item_idx(item)))
 	{
-		msg_print("You cannot take off that item.");
+		msg("You cannot take off that item.");
 		return;
 	}
 
@@ -273,6 +232,141 @@ void do_cmd_takeoff(cmd_code code, cmd_arg args[])
 	p_ptr->energy_use = 50;
 }
 
+
+/*
+ * Wield or wear a single item from the pack or floor
+ */
+void wield_item(object_type *o_ptr, int item, int slot)
+{
+	object_type object_type_body;
+	object_type *i_ptr = &object_type_body;
+
+	const char *fmt;
+	char o_name[80];
+
+	bool combined_ammo = FALSE;
+	bool track_wielded_item = FALSE;
+	int num = 1;
+
+	/* If we are stacking ammo in the quiver */
+	if (obj_is_ammo(o_ptr))
+	{
+		num = o_ptr->number;
+		combined_ammo = object_similar(o_ptr, &p_ptr->inventory[slot],
+			OSTACK_QUIVER);
+	}
+
+	/* Take a turn */
+	p_ptr->energy_use = 100;
+
+	/* Obtain local object */
+	object_copy(i_ptr, o_ptr);
+
+	/* Modify quantity */
+	i_ptr->number = num;
+
+	/* Update object_idx if necessary, once object is in slot */
+	if (tracked_object_is(item))
+	{
+		track_wielded_item = TRUE;
+	}
+
+	/* Decrease the item (from the pack) */
+	if (item >= 0)
+	{
+		inven_item_increase(item, -num);
+		inven_item_optimize(item);
+	}
+
+	/* Decrease the item (from the floor) */
+	else
+	{
+		floor_item_increase(0 - item, -num);
+		floor_item_optimize(0 - item);
+	}
+
+	/* Get the wield slot */
+	o_ptr = &p_ptr->inventory[slot];
+
+	if (combined_ammo)
+	{
+		/* Add the new ammo to the already-quiver-ed ammo */
+		object_absorb(o_ptr, i_ptr);
+	}
+	else 
+	{
+		/* Take off existing item */
+		if (o_ptr->kind)
+			(void)inven_takeoff(slot, 255);
+
+		/* If we are wielding ammo we may need to "open" the slot by shifting
+		 * later ammo up the quiver; this is because we already called the
+		 * inven_item_optimize() function. */
+		if (slot >= QUIVER_START)
+			open_quiver_slot(slot);
+	
+		/* Wear the new stuff */
+		object_copy(o_ptr, i_ptr);
+
+		/* Increment the equip counter by hand */
+		p_ptr->equip_cnt++;
+	}
+
+	/* Increase the weight */
+	p_ptr->total_weight += i_ptr->weight * num;
+
+	/* Track object if necessary */
+	if (track_wielded_item)
+	{
+		track_object(slot);
+	}
+
+	/* Do any ID-on-wield */
+	object_notice_on_wield(o_ptr);
+
+	/* Where is the item now */
+	if (slot == INVEN_WIELD)
+		fmt = "You are wielding %s (%c).";
+	else if (slot == INVEN_BOW)
+		fmt = "You are shooting with %s (%c).";
+	else if (slot == INVEN_LIGHT)
+		fmt = "Your light source is %s (%c).";
+	else if (combined_ammo)
+		fmt = "You combine %s in your quiver (%c).";
+	else if (slot >= QUIVER_START && slot < QUIVER_END)
+		fmt = "You add %s to your quiver (%c).";
+	else
+		fmt = "You are wearing %s (%c).";
+
+	/* Describe the result */
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
+
+	/* Message */
+	msgt(MSG_WIELD, fmt, o_name, index_to_label(slot));
+
+	/* Cursed! */
+	if (cursed_p(o_ptr->flags))
+	{
+		/* Warn the player */
+		msgt(MSG_CURSED, "Oops! It feels deathly cold!");
+
+		/* Sense the object */
+		object_notice_curses(o_ptr);
+	}
+
+	/* Save quiver size */
+	save_quiver_size(p_ptr);
+
+	/* See if we have to overflow the pack */
+	pack_overflow();
+
+	/* Recalculate bonuses, torch, mana */
+	p_ptr->notice |= PN_SORT_QUIVER;
+	p_ptr->update |= (PU_BONUS | PU_TORCH | PU_MANA);
+	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
+}
+
+
 /* Wield or wear an item */
 void do_cmd_wield(cmd_code code, cmd_arg args[])
 {
@@ -287,21 +381,21 @@ void do_cmd_wield(cmd_code code, cmd_arg args[])
 
 	if (!item_is_available(item, NULL, USE_INVEN | USE_FLOOR))
 	{
-		msg_print("You do not have that item to wield.");
+		msg("You do not have that item to wield.");
 		return;
 	}
 
 	/* Check the slot */
 	if (!slot_can_wield_item(slot, o_ptr))
 	{
-		msg_print("You cannot wield that item there.");
+		msg("You cannot wield that item there.");
 		return;
 	}
 
 	equip_o_ptr = &p_ptr->inventory[slot];
 
 	/* If the slot is open, wield and be done */
-	if (!equip_o_ptr->k_idx) 
+	if (!equip_o_ptr->kind)
 	{
 		wield_item(o_ptr, item, slot);
 		return;
@@ -316,10 +410,10 @@ void do_cmd_wield(cmd_code code, cmd_arg args[])
 	}
 
 	/* Prevent wielding into a cursed slot */
-	if (cursed_p(equip_o_ptr))
+	if (cursed_p(equip_o_ptr->flags))
 	{
 		object_desc(o_name, sizeof(o_name), equip_o_ptr, ODESC_BASE);
-		msg_format("The %s you are %s appears to be cursed.", o_name,
+		msg("The %s you are %s appears to be cursed.", o_name,
 				   describe_use(slot));
 		return;
 	}
@@ -348,14 +442,14 @@ void do_cmd_drop(cmd_code code, cmd_arg args[])
 
 	if (!item_is_available(item, NULL, USE_INVEN | USE_EQUIP))
 	{
-		msg_print("You do not have that item to drop it.");
+		msg("You do not have that item to drop it.");
 		return;
 	}
 
 	/* Hack -- Cannot remove cursed items */
-	if ((item >= INVEN_WIELD) && cursed_p(o_ptr))
+	if ((item >= INVEN_WIELD) && cursed_p(o_ptr->flags))
 	{
-		msg_print("Hmmm, it seems to be cursed.");
+		msg("Hmmm, it seems to be cursed.");
 		return;
 	}
 
@@ -363,36 +457,31 @@ void do_cmd_drop(cmd_code code, cmd_arg args[])
 	p_ptr->energy_use = 50;
 }
 
-void textui_obj_wield(object_type *o_ptr, int item)
+/* Destroy an item */
+void do_cmd_destroy(cmd_code code, cmd_arg args[])
 {
-	int slot = wield_slot(o_ptr);
+	object_type *o_ptr;
+	int item = args[0].item;
 
-	/* Usually if the slot is taken we'll just replace the item in the slot,
-	 * but in some cases we need to ask the user which slot they actually
-	 * want to replace */
-	if (p_ptr->inventory[slot].k_idx)
+	if (!item_is_available(item, NULL, USE_INVEN | USE_EQUIP | USE_FLOOR))
 	{
-		if (o_ptr->tval == TV_RING)
-		{
-			cptr q = "Replace which ring? ";
-			cptr s = "Error in obj_wield, please report";
-			item_tester_hook = obj_is_ring;
-			if (!get_item(&slot, q, s, CMD_WIELD, USE_EQUIP)) return;
-		}
-
-		if (obj_is_ammo(o_ptr) && !object_similar(&p_ptr->inventory[slot],
-			o_ptr, OSTACK_QUIVER))
-		{
-			cptr q = "Replace which ammunition? ";
-			cptr s = "Error in obj_wield, please report";
-			item_tester_hook = obj_is_ammo;
-			if (!get_item(&slot, q, s, CMD_WIELD, USE_EQUIP)) return;
-		}
+		msg("You do not have that item to ignore it.");
+		return;
 	}
 
-	cmd_insert(CMD_WIELD);
-	cmd_set_arg_item(cmd_get_top(), 0, item);
-	cmd_set_arg_number(cmd_get_top(), 1, slot);
+	o_ptr = object_from_item_idx(item);
+
+	if ((item >= INVEN_WIELD) && cursed_p(o_ptr->flags)) {
+		msg("You cannot ignore cursed items.");
+	} else {	
+		char o_name[80];
+
+		object_desc(o_name, sizeof o_name, o_ptr, ODESC_PREFIX | ODESC_FULL);
+		msgt(MSG_DESTROY, "Ignoring %s.", o_name);
+
+		o_ptr->ignore = TRUE;
+		p_ptr->notice |= PN_SQUELCH;
+	}
 }
 
 
@@ -433,7 +522,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	{
 		if (!obj_can_zap(o_ptr))
 		{
-			msg_print("That rod is still charging.");
+			msg("That rod is still charging.");
 			return;
 		}
 
@@ -445,7 +534,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	{
 		if (!obj_has_charges(o_ptr))
 		{
-			msg_print("That wand has no charges.");
+			msg("That wand has no charges.");
 			return;
 		}
 
@@ -457,7 +546,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	{
 		if (!obj_has_charges(o_ptr))
 		{
-			msg_print("That staff has no charges.");
+			msg("That staff has no charges.");
 			return;
 		}
 
@@ -491,7 +580,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	{
 		if (!obj_can_activate(o_ptr))
 		{
-			msg_print("That item is still charging.");
+			msg("That item is still charging.");
 			return;
 		}
 
@@ -501,13 +590,13 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	}
 	else
 	{
-		msg_print("The item cannot be used at the moment");
+		msg("The item cannot be used at the moment");
 	}
 
 	/* Check if item is within player's reach. */
 	if (items_allowed == 0 || !item_is_available(item, NULL, items_allowed))
 	{
-		msg_print("You cannot use that item from its current location.");
+		msg("You cannot use that item from its current location.");
 		return;
 	}
 
@@ -522,22 +611,23 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 		dir = args[1].direction;
 
 	/* Check for use if necessary, and execute the effect */
-	if ((use != USE_CHARGE && use != USE_TIMEOUT) ||
-	    check_devices(o_ptr))
+	if ((use != USE_CHARGE && use != USE_TIMEOUT) || check_devices(o_ptr))
 	{
+		int beam = beam_chance(o_ptr->tval);
+
 		/* Special message for artifacts */
-		if (artifact_p(o_ptr))
+		if (o_ptr->artifact)
 		{
-			message(snd, 0, "You activate it.");
-			if (a_info[o_ptr->name1].effect_msg)
-				activation_message(o_ptr, a_info[o_ptr->name1].effect_msg);
-			level = a_info[o_ptr->name1].level;
+			msgt(snd, "You activate it.");
+			if (o_ptr->artifact->effect_msg)
+				activation_message(o_ptr, o_ptr->artifact->effect_msg);
+			level = o_ptr->artifact->level;
 		}
 		else
 		{
 			/* Make a noise! */
 			sound(snd);
-			level = k_info[o_ptr->k_idx].level;
+			level = o_ptr->kind->level;
 		}
 
 		/* A bit of a hack to make ID work better.
@@ -545,25 +635,23 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 		if (effect_obvious(effect)) object_flavor_aware(o_ptr);
 
 		/* Boost damage effects if skill > difficulty */
-		boost = p_ptr->state.skills[SKILL_DEVICE] - level;
-		if (boost < 0) boost = 0;
+		boost = MAX(p_ptr->state.skills[SKILL_DEVICE] - level, 0);
 
 		/* Do effect */
-		used = effect_do(effect, &ident, was_aware, dir,
-			beam_chance(o_ptr->tval), boost);
+		used = effect_do(effect, &ident, was_aware, dir, beam, boost);
 
 		/* Quit if the item wasn't used and no knowledge was gained */
 		if (!used && (was_aware || !ident)) return;
 	}
 
 	/* If the item is a null pointer or has been wiped, be done now */
-	if (!o_ptr || o_ptr->k_idx <= 1) return;
+	if (!o_ptr || !o_ptr->kind) return;
 
 	if (ident) object_notice_effect(o_ptr);
 
 	/* Food feeds the player */
 	if (o_ptr->tval == TV_FOOD || o_ptr->tval == TV_POTION)
-		(void)set_food(p_ptr->food + o_ptr->pval);
+		player_set_food(p_ptr, p_ptr->food + o_ptr->pval[DEFAULT_PVAL]);
 
 	/* Use the turn */
 	p_ptr->energy_use = 100;
@@ -580,11 +668,11 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	if (ident && !was_aware)
 	{
 		/* Object level */
-		int lev = k_info[o_ptr->k_idx].level;
+		int lev = o_ptr->kind->level;
 
 		object_flavor_aware(o_ptr);
 		if (o_ptr->tval == TV_ROD) object_notice_everything(o_ptr);
-		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
+		player_exp_gain(p_ptr, (lev + (p_ptr->lev / 2)) / p_ptr->lev);
 		p_ptr->notice |= PN_SQUELCH;
 	}
 	else if (used)
@@ -599,7 +687,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	if (used && use == USE_CHARGE)
 	{
 		/* Use a single charge */
-		o_ptr->pval--;
+		o_ptr->pval[DEFAULT_PVAL]--;
 
 		/* Describe charges */
 		if (item >= 0)
@@ -610,16 +698,10 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	else if (used && use == USE_TIMEOUT)
 	{
 		/* Artifacts use their own special field */
-		if (o_ptr->name1)
-		{
-			const artifact_type *a_ptr = &a_info[o_ptr->name1];
-			o_ptr->timeout = randcalc(a_ptr->time, 0, RANDOMISE);
-		}
+		if (o_ptr->artifact)
+			o_ptr->timeout = randcalc(o_ptr->artifact->time, 0, RANDOMISE);
 		else
-		{
-			const object_kind *k_ptr = &k_info[o_ptr->k_idx];
-			o_ptr->timeout += randcalc(k_ptr->time, 0, RANDOMISE);
-		}
+			o_ptr->timeout += randcalc(o_ptr->kind->time, 0, RANDOMISE);
 	}
 	else if (used && use == USE_SINGLE)
 	{
@@ -641,17 +723,10 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	}
 	
 	/* Hack to make Glyph of Warding work properly */
-	if (cave_feat[py][px] == FEAT_GLYPH)
+	if (cave->feat[py][px] == FEAT_GLYPH)
 	{
-		/* Shift any objects to further away */
-		for (o_ptr = get_first_object(py, px); o_ptr; o_ptr =
-			get_next_object(o_ptr))
-		{
-			drop_near(o_ptr, 0, py, px, FALSE);
-		}
-
-		/* Delete the "moved" objects from their original position */
-		delete_object(py, px);
+		/* Push objects off the grid */
+		if (cave->o_idx[py][px]) push_object(py, px);
 	}
 
 
@@ -659,6 +734,156 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 
 
 /*** Refuelling ***/
+
+static void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item)
+{
+	/* Refuel */
+	j_ptr->timeout += o_ptr->timeout ? o_ptr->timeout : o_ptr->pval[DEFAULT_PVAL];
+
+	/* Message */
+	msg("You fuel your lamp.");
+
+	/* Comment */
+	if (j_ptr->timeout >= FUEL_LAMP)
+	{
+		j_ptr->timeout = FUEL_LAMP;
+		msg("Your lamp is full.");
+	}
+
+	/* Refilled from a lantern */
+	if (o_ptr->sval == SV_LIGHT_LANTERN)
+	{
+		/* Unstack if necessary */
+		if (o_ptr->number > 1)
+		{
+			object_type *i_ptr;
+			object_type object_type_body;
+
+			/* Get local object */
+			i_ptr = &object_type_body;
+
+			/* Obtain a local object */
+			object_copy(i_ptr, o_ptr);
+
+			/* Modify quantity */
+			i_ptr->number = 1;
+
+			/* Remove fuel */
+			i_ptr->timeout = 0;
+
+			/* Unstack the used item */
+			o_ptr->number--;
+			p_ptr->total_weight -= i_ptr->weight;
+
+			/* Carry or drop */
+			if (item >= 0)
+				item = inven_carry(p_ptr, i_ptr);
+			else
+				drop_near(cave, i_ptr, 0, p_ptr->py, p_ptr->px, FALSE);
+		}
+
+		/* Empty a single lantern */
+		else
+		{
+			/* No more fuel */
+			o_ptr->timeout = 0;
+		}
+
+		/* Combine / Reorder the pack (later) */
+		p_ptr->notice |= (PN_COMBINE | PN_REORDER);
+
+		/* Redraw stuff */
+		p_ptr->redraw |= (PR_INVEN);
+	}
+
+	/* Refilled from a flask */
+	else
+	{
+		/* Decrease the item (from the pack) */
+		if (item >= 0)
+		{
+			inven_item_increase(item, -1);
+			inven_item_describe(item);
+			inven_item_optimize(item);
+		}
+
+		/* Decrease the item (from the floor) */
+		else
+		{
+			floor_item_increase(0 - item, -1);
+			floor_item_describe(0 - item);
+			floor_item_optimize(0 - item);
+		}
+	}
+
+	/* Recalculate torch */
+	p_ptr->update |= (PU_TORCH);
+
+	/* Redraw stuff */
+	p_ptr->redraw |= (PR_EQUIP);
+}
+
+
+static void refuel_torch(object_type *j_ptr, object_type *o_ptr, int item)
+{
+	bitflag f[OF_SIZE];
+	bitflag g[OF_SIZE];
+
+	/* Refuel */
+	j_ptr->timeout += o_ptr->timeout + 5;
+
+	/* Message */
+	msg("You combine the torches.");
+
+	/* Transfer the LIGHT flag if refuelling from a torch with it to one
+	   without it */
+	object_flags(o_ptr, f);
+	object_flags(j_ptr, g);
+	if (of_has(f, OF_LIGHT) && !of_has(g, OF_LIGHT))
+	{
+		of_on(j_ptr->flags, OF_LIGHT);
+		if (!j_ptr->ego && o_ptr->ego)
+			j_ptr->ego = o_ptr->ego;
+		msg("Your torch shines further!");
+	}
+
+	/* Over-fuel message */
+	if (j_ptr->timeout >= FUEL_TORCH)
+	{
+		j_ptr->timeout = FUEL_TORCH;
+		msg("Your torch is fully fueled.");
+	}
+
+	/* Refuel message */
+	else
+	{
+		msg("Your torch glows more brightly.");
+	}
+
+	/* Decrease the item (from the pack) */
+	if (item >= 0)
+	{
+		inven_item_increase(item, -1);
+		inven_item_describe(item);
+		inven_item_optimize(item);
+	}
+
+	/* Decrease the item (from the floor) */
+	else
+	{
+		floor_item_increase(0 - item, -1);
+		floor_item_describe(0 - item);
+		floor_item_optimize(0 - item);
+	}
+
+	/* Recalculate torch */
+	p_ptr->update |= (PU_TORCH);
+
+	/* Redraw stuff */
+	p_ptr->redraw |= (PR_EQUIP);
+}
+
+
 void do_cmd_refill(cmd_code code, cmd_arg args[])
 {
 	object_type *j_ptr = &p_ptr->inventory[INVEN_LIGHT];
@@ -669,7 +894,7 @@ void do_cmd_refill(cmd_code code, cmd_arg args[])
 
 	if (!item_is_available(item, NULL, USE_INVEN | USE_FLOOR))
 	{
-		msg_print("You do not have that item to refill with it.");
+		msg("You do not have that item to refill with it.");
 		return;
 	}
 
@@ -678,13 +903,13 @@ void do_cmd_refill(cmd_code code, cmd_arg args[])
 
 	if (j_ptr->tval != TV_LIGHT)
 	{
-		msg_print("You are not wielding a light.");
+		msg("You are not wielding a light.");
 		return;
 	}
 
 	else if (of_has(f, OF_NO_FUEL))
 	{
-		msg_print("Your light cannot be refilled.");
+		msg("Your light cannot be refilled.");
 		return;
 	}
 
@@ -734,7 +959,7 @@ void do_cmd_study_spell(cmd_code code, cmd_arg args[])
 			else
 			{
 				/* Spell is present, but player incapable. */
-				msg_format("You cannot learn that spell.");
+				msg("You cannot learn that spell.");
 			}
 
 			return;
@@ -752,8 +977,8 @@ void do_cmd_cast(cmd_code code, cmd_arg args[])
 	int item_num;
 	int i;
 
-	cptr verb = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
-	cptr noun = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+	const char *verb = ((p_ptr->class->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
+	const char *noun = ((p_ptr->class->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
 
 	/* Check the player can cast spells at all */
 	if (!player_can_cast())
@@ -771,13 +996,13 @@ void do_cmd_cast(cmd_code code, cmd_arg args[])
 			if (spell_okay_to_cast(spell))
 			{
 				/* Get the spell */
-				const magic_type *s_ptr = &mp_ptr->info[spell];	
+				const magic_type *s_ptr = &p_ptr->class->spells.info[spell];
 				
 				/* Verify "dangerous" spells */
 				if (s_ptr->smana > p_ptr->csp)
 				{
 					/* Warning */
-					msg_format("You do not have enough mana to %s this %s.", verb, noun);
+					msg("You do not have enough mana to %s this %s.", verb, noun);
 					
 					/* Flush input */
 					flush();
@@ -793,7 +1018,7 @@ void do_cmd_cast(cmd_code code, cmd_arg args[])
 			else
 			{
 				/* Spell is present, but player incapable. */
-				msg_format("You cannot %s that %s.", verb, noun);
+				msg("You cannot %s that %s.", verb, noun);
 			}
 
 			return;
@@ -810,9 +1035,10 @@ void do_cmd_study_book(cmd_code code, cmd_arg args[])
 	object_type *o_ptr = object_from_item_idx(book);
 
 	int spell = -1;
-	int i, k = 0;
+	struct spell *sp;
+	int k = 0;
 
-	cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+	const char *p = ((p_ptr->class->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
 
 	/* Check the player can study at all atm */
 	if (!player_can_study())
@@ -821,29 +1047,22 @@ void do_cmd_study_book(cmd_code code, cmd_arg args[])
 	/* Check that the player has access to the nominated spell book. */
 	if (!item_is_available(book, obj_can_browse, (USE_INVEN | USE_FLOOR)))
 	{
-		msg_format("That item is not within your reach.");
+		msg("That item is not within your reach.");
 		return;
 	}
 
 	/* Extract spells */
-	for (i = 0; i < SPELLS_PER_BOOK; i++)
-	{
-		int s = get_spell_index(o_ptr, i);
-		
-		/* Skip non-OK spells */
-		if (s == -1) continue;
-		if (!spell_okay_to_study(s)) continue;
-		
-		/* Apply the randomizer */
-		if ((++k > 1) && (randint0(k) != 0)) continue;
-		
-		/* Track it */
-		spell = s;
+	for (sp = o_ptr->kind->spells; sp; sp = sp->next) {
+		if (!spell_okay_to_study(sp->spell_index))
+			continue;
+		if ((++k > 1) && (randint0(k) != 0))
+			continue;
+		spell = sp->spell_index;
 	}
 
 	if (spell < 0)
 	{
-		msg_format("You cannot learn any %ss in that book.", p);
+		msg("You cannot learn any %ss in that book.", p);
 	}
 	else
 	{
