@@ -25,8 +25,12 @@
 #include "generate.h"
 #include "init.h"
 #include "monster/monster.h"
+#include "monster/mon-make.h"
+#include "monster/mon-spell.h"
+#include "monster/mon-util.h"
 #include "object/tvalsval.h"
 #include "prefs.h"
+#include "savefile.h"
 #include "spells.h"
 #include "target.h"
 
@@ -42,7 +46,7 @@ void dungeon_change_level(int dlev)
 	   according to how long we've been away */
 	if (!dlev && daycount)
 	{
-		if (OPT(cheat_xtra)) msg_print("Updating Shops...");
+		if (OPT(cheat_xtra)) msg("Updating Shops...");
 		while (daycount--)
 		{
 			int n;
@@ -54,14 +58,14 @@ void dungeon_change_level(int dlev)
 				if (n == STORE_HOME) continue;
 
 				/* Maintain */
-				store_maint(n);
+				store_maint(&stores[n]);
 			}
 
 			/* Sometimes, shuffle the shop-keepers */
 			if (one_in_(STORE_SHUFFLE))
 			{
 				/* Message */
-				if (OPT(cheat_xtra)) msg_print("Shuffling a Shopkeeper...");
+				if (OPT(cheat_xtra)) msg("Shuffling a Shopkeeper...");
 
 				/* Pick a random shop (except home) */
 				while (1)
@@ -71,11 +75,11 @@ void dungeon_change_level(int dlev)
 				}
 
 				/* Shuffle it */
-				store_shuffle(n);
+				store_shuffle(&stores[n]);
 			}
 		}
 		daycount = 0;
-		if (OPT(cheat_xtra)) msg_print("Done.");
+		if (OPT(cheat_xtra)) msg("Done.");
 	}
 
 	/* Leaving */
@@ -126,8 +130,8 @@ static void regenhp(int percent)
 	{
 		/* Redraw */
 		p_ptr->redraw |= (PR_HP);
-		wieldeds_notice_flag(OF_REGEN);
-		wieldeds_notice_flag(OF_IMPAIR_HP);
+		wieldeds_notice_flag(p_ptr, OF_REGEN);
+		wieldeds_notice_flag(p_ptr, OF_IMPAIR_HP);
 	}
 }
 
@@ -171,8 +175,8 @@ static void regenmana(int percent)
 	{
 		/* Redraw */
 		p_ptr->redraw |= (PR_MANA);
-		wieldeds_notice_flag(OF_REGEN);
-		wieldeds_notice_flag(OF_IMPAIR_MANA);
+		wieldeds_notice_flag(p_ptr, OF_REGEN);
+		wieldeds_notice_flag(p_ptr, OF_IMPAIR_MANA);
 	}
 }
 
@@ -191,10 +195,10 @@ static void regen_monsters(void)
 	int i, frac;
 
 	/* Regenerate everyone */
-	for (i = 1; i < mon_max; i++)
+	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		/* Check the i'th monster */
-		monster_type *m_ptr = &mon_list[i];
+		monster_type *m_ptr = cave_monster(cave, i);
 		monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 		/* Skip dead monsters */
@@ -235,7 +239,7 @@ static void recharged_notice(const object_type *o_ptr, bool all)
 {
 	char o_name[120];
 
-	cptr s;
+	const char *s;
 
 	bool notify = FALSE;
 
@@ -270,23 +274,23 @@ static void recharged_notice(const object_type *o_ptr, bool all)
 	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_BASE);
 
 	/* Disturb the player */
-	disturb(0, 0);
+	disturb(p_ptr, 0, 0);
 
 	/* Notify the player */
 	if (o_ptr->number > 1)
 	{
-		if (all) msg_format("Your %s have recharged.", o_name);
-		else msg_format("One of your %s has recharged.", o_name);
+		if (all) msg("Your %s have recharged.", o_name);
+		else msg("One of your %s has recharged.", o_name);
 	}
 
 	/* Artifacts */
-	else if (o_ptr->name1)
+	else if (o_ptr->artifact)
 	{
-		msg_format("The %s has recharged.", o_name);
+		msg("The %s has recharged.", o_name);
 	}
 
 	/* Single, non-artifact items */
-	else msg_format("Your %s has recharged.", o_name);
+	else msg("Your %s has recharged.", o_name);
 }
 
 
@@ -301,7 +305,6 @@ static void recharge_objects(void)
 	bool charged = FALSE, discharged_stack;
 
 	object_type *o_ptr;
-	object_kind *k_ptr;
 
 	/*** Recharge equipment ***/
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
@@ -310,7 +313,7 @@ static void recharge_objects(void)
 		o_ptr = &p_ptr->inventory[i];
 
 		/* Skip non-objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 
 		/* Recharge activatable objects */
 		if (recharge_timeout(o_ptr))
@@ -335,10 +338,9 @@ static void recharge_objects(void)
 	for (i = 0; i < INVEN_PACK; i++)
 	{
 		o_ptr = &p_ptr->inventory[i];
-		k_ptr = &k_info[o_ptr->k_idx];
 
 		/* Skip non-objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 
 		discharged_stack = (number_charging(o_ptr) == o_ptr->number) ? TRUE : FALSE;
 
@@ -349,15 +351,11 @@ static void recharge_objects(void)
 
 			/* Entire stack is recharged */
 			if (o_ptr->timeout == 0)
-			{
 				recharged_notice(o_ptr, TRUE);
-			}
 
 			/* Previously exhausted stack has acquired a charge */
 			else if (discharged_stack)
-			{
 				recharged_notice(o_ptr, FALSE);
-			}
 		}
 	}
 
@@ -375,10 +373,10 @@ static void recharge_objects(void)
 	for (i = 1; i < o_max; i++)
 	{
 		/* Get the object */
-		o_ptr = &o_list[i];
+		o_ptr = object_byid(i);
 
 		/* Skip dead objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 
 		/* Recharge rods on the ground */
 		if (o_ptr->tval == TV_ROD)
@@ -469,7 +467,7 @@ static void decrease_timeouts(void)
 			}
 		}
 		/* Decrement the effect */
-		dec_timed(i, decr, FALSE);
+		player_dec_timed(p_ptr, i, decr, FALSE);
 	}
 
 	return;
@@ -479,7 +477,7 @@ static void decrease_timeouts(void)
 /*
  * Handle certain things once every 10 game turns
  */
-static void process_world(void)
+static void process_world(struct cave *c)
 {
 	int i;
 
@@ -514,14 +512,14 @@ static void process_world(void)
 
 			/* Day breaks */
 			if (dawn)
-				msg_print("The sun has risen.");
+				msg("The sun has risen.");
 
 			/* Night falls */
 			else
-				msg_print("The sun has fallen.");
+				msg("The sun has fallen.");
 
 			/* Illuminate */
-			town_illuminate(dawn);
+			cave_illuminate(c, dawn);
 		}
 	}
 
@@ -542,7 +540,7 @@ static void process_world(void)
 	if (one_in_(MAX_M_ALLOC_CHANCE))
 	{
 		/* Make a new monster */
-		(void)alloc_monster(MAX_SIGHT + 5, FALSE, p_ptr->depth);
+		(void)pick_and_place_distant_monster(cave, loc(p_ptr->px, p_ptr->py), MAX_SIGHT + 5, FALSE, p_ptr->depth);
 	}
 
 	/* Hack -- Check for creature regeneration */
@@ -555,7 +553,7 @@ static void process_world(void)
 	if (p_ptr->timed[TMD_POISONED])
 	{
 		/* Take damage */
-		take_hit(1, "poison");
+		take_hit(p_ptr, 1, "poison");
 	}
 
 	/* Take damage from cuts */
@@ -574,7 +572,7 @@ static void process_world(void)
 			i = 1;
 
 		/* Take damage */
-		take_hit(i, "a fatal wound");
+		take_hit(p_ptr, i, "a fatal wound");
 	}
 
 
@@ -590,16 +588,16 @@ static void process_world(void)
 			i = extract_energy[p_ptr->state.speed] * 2;
 
 			/* Regeneration takes more food */
-			if (p_ptr->state.regenerate) i += 30;
+			if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags)) i += 30;
 
 			/* Slow digestion takes less food */
-			if (p_ptr->state.slow_digest) i -= 10;
+			if (check_state(p_ptr, OF_SLOW_DIGEST, p_ptr->state.flags)) i -= 10;
 
 			/* Minimal digestion */
 			if (i < 1) i = 1;
 
 			/* Digest some food */
-			(void)set_food(p_ptr->food - i);
+			player_set_food(p_ptr, p_ptr->food - i);
 		}
 	}
 
@@ -607,7 +605,7 @@ static void process_world(void)
 	else
 	{
 		/* Digest a lot of food */
-		(void)set_food(p_ptr->food - 100);
+		player_set_food(p_ptr, p_ptr->food - 100);
 	}
 
 	/* Getting Faint */
@@ -617,11 +615,11 @@ static void process_world(void)
 		if (!p_ptr->timed[TMD_PARALYZED] && one_in_(10))
 		{
 			/* Message */
-			msg_print("You faint from the lack of food.");
-			disturb(1, 0);
+			msg("You faint from the lack of food.");
+			disturb(p_ptr, 1, 0);
 
-			/* Hack -- faint (bypass free action) */
-			(void)inc_timed(TMD_PARALYZED, 1 + randint0(5), TRUE);
+			/* Faint (bypass free action) */
+			(void)player_inc_timed(p_ptr, TMD_PARALYZED, 1 + randint0(5), TRUE, FALSE);
 		}
 	}
 
@@ -633,7 +631,7 @@ static void process_world(void)
 		i = (PY_FOOD_STARVE - p_ptr->food) / 10;
 
 		/* Take damage */
-		take_hit(i, "starvation");
+		take_hit(p_ptr, i, "starvation");
 	}
 
 	/** Regenerate HP **/
@@ -649,13 +647,13 @@ static void process_world(void)
 		regen_amount = PY_REGEN_WEAK;
 
 	/* Various things speed up regeneration */
-	if (p_ptr->state.regenerate)
+	if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags))
 		regen_amount *= 2;
 	if (p_ptr->searching || p_ptr->resting)
 		regen_amount *= 2;
 
 	/* Some things slow it down */
-	if (p_ptr->state.impair_hp)
+	if (check_state(p_ptr, OF_IMPAIR_HP, p_ptr->state.flags))
 		regen_amount /= 2;
 
 	/* Various things interfere with physical healing */
@@ -675,13 +673,13 @@ static void process_world(void)
 	regen_amount = PY_REGEN_NORMAL;
 
 	/* Various things speed up regeneration */
-	if (p_ptr->state.regenerate)
+	if (check_state(p_ptr, OF_REGEN, p_ptr->state.flags))
 		regen_amount *= 2;
 	if (p_ptr->searching || p_ptr->resting)
 		regen_amount *= 2;
 
 	/* Some things slow it down */
-	if (p_ptr->state.impair_mana)
+	if (check_state(p_ptr, OF_IMPAIR_MANA, p_ptr->state.flags))
 		regen_amount /= 2;
 
 	/* Regenerate mana */
@@ -741,15 +739,15 @@ static void process_world(void)
 			/* The light is now out */
 			else if (o_ptr->timeout == 0)
 			{
-				disturb(0, 0);
-				msg_print("Your light has gone out!");
+				disturb(p_ptr, 0, 0);
+				msg("Your light has gone out!");
 			}
 
 			/* The light is getting dim */
 			else if ((o_ptr->timeout < 100) && (!(o_ptr->timeout % 10)))
 			{
-				disturb(0, 0);
-				msg_print("Your light is growing faint.");
+				disturb(p_ptr, 0, 0);
+				msg("Your light is growing faint.");
 			}
 		}
 	}
@@ -761,16 +759,12 @@ static void process_world(void)
 	/*** Process Inventory ***/
 
 	/* Handle experience draining */
-	if (p_ptr->state.exp_drain)
+	if (check_state(p_ptr, OF_DRAIN_EXP, p_ptr->state.flags))
 	{
 		if ((p_ptr->exp > 0) && one_in_(10))
-		{
-			p_ptr->exp--;
-			p_ptr->max_exp--;
-			check_experience();
-		}
+			player_exp_lose(p_ptr, 1, FALSE);
 
-		wieldeds_notice_flag(OF_DRAIN_EXP);
+		wieldeds_notice_flag(p_ptr, OF_DRAIN_EXP);
 	}
 
 	/* Recharge activatable objects and rods */
@@ -783,11 +777,11 @@ static void process_world(void)
 	/*** Involuntary Movement ***/
 
 	/* Random teleportation */
-	if (p_ptr->state.teleport && one_in_(100))
+	if (check_state(p_ptr, OF_TELEPORT, p_ptr->state.flags) && one_in_(100))
 	{
-		wieldeds_notice_flag(OF_TELEPORT);
+		wieldeds_notice_flag(p_ptr, OF_TELEPORT);
 		teleport_player(40);
-		disturb(0, 0);
+		disturb(p_ptr, 0, 0);
 	}
 
 	/* Delayed Word-of-Recall */
@@ -800,17 +794,17 @@ static void process_world(void)
 		if (!p_ptr->word_recall)
 		{
 			/* Disturbing! */
-			disturb(0, 0);
+			disturb(p_ptr, 0, 0);
 
 			/* Determine the level */
 			if (p_ptr->depth)
 			{
-				message_format(MSG_TPLEVEL, 0, "You feel yourself yanked upwards!");
+				msgt(MSG_TPLEVEL, "You feel yourself yanked upwards!");
 				dungeon_change_level(0);
 			}
 			else
 			{
-				message_format(MSG_TPLEVEL, 0, "You feel yourself yanked downwards!");
+				msgt(MSG_TPLEVEL, "You feel yourself yanked downwards!");
 
 				/* New depth - back to max depth or 1, whichever is deeper */
 				dungeon_change_level(p_ptr->max_depth < 1 ? 1: p_ptr->max_depth);
@@ -881,7 +875,7 @@ static void process_player_aux(void)
 
 			/* Redraw stuff */
 			p_ptr->redraw |= (PR_MONSTER);
-			redraw_stuff();
+			redraw_stuff(p_ptr);
 		}
 	}
 }
@@ -921,7 +915,7 @@ static void process_player(void)
 			if ((p_ptr->chp == p_ptr->mhp) &&
 			    (p_ptr->csp == p_ptr->msp))
 			{
-				disturb(0, 0);
+				disturb(p_ptr, 0, 0);
 			}
 		}
 
@@ -938,7 +932,7 @@ static void process_player(void)
 			    !p_ptr->timed[TMD_SLOW] && !p_ptr->timed[TMD_PARALYZED] &&
 			    !p_ptr->timed[TMD_IMAGE] && !p_ptr->word_recall)
 			{
-				disturb(0, 0);
+				disturb(p_ptr, 0, 0);
 			}
 		}
 		
@@ -949,7 +943,7 @@ static void process_player(void)
 			if ((p_ptr->chp == p_ptr->mhp) ||
 			    (p_ptr->csp == p_ptr->msp))
 			{
-				disturb(0, 0);
+				disturb(p_ptr, 0, 0);
 			}
 		}
 	}
@@ -959,20 +953,18 @@ static void process_player(void)
 	    cmd_get_nrepeats() > 0 ||
 	    (p_ptr->resting && !(turn & 0x7F)))
 	{
+		ui_event e;
+
 		/* Do not wait */
 		inkey_scan = SCAN_INSTANT;
 
 		/* Check for a key */
-		if (inkey())
-		{
-			/* Flush input */
+		e = inkey_ex();
+		if (e.type != EVT_NONE) {
+			/* Flush and disturb */
 			flush();
-
-			/* Disturb */
-			disturb(0, 0);
-
-			/* Hack -- Show a Message */
-			msg_print("Cancelled.");
+			disturb(p_ptr, 0, 0);
+			msg("Cancelled.");
 		}
 	}
 
@@ -983,13 +975,13 @@ static void process_player(void)
 	do
 	{
 		/* Notice stuff (if needed) */
-		if (p_ptr->notice) notice_stuff();
+		if (p_ptr->notice) notice_stuff(p_ptr);
 
 		/* Update stuff (if needed) */
-		if (p_ptr->update) update_stuff();
+		if (p_ptr->update) update_stuff(p_ptr);
 
 		/* Redraw stuff (if needed) */
-		if (p_ptr->redraw) redraw_stuff();
+		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 
 		/* Place the cursor on the player */
@@ -1028,11 +1020,13 @@ static void process_player(void)
 		/* Picking up objects */
 		else if (p_ptr->notice & PN_PICKUP)
 		{
-			/* Recursively call the pickup function, use energy */
-			p_ptr->energy_use = py_pickup(0) * 10;
+			p_ptr->energy_use = do_autopickup() * 10;
 			if (p_ptr->energy_use > 100)
 				p_ptr->energy_use = 100;
 			p_ptr->notice &= ~(PN_PICKUP);
+			
+			/* Appropriate time for the player to see objects */
+			event_signal(EVENT_SEEFLOOR);
 		}
 
 		/* Resting */
@@ -1110,127 +1104,49 @@ static void process_player(void)
 				p_ptr->redraw |= (PR_MAP);
 			}
 
-			/* Shimmer monsters if needed */
-			if (shimmer_monsters)
+			/* Shimmer multi-hued monsters */
+			for (i = 1; i < cave_monster_max(cave); i++)
 			{
-				/* Clear the flag */
-				shimmer_monsters = FALSE;
-
-				/* Shimmer multi-hued monsters */
-				for (i = 1; i < mon_max; i++)
-				{
-					monster_type *m_ptr;
-					monster_race *r_ptr;
-
-					/* Get the monster */
-					m_ptr = &mon_list[i];
-
-					/* Skip dead monsters */
-					if (!m_ptr->r_idx) continue;
-
-					/* Get the monster race */
-					r_ptr = &r_info[m_ptr->r_idx];
-
-					/* Skip non-multi-hued monsters */
-					if (!rf_has(r_ptr->flags, RF_ATTR_MULTI)) continue;
-
-					/* Reset the flag */
-					shimmer_monsters = TRUE;
-
-					/* Redraw regardless */
-					light_spot(m_ptr->fy, m_ptr->fx);
-				}
+				struct monster_race *race;
+				struct monster *mon = cave_monster(cave, i);
+				if (!mon->r_idx)
+					continue;
+				race = &r_info[mon->r_idx];
+				if (!rf_has(race->flags, RF_ATTR_MULTI))
+					continue;
+				cave_light_spot(cave, mon->fy, mon->fx);
 			}
 
-			/* Repair "nice" flags */
-			if (repair_mflag_nice)
+			/* Clear NICE flag, and show marked monsters */
+			for (i = 1; i < cave_monster_max(cave); i++)
 			{
-				/* Clear flag */
-				repair_mflag_nice = FALSE;
-
-				/* Process monsters */
-				for (i = 1; i < mon_max; i++)
-				{
-					monster_type *m_ptr;
-
-					/* Get the monster */
-					m_ptr = &mon_list[i];
-
-					/* Skip dead monsters */
-					/* if (!m_ptr->r_idx) continue; */
-
-					/* Clear "nice" flag */
-					m_ptr->mflag &= ~(MFLAG_NICE);
-				}
-			}
-
-			/* Repair "mark" flags */
-			if (repair_mflag_mark)
-			{
-				/* Reset the flag */
-				repair_mflag_mark = FALSE;
-
-				/* Process the monsters */
-				for (i = 1; i < mon_max; i++)
-				{
-					monster_type *m_ptr;
-
-					/* Get the monster */
-					m_ptr = &mon_list[i];
-
-					/* Skip dead monsters */
-					/* if (!m_ptr->r_idx) continue; */
-
-					/* Repair "mark" flag */
-					if (m_ptr->mflag & (MFLAG_MARK))
-					{
-						/* Skip "show" monsters */
-						if (m_ptr->mflag & (MFLAG_SHOW))
-						{
-							/* Repair "mark" flag */
-							repair_mflag_mark = TRUE;
-
-							/* Skip */
-							continue;
-						}
-
-						/* Forget flag */
-						m_ptr->mflag &= ~(MFLAG_MARK);
-
-						/* Update the monster */
+				struct monster *mon = cave_monster(cave, i);
+				mon->mflag &= ~MFLAG_NICE;
+				if (mon->mflag & MFLAG_MARK) {
+					if (!(mon->mflag & MFLAG_SHOW)) {
+						mon->mflag &= ~MFLAG_MARK;
 						update_mon(i, FALSE);
 					}
 				}
 			}
 		}
 
-		/* Repair "show" flags */
-		if (repair_mflag_show)
+		/* Clear SHOW flag */
+		for (i = 1; i < cave_monster_max(cave); i++)
 		{
-			/* Reset the flag */
-			repair_mflag_show = FALSE;
-
-			/* Process the monsters */
-			for (i = 1; i < mon_max; i++)
-			{
-				monster_type *m_ptr;
-
-				/* Get the monster */
-				m_ptr = &mon_list[i];
-
-				/* Skip dead monsters */
-				/* if (!m_ptr->r_idx) continue; */
-
-				/* Clear "show" flag */
-				m_ptr->mflag &= ~(MFLAG_SHOW);
-			}
+			struct monster *mon = cave_monster(cave, i);
+			mon->mflag &= ~MFLAG_SHOW;
 		}
+
 		/* HACK: This will redraw the itemlist too frequently, but I'm don't
 		   know all the individual places it should go. */
 		p_ptr->redraw |= PR_ITEMLIST;
 	}
 
 	while (!p_ptr->energy_use && !p_ptr->leaving);
+
+	/* Notice stuff (if needed) */
+	if (p_ptr->notice) notice_stuff(p_ptr);
 }
 
 byte flicker = 0;
@@ -1266,7 +1182,7 @@ byte color_flicker[MAX_COLORS][3] =
 	{TERM_DEEP_L_BLUE, TERM_L_BLUE, TERM_BLUE},
 };
 
-byte get_flicker(byte a)
+static byte get_flicker(byte a)
 {
 	switch(flicker % 3)
 	{
@@ -1279,14 +1195,14 @@ byte get_flicker(byte a)
 /*
  * This animates monsters and/or items as necessary.
  */
-void do_animation(void)
+static void do_animation(void)
 {
 	int i;
 
-	for (i = 1; i < mon_max; i++)
+	for (i = 1; i < cave_monster_max(cave); i++)
 	{
 		byte attr;
-		monster_type *m_ptr = &mon_list[i];
+		monster_type *m_ptr = cave_monster(cave, i);
 		monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 		if (!m_ptr || !m_ptr->ml)
@@ -1317,7 +1233,7 @@ void idle_update(void)
 
 	/* Animate and redraw if necessary */
 	do_animation();
-	redraw_stuff();
+	redraw_stuff(p_ptr);
 
 	/* Refresh the main screen */
 	Term_fresh();
@@ -1330,7 +1246,7 @@ void idle_update(void)
  * This function will not exit until the level is completed,
  * the user dies, or the game is terminated.
  */
-static void dungeon(void)
+static void dungeon(struct cave *c)
 {
 	monster_type *m_ptr;
 	int i;
@@ -1354,21 +1270,10 @@ static void dungeon(void)
 	target_set_monster(0);
 
 	/* Cancel the health bar */
-	health_track(0);
-
-
-	/* Reset shimmer flags */
-	shimmer_monsters = TRUE;
-	shimmer_objects = TRUE;
-
-	/* Reset repair flags */
-	repair_mflag_nice = TRUE;
-	repair_mflag_show = TRUE;
-	repair_mflag_mark = TRUE;
-
+	health_track(p_ptr, 0);
 
 	/* Disturb */
-	disturb(1, 0);
+	disturb(p_ptr, 1, 0);
 
 
 	/* Track maximum player level */
@@ -1387,11 +1292,13 @@ static void dungeon(void)
 	/* If autosave is pending, do it now. */
 	if (p_ptr->autosave)
 	{
+/* The borg runs so quickly that this is a bad idea. */
+#ifndef ALLOW_BORG 
 		save_game();
+#endif
 		p_ptr->autosave = FALSE;
 	}
 
-
 	/* Choose panel */
 	verify_panel();
 
@@ -1415,7 +1322,7 @@ static void dungeon(void)
 	p_ptr->update |= (PU_TORCH);
 
 	/* Update stuff */
-	update_stuff();
+	update_stuff(p_ptr);
 
 
 	/* Fully update the visuals (and monster distances) */
@@ -1431,10 +1338,10 @@ static void dungeon(void)
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP | PR_MONSTER | PR_MONLIST | PR_ITEMLIST);
 
 	/* Update stuff */
-	update_stuff();
+	update_stuff(p_ptr);
 
 	/* Redraw stuff */
-	redraw_stuff();
+	redraw_stuff(p_ptr);
 
 
 	/* Hack -- Decrease "xtra" depth */
@@ -1458,13 +1365,13 @@ static void dungeon(void)
 	p_ptr->redraw |= (PR_BUTTONS);
 
 	/* Notice stuff */
-	notice_stuff();
+	notice_stuff(p_ptr);
 
 	/* Update stuff */
-	update_stuff();
+	update_stuff(p_ptr);
 
 	/* Redraw stuff */
-	redraw_stuff();
+	redraw_stuff(p_ptr);
 
 	/* Refresh */
 	Term_fresh();
@@ -1473,7 +1380,7 @@ static void dungeon(void)
 	if (p_ptr->is_dead) return;
 
 	/* Announce (or repeat) the feeling */
-	if (p_ptr->depth) do_cmd_feeling();
+	if (p_ptr->depth) display_feeling(FALSE);
 
 	/* Give player minimum energy to start a new level, but do not reduce higher value from savefile for level in progress */
 	if (p_ptr->energy < INITIAL_DUNGEON_ENERGY)
@@ -1486,11 +1393,10 @@ static void dungeon(void)
 	while (TRUE)
 	{
 		/* Hack -- Compact the monster list occasionally */
-		if (mon_cnt + 32 > z_info->m_max) compact_monsters(64);
+		if (cave_monster_count(cave) + 32 > z_info->m_max) compact_monsters(64);
 
 		/* Hack -- Compress the monster list occasionally */
-		if (mon_cnt + 32 < mon_max) compact_monsters(0);
-
+		if (cave_monster_count(cave) + 32 < cave_monster_max(cave)) compact_monsters(0);
 
 		/* Hack -- Compact the object list occasionally */
 		if (o_cnt + 32 > z_info->o_max) compact_objects(64);
@@ -1502,10 +1408,10 @@ static void dungeon(void)
 		while ((p_ptr->energy >= 100) && !p_ptr->leaving)
 		{
     		/* Do any necessary animations */
-    		do_animation(); 
+    		do_animation();
 
 			/* process monster with even more energy first */
-			process_monsters((byte)(p_ptr->energy + 1));
+			process_monsters(c, (byte)(p_ptr->energy + 1));
 
 			/* if still alive */
 			if (!p_ptr->leaving)
@@ -1520,13 +1426,13 @@ static void dungeon(void)
 		}
 
 		/* Notice stuff */
-		if (p_ptr->notice) notice_stuff();
+		if (p_ptr->notice) notice_stuff(p_ptr);
 
 		/* Update stuff */
-		if (p_ptr->update) update_stuff();
+		if (p_ptr->update) update_stuff(p_ptr);
 
 		/* Redraw stuff */
-		if (p_ptr->redraw) redraw_stuff();
+		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
@@ -1536,16 +1442,16 @@ static void dungeon(void)
 
 
 		/* Process all of the monsters */
-		process_monsters(100);
+		process_monsters(c, 100);
 
 		/* Notice stuff */
-		if (p_ptr->notice) notice_stuff();
+		if (p_ptr->notice) notice_stuff(p_ptr);
 
 		/* Update stuff */
-		if (p_ptr->update) update_stuff();
+		if (p_ptr->update) update_stuff(p_ptr);
 
 		/* Redraw stuff */
-		if (p_ptr->redraw) redraw_stuff();
+		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
@@ -1555,16 +1461,16 @@ static void dungeon(void)
 
 
 		/* Process the world */
-		process_world();
+		process_world(c);
 
 		/* Notice stuff */
-		if (p_ptr->notice) notice_stuff();
+		if (p_ptr->notice) notice_stuff(p_ptr);
 
 		/* Update stuff */
-		if (p_ptr->update) update_stuff();
+		if (p_ptr->update) update_stuff(p_ptr);
 
 		/* Redraw stuff */
-		if (p_ptr->redraw) redraw_stuff();
+		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
@@ -1578,16 +1484,25 @@ static void dungeon(void)
 		p_ptr->energy += extract_energy[p_ptr->state.speed];
 
 		/* Give energy to all monsters */
-		for (i = mon_max - 1; i >= 1; i--)
+		for (i = cave_monster_max(cave) - 1; i >= 1; i--)
 		{
+			int mspeed;
+
 			/* Access the monster */
-			m_ptr = &mon_list[i];
+			m_ptr = cave_monster(cave, i);
 
 			/* Ignore "dead" monsters */
 			if (!m_ptr->r_idx) continue;
 
+			/* Calculate the net speed */
+			mspeed = m_ptr->mspeed;
+			if (m_ptr->m_timed[MON_TMD_FAST])
+				mspeed += 10;
+			if (m_ptr->m_timed[MON_TMD_SLOW])
+				mspeed -= 10;
+
 			/* Give this monster some energy */
-			m_ptr->energy += extract_energy[m_ptr->mspeed];
+			m_ptr->energy += extract_energy[mspeed];
 		}
 
 		/* Count game turns */
@@ -1604,15 +1519,14 @@ static void process_some_user_pref_files(void)
 {
 	char buf[1024];
 
-
 	/* Process the "user.prf" file */
-	(void)process_pref_file("user.prf", TRUE);
+	(void)process_pref_file("user.prf", TRUE, TRUE);
 
 	/* Get the "PLAYER.prf" filename */
-	(void)strnfmt(buf, sizeof(buf), "%s.prf", op_ptr->base_name);
+	(void)strnfmt(buf, sizeof(buf), "%s.prf", op_ptr->base_name, TRUE);
 
 	/* Process the "PLAYER.prf" file */
-	(void)process_pref_file(buf, TRUE);
+	(void)process_pref_file(buf, TRUE, TRUE);
 }
 
 
@@ -1645,8 +1559,6 @@ static void process_some_user_pref_files(void)
  */
 void play_game(void)
 {
-	bool existing_dead_save = FALSE;
-
 	/* Initialize */
 	bool new_game = init_angband();
 
@@ -1674,25 +1586,15 @@ void play_game(void)
 
 	p_ptr->is_dead = TRUE;
 
-	if (savefile[0] && file_exists(savefile))
-	{
-		bool ok = old_load();
-		if (!ok) quit("broken savefile");
+	if (savefile[0] && file_exists(savefile)) {
+		if (!savefile_load(savefile))
+			quit("broken savefile");
 
-		if (p_ptr->is_dead && arg_wizard)
-		{
-			p_ptr->is_dead = FALSE;
-			p_ptr->noscore |= NOSCORE_WIZARD;
+		if (p_ptr->is_dead && arg_wizard) {
+				p_ptr->is_dead = FALSE;
+				p_ptr->chp = p_ptr->mhp;
+				p_ptr->noscore |= NOSCORE_WIZARD;
 		}
-
-		else if (p_ptr->is_dead)
-		{
-			existing_dead_save = TRUE;
-		}
-	}
-	else
-	{
-		existing_dead_save = TRUE;
 	}
 
 	/* No living character loaded */
@@ -1753,11 +1655,17 @@ void play_game(void)
 		/* Roll up a new character. Quickstart is allowed if ht_birth is set */
 		player_birth(p_ptr->ht_birth ? TRUE : FALSE);
 
-		/* Randomize the artifacts */
-		if (OPT(adult_randarts))
+		/* Randomize the artifacts if required */
+		if (OPT(birth_randarts) &&
+				(!OPT(birth_keep_randarts) || !p_ptr->randarts)) {
 			do_randart(seed_randart, TRUE);
+			p_ptr->randarts = TRUE;
+		}
 	}
 
+	/* Initialize temporary fields sensibly */
+	p_ptr->object_idx = p_ptr->object_kind_idx = NO_OBJECT;
+	p_ptr->monster_race_idx = 0;
 
 	/* Normal machine (process player name) */
 	if (savefile[0])
@@ -1767,22 +1675,9 @@ void play_game(void)
 	else
 		process_player_name(TRUE);
 
-#if 0        
-	/* Check if we're overwriting a savefile */
-	while (new_game && !existing_dead_save)
-	{
-		bool overwrite = get_check("Continuing will overwrite an existing savefile.  Overwrite? ");
-
-		if (overwrite) break;
-		get_name(TRUE);
-	}
-#endif
-
 	/* Stop the player being quite so dead */
 	p_ptr->is_dead = FALSE;
 
-
-
 	/* Flash a message */
 	prt("Please wait...", 0, 0);
 
@@ -1803,7 +1698,7 @@ void play_game(void)
 
 	/* Redraw stuff */
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP | PR_MONSTER | PR_MESSAGE);
-	redraw_stuff();
+	redraw_stuff(p_ptr);
 
 
 	/* Process some user pref files */
@@ -1815,7 +1710,8 @@ void play_game(void)
 
 
 	/* Generate a dungeon level if needed */
-	if (!character_dungeon) generate_cave();
+	if (!character_dungeon)
+		cave_generate(cave, p_ptr);
 
 
 	/* Character is now "complete" */
@@ -1842,24 +1738,23 @@ void play_game(void)
 		play_ambient_sound();
 
 		/* Process the level */
-		dungeon();
-
+		dungeon(cave);
 
 		/* Notice stuff */
-		if (p_ptr->notice) notice_stuff();
+		if (p_ptr->notice) notice_stuff(p_ptr);
 
 		/* Update stuff */
-		if (p_ptr->update) update_stuff();
+		if (p_ptr->update) update_stuff(p_ptr);
 
 		/* Redraw stuff */
-		if (p_ptr->redraw) redraw_stuff();
+		if (p_ptr->redraw) redraw_stuff(p_ptr);
 
 
 		/* Cancel the target */
 		target_set_monster(0);
 
 		/* Cancel the health bar */
-		health_track(0);
+		health_track(p_ptr, 0);
 
 
 		/* Forget the view */
@@ -1874,11 +1769,10 @@ void play_game(void)
 		message_flush();
 
 		/* Accidental Death */
-		if (p_ptr->playing && p_ptr->is_dead)
-		{
-			/* Mega-Hack -- Allow player to cheat death */
-			if ((p_ptr->wizard || OPT(cheat_live)) && !get_check("Die? "))
-			{
+		if (p_ptr->playing && p_ptr->is_dead) {
+			/* XXX-elly: this does not belong here. Refactor or
+			 * remove. Very similar to do_cmd_wiz_cure_all(). */
+			if ((p_ptr->wizard || OPT(cheat_live)) && !get_check("Die? ")) {
 				/* Mark social class, reset age, if needed */
 				if (p_ptr->sc) p_ptr->sc = p_ptr->age = 0;
 
@@ -1889,7 +1783,7 @@ void play_game(void)
 				p_ptr->noscore |= NOSCORE_WIZARD;
 
 				/* Message */
-				msg_print("You invoke wizard mode and cheat death.");
+				msg("You invoke wizard mode and cheat death.");
 				message_flush();
 
 				/* Cheat death */
@@ -1904,23 +1798,23 @@ void play_game(void)
 				p_ptr->csp_frac = 0;
 
 				/* Hack -- Healing */
-				(void)clear_timed(TMD_BLIND, TRUE);
-				(void)clear_timed(TMD_CONFUSED, TRUE);
-				(void)clear_timed(TMD_POISONED, TRUE);
-				(void)clear_timed(TMD_AFRAID, TRUE);
-				(void)clear_timed(TMD_PARALYZED, TRUE);
-				(void)clear_timed(TMD_IMAGE, TRUE);
-				(void)clear_timed(TMD_STUN, TRUE);
-				(void)clear_timed(TMD_CUT, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_BLIND, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_CONFUSED, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_POISONED, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_AFRAID, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_PARALYZED, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_IMAGE, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_STUN, TRUE);
+				(void)player_clear_timed(p_ptr, TMD_CUT, TRUE);
 
 				/* Hack -- Prevent starvation */
-				(void)set_food(PY_FOOD_MAX - 1);
+				player_set_food(p_ptr, PY_FOOD_MAX - 1);
 
 				/* Hack -- cancel recall */
 				if (p_ptr->word_recall)
 				{
 					/* Message */
-					msg_print("A tension leaves the air around you...");
+					msg("A tension leaves the air around you...");
 					message_flush();
 
 					/* Hack -- Prevent recall */
@@ -1942,7 +1836,7 @@ void play_game(void)
 		if (p_ptr->is_dead) break;
 
 		/* Make a new level */
-		generate_cave();
+		cave_generate(cave, p_ptr);
 	}
 
 	/* Disallow big cursor */
