@@ -17,12 +17,16 @@
  */
 
 #include "angband.h"
+#include "buildid.h"
 #include "cave.h"
 #include "cmds.h"
+#include "files.h"
 #include "game-cmd.h"
 #include "history.h"
 #include "object/tvalsval.h"
+#include "object/pval.h"
 #include "option.h"
+#include "savefile.h"
 #include "ui-menu.h"
 
 #define MAX_PANEL 12
@@ -31,7 +35,7 @@
  * Returns a "rating" of x depending on y, and sets "attr" to the
  * corresponding "attribute".
  */
-static cptr likert(int x, int y, byte *attr)
+static const char *likert(int x, int y, byte *attr)
 {
 	/* Paranoia */
 	if (y <= 0) y = 1;
@@ -111,7 +115,7 @@ static cptr likert(int x, int y, byte *attr)
 void player_flags(bitflag f[OF_SIZE])
 {
 	/* Add racial flags */
-	memcpy(f, rp_ptr->flags, sizeof(rp_ptr->flags));
+	memcpy(f, p_ptr->race->flags, sizeof(p_ptr->race->flags));
 
 	/* Some classes become immune to fear at a certain plevel */
 	if (player_has(PF_BRAVERY_30) && p_ptr->lev >= 30)
@@ -139,7 +143,7 @@ static void display_player_equippy(int y, int x)
 		o_ptr = &p_ptr->inventory[i];
 
 		/* Skip empty objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 
 		/* Get attr/char for display */
 		a = object_attr(o_ptr);
@@ -172,20 +176,20 @@ static const struct player_flag_record player_flag_table[RES_ROWS*4] =
 	{ "rFire",	OF_RES_FIRE,    OF_IM_FIRE, OF_VULN_FIRE },
 	{ "rCold",	OF_RES_COLD,    OF_IM_COLD, OF_VULN_COLD },
 	{ "rPois",	OF_RES_POIS,    FLAG_END,   FLAG_END },
-	{ "rFear",	OF_RES_FEAR,    FLAG_END,   FLAG_END },
 	{ "rLite",	OF_RES_LIGHT,   FLAG_END,   FLAG_END },
 	{ "rDark",	OF_RES_DARK,    FLAG_END,   FLAG_END },
-	{ "rBlnd",	OF_RES_BLIND,   FLAG_END,   FLAG_END },
-
-	{ "rConf",	OF_RES_CONFU,   FLAG_END,   FLAG_END },
 	{ "Sound",	OF_RES_SOUND,   FLAG_END,   FLAG_END },
 	{ "Shard",	OF_RES_SHARD,   FLAG_END,   FLAG_END },
+
 	{ "Nexus",	OF_RES_NEXUS,   FLAG_END,   FLAG_END },
 	{ "Nethr",	OF_RES_NETHR,   FLAG_END,   FLAG_END },
 	{ "Chaos",	OF_RES_CHAOS,   FLAG_END,   FLAG_END },
 	{ "Disen",	OF_RES_DISEN,   FLAG_END,   FLAG_END },
-	{ "S.Dig",	OF_SLOW_DIGEST, FLAG_END,   FLAG_END },
 	{ "Feath",	OF_FEATHER,     FLAG_END,   FLAG_END },
+	{ "pFear",	OF_RES_FEAR,    FLAG_END,   FLAG_END },
+	{ "pBlnd",	OF_RES_BLIND,   FLAG_END,   FLAG_END },
+	{ "pConf",	OF_RES_CONFU,   FLAG_END,   FLAG_END },
+	{ "pStun",	OF_RES_STUN,	FLAG_END,   FLAG_END },
 
 	{ "Light",	OF_LIGHT,       FLAG_END,   FLAG_END },
 	{ "Regen",	OF_REGEN,       FLAG_END,   FLAG_END },
@@ -193,19 +197,19 @@ static const struct player_flag_record player_flag_table[RES_ROWS*4] =
 	{ "Invis",	OF_SEE_INVIS,   FLAG_END,   FLAG_END },
 	{ "FrAct",	OF_FREE_ACT,    FLAG_END,   FLAG_END },
 	{ "HLife",	OF_HOLD_LIFE,   FLAG_END,   FLAG_END },
-	{ "ImpHP",	OF_IMPAIR_HP,   FLAG_END,   FLAG_END },
-	{ "ImpSP",	OF_IMPAIR_MANA, FLAG_END,   FLAG_END },
-	{ " Fear",	OF_AFRAID,      FLAG_END,   FLAG_END },
-
-	{ "Aggrv",	OF_AGGRAVATE,   FLAG_END,   FLAG_END },
 	{ "Stea.",	OF_STEALTH,     FLAG_END,   FLAG_END },
 	{ "Sear.",	OF_SEARCH,      FLAG_END,   FLAG_END },
 	{ "Infra",	OF_INFRA,       FLAG_END,   FLAG_END },
+
 	{ "Tunn.",	OF_TUNNEL,      FLAG_END,   FLAG_END },
 	{ "Speed",	OF_SPEED,       FLAG_END,   FLAG_END },
 	{ "Blows",	OF_BLOWS,       FLAG_END,   FLAG_END },
 	{ "Shots",	OF_SHOTS,       FLAG_END,   FLAG_END },
 	{ "Might",	OF_MIGHT,       FLAG_END,   FLAG_END },
+	{ "S.Dig",	OF_SLOW_DIGEST, FLAG_END,   FLAG_END },
+	{ "ImpHP",	OF_IMPAIR_HP,   FLAG_END,   FLAG_END },
+	{ " Fear",	OF_AFRAID,      FLAG_END,   FLAG_END },
+	{ "Aggrv",	OF_AGGRAVATE,   FLAG_END,   FLAG_END },
 };
 
 #define RES_COLS (5 + 2 + INVEN_TOTAL - INVEN_WIELD)
@@ -242,7 +246,7 @@ static void display_resistance_panel(const struct player_flag_record *resists,
 			/* Wipe flagset */
 			of_wipe(f);
 
-			if (j < INVEN_TOTAL && o_ptr->k_idx)
+			if (j < INVEN_TOTAL && o_ptr->kind)
 			{
 				object_flags_known(o_ptr, f);
 			}
@@ -253,9 +257,9 @@ static void display_resistance_panel(const struct player_flag_record *resists,
 				/* If the race has innate infravision/digging, force the corresponding flag
 				   here.  If we set it in player_flags(), then all callers of that
 				   function will think the infravision is caused by equipment. */
-				if (rp_ptr->infra > 0)
+				if (p_ptr->race->infra > 0)
 					of_on(f, OF_INFRA);
-				if (rp_ptr->r_skills[SKILL_DIGGING] > 0)
+				if (p_ptr->race->r_skills[SKILL_DIGGING] > 0)
 					of_on(f, OF_TUNNEL);
 			}
 
@@ -269,7 +273,7 @@ static void display_resistance_panel(const struct player_flag_record *resists,
 			if (vuln) sym = '-';
 			else if (imm) sym = '*';
 			else if (res) sym = '+';
-			else if ((j < INVEN_TOTAL) && o_ptr->k_idx && 
+			else if ((j < INVEN_TOTAL) && o_ptr->kind && 
 				!object_flag_is_known(o_ptr, resists[i].res_flag)) sym = '?';
 			Term_addch(attr, sym);
 		}
@@ -318,7 +322,7 @@ void display_player_stat_info(void)
 	for (i = 0; i < A_MAX; i++)
 	{
 		/* Reduced */
-		if (p_ptr->state.stat_use[i] < p_ptr->state.stat_top[i])
+		if (p_ptr->stat_cur[i] < p_ptr->stat_max[i])
 		{
 			/* Use lowercase stat name */
 			put_str(stat_names_reduced[i], row+i, col);
@@ -342,11 +346,11 @@ void display_player_stat_info(void)
 		c_put_str(TERM_L_GREEN, buf, row+i, col+5);
 
 		/* Race Bonus */
-		strnfmt(buf, sizeof(buf), "%+3d", rp_ptr->r_adj[i]);
+		strnfmt(buf, sizeof(buf), "%+3d", p_ptr->race->r_adj[i]);
 		c_put_str(TERM_L_BLUE, buf, row+i, col+12);
 
 		/* Class Bonus */
-		strnfmt(buf, sizeof(buf), "%+3d", cp_ptr->c_adj[i]);
+		strnfmt(buf, sizeof(buf), "%+3d", p_ptr->class->c_adj[i]);
 		c_put_str(TERM_L_BLUE, buf, row+i, col+16);
 
 		/* Equipment Bonus */
@@ -357,8 +361,8 @@ void display_player_stat_info(void)
 		cnv_stat(p_ptr->state.stat_top[i], buf, sizeof(buf));
 		c_put_str(TERM_L_GREEN, buf, row+i, col+24);
 
-		/* Only display stat_use if not maximal */
-		if (p_ptr->state.stat_use[i] < p_ptr->state.stat_top[i])
+		/* Only display stat_use if there has been draining */
+		if (p_ptr->stat_cur[i] < p_ptr->stat_max[i])
 		{
 			cnv_stat(p_ptr->state.stat_use[i], buf, sizeof(buf));
 			c_put_str(TERM_YELLOW, buf, row+i, col+31);
@@ -380,7 +384,7 @@ void display_player_stat_info(void)
  */
 static void display_player_sust_info(void)
 {
-	int i, row, col, stat;
+	int i, j, row, col, stat;
 
 	object_type *o_ptr;
 	bitflag f[OF_SIZE];
@@ -421,6 +425,11 @@ static void display_player_sust_info(void)
 		/* Get the object */
 		o_ptr = &p_ptr->inventory[i];
 
+		if (!o_ptr->kind) {
+			col++;
+			continue;
+		}
+
 		/* Get the "known" flags */
 		object_flags_known(o_ptr, f);
 
@@ -437,24 +446,29 @@ static void display_player_sust_info(void)
 				/* Default */
 				c = '*';
 
+				/* Work out which pval we're talking about */
+				j = which_pval(o_ptr, stat_flags[stat]);
+
 				/* Good */
-				if (o_ptr->pval > 0)
+				if (o_ptr->pval[j] > 0)
 				{
 					/* Good */
 					a = TERM_L_GREEN;
 
 					/* Label boost */
-					if (o_ptr->pval < 10) c = I2D(o_ptr->pval);
+					if (o_ptr->pval[j] < 10)
+						c = I2D(o_ptr->pval[j]);
 				}
 
 				/* Bad */
-				if (o_ptr->pval < 0)
+				if (o_ptr->pval[j] < 0)
 				{
 					/* Bad */
 					a = TERM_RED;
 
 					/* Label boost */
-					if (o_ptr->pval > -10) c = I2D(-(o_ptr->pval));
+					if (o_ptr->pval[j] > -10)
+						c = I2D(-(o_ptr->pval[j]));
 				}
 			}
 
@@ -468,7 +482,7 @@ static void display_player_sust_info(void)
 				if (c == '.') c = 's';
 			}
 
-			if ((c == '.') && o_ptr->k_idx && !object_flag_is_known(o_ptr, sustain_flags[stat]))
+			if ((c == '.') && o_ptr->kind && !object_flag_is_known(o_ptr, sustain_flags[stat]))
 				c = '?';
 
 			/* Dump proper character */
@@ -571,7 +585,7 @@ static const char *show_title(void)
 	else if (p_ptr->total_winner || p_ptr->lev > PY_MAX_LEVEL)
 		return "***WINNER***";
 	else
-		return cp_ptr->title[(p_ptr->lev - 1) / 5];
+		return p_ptr->class->title[(p_ptr->lev - 1) / 5];
 }
 
 static const char *show_adv_exp(void)
@@ -722,9 +736,9 @@ static int get_panel(int oid, data_panel *panel, size_t size)
 	assert( size >= (u32b) boundaries[1].page_rows);
 	ret = boundaries[1].page_rows;
 	P_I(TERM_L_BLUE, "Name",	"%y",	s2u(op_ptr->full_name), END  );
-	P_I(TERM_L_BLUE, "Sex",		"%y",	s2u(sp_ptr->title), END  );
-	P_I(TERM_L_BLUE, "Race",	"%y",	s2u(rp_ptr->name), END  );
-	P_I(TERM_L_BLUE, "Class",	"%y",	s2u(cp_ptr->name), END  );
+	P_I(TERM_L_BLUE, "Sex",		"%y",	s2u(p_ptr->sex->title), END  );
+	P_I(TERM_L_BLUE, "Race",	"%y",	s2u(p_ptr->race->name), END  );
+	P_I(TERM_L_BLUE, "Class",	"%y",	s2u(p_ptr->class->name), END  );
 	P_I(TERM_L_BLUE, "Title",	"%y",	s2u(show_title()), END  );
 	P_I(TERM_L_BLUE, "HP",	"%y/%y",	i2u(p_ptr->chp), i2u(p_ptr->mhp)  );
 	P_I(TERM_L_BLUE, "SP",	"%y/%y",	i2u(p_ptr->csp), i2u(p_ptr->msp)  );
@@ -758,8 +772,8 @@ static int get_panel(int oid, data_panel *panel, size_t size)
 	P_I(TERM_L_BLUE, "Fight", "(%+y,%+y)",	i2u(p_ptr->state.dis_to_h), i2u(p_ptr->state.dis_to_d)  );
 	P_I(TERM_L_BLUE, "Melee", "%y",		s2u(show_melee_weapon(&p_ptr->inventory[INVEN_WIELD])), END  );
 	P_I(TERM_L_BLUE, "Shoot", "%y",		s2u(show_missile_weapon(&p_ptr->inventory[INVEN_BOW])), END  );
-	P_I(TERM_L_BLUE, "Blows", "%y.%y/turn",	i2u(p_ptr->state.num_blow / 100), i2u((p_ptr->state.num_blow / 10) % 10) );
-	P_I(TERM_L_BLUE, "Shots", "%y/turn",	i2u(p_ptr->state.num_fire), END  );
+	P_I(TERM_L_BLUE, "Blows", "%y.%y/turn",	i2u(p_ptr->state.num_blows / 100), i2u((p_ptr->state.num_blows / 10) % 10) );
+	P_I(TERM_L_BLUE, "Shots", "%y/turn",	i2u(p_ptr->state.num_shots), END  );
 	P_I(TERM_L_BLUE, "Infra", "%y ft",	i2u(p_ptr->state.see_infra * 10), END  );
 	P_I(TERM_L_BLUE, "Speed", "%y",		s2u(show_speed()), END );
 	P_I(TERM_L_BLUE, "Burden","%.1y lbs",	f2u(p_ptr->total_weight/10.0), END  );
@@ -852,10 +866,10 @@ static int get_panel(int oid, data_panel *panel, size_t size)
 	P_I(TERM_L_BLUE, "Height",		"%y",	i2u(p_ptr->ht), END  );
 	P_I(TERM_L_BLUE, "Weight",		"%y",	i2u(p_ptr->wt), END  );
 	P_I(TERM_L_BLUE, "Social",		"%y",	s2u(show_status()), END  );
-	P_I(TERM_L_BLUE, "Maximize",	"%y",	c2u(OPT(adult_maximize) ? 'Y' : 'N'), END);
+	P_I(TERM_L_BLUE, "Maximize",	"%y",	c2u(OPT(birth_maximize) ? 'Y' : 'N'), END);
 #if 0
 	/* Preserve mode deleted */
-	P_I(TERM_L_BLUE, "Preserve",	"%y",	c2u(adult_preserve ? 'Y' : 'N'), END);
+	P_I(TERM_L_BLUE, "Preserve",	"%y",	c2u(birth_preserve ? 'Y' : 'N'), END);
 #endif
 	assert(i == boundaries[5].page_rows);
 	return ret;
@@ -911,6 +925,8 @@ void display_player(int mode)
 	/* Erase screen */
 	clear_from(0);
 
+	/* When not playing, do not display in subwindows */
+	if (Term != angband_term[0] && !p_ptr->playing) return;
 
 	/* Stat info */
 	display_player_stat_info();
@@ -953,7 +969,7 @@ errr file_character(const char *path, bool full)
 
 	ang_file *fp;
 
-	store_type *st_ptr = &store[STORE_HOME];
+	struct store *st_ptr = &stores[STORE_HOME];
 
 	char o_name[80];
 
@@ -976,8 +992,7 @@ errr file_character(const char *path, bool full)
  	if (!OPT(xchars_to_file)) Term->xchar_hook = NULL;
 
 	/* Begin dump */
-	file_putf(fp, "  [%s %s Character Dump]\n\n",
-	        VERSION_NAME, VERSION_STRING);
+	file_putf(fp, "  [%s Character Dump]\n\n", buildid);
 
 
 	/* Display player */
@@ -1092,7 +1107,7 @@ errr file_character(const char *path, bool full)
 				ODESC_PREFIX | ODESC_FULL);
 
 		x_file_putf(fp, encoding, "%c) %s\n", index_to_label(i), o_name);
-		if (p_ptr->inventory[i].k_idx)
+		if (p_ptr->inventory[i].kind)
 			object_info_chardump(fp, &p_ptr->inventory[i], 5, 72);
 	}
 
@@ -1100,7 +1115,7 @@ errr file_character(const char *path, bool full)
 	file_putf(fp, "\n\n  [Character Inventory]\n\n");
 	for (i = 0; i < INVEN_PACK; i++)
 	{
-		if (!p_ptr->inventory[i].k_idx) break;
+		if (!p_ptr->inventory[i].kind) break;
 
 		object_desc(o_name, sizeof(o_name), &p_ptr->inventory[i],
 					ODESC_PREFIX | ODESC_FULL);
@@ -1139,7 +1154,7 @@ errr file_character(const char *path, bool full)
 	file_putf(fp, "  [Options]\n\n");
 
 	/* Dump options */
-	for (i = OPT_ADULT; i < OPT_MAX; i++)
+	for (i = OPT_BIRTH; i < OPT_BIRTH + N_OPTS_BIRTH; i++)
 	{
 		if (option_name(i))
 		{
@@ -1185,11 +1200,11 @@ static void string_lower(char *buf)
  * functionality, especially when moving backwards through a file, or
  * forwards through a file by less than a page at a time.  XXX XXX XXX
  */
-bool show_file(cptr name, cptr what, int line, int mode)
+bool show_file(const char *name, const char *what, int line, int mode)
 {
 	int i, k, n;
 
-	char ch;
+	struct keypress ch;
 
 	/* Number of "real" lines passed by */
 	int next = 0;
@@ -1213,7 +1228,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 	char *find = NULL;
 
 	/* Jump to this tag */
-	cptr tag = NULL;
+	const char *tag = NULL;
 
 	/* Hold a string to find */
 	char finder[80] = "";
@@ -1300,7 +1315,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 	if (!fff)
 	{
 		/* Message */
-		msg_format("Cannot open '%s'.", name);
+		msg("Cannot open '%s'.", name);
 		message_flush();
 
 		/* Oops */
@@ -1437,7 +1452,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 			/* Highlight "shower" */
 			if (shower[0])
 			{
-				cptr str = lc_buf;
+				const char *str = lc_buf;
 
 				/* Display matches */
 				while ((str = strstr(str, shower)) != NULL)
@@ -1467,7 +1482,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 
 
 		/* Show a general "title" */
-		prt(format("[%s %s, %s, Line %d-%d/%d]", VERSION_NAME, VERSION_STRING,
+		prt(format("[%s, %s, Line %d-%d/%d]", buildid,
 		           caption, line, line + hgt - 4, size), 0, 0);
 
 
@@ -1496,16 +1511,16 @@ bool show_file(cptr name, cptr what, int line, int mode)
 		ch = inkey();
 
 		/* Exit the help */
-		if (ch == '?') break;
+		if (ch.code == '?') break;
 
 		/* Toggle case sensitive on/off */
-		if (ch == '!')
+		if (ch.code == '!')
 		{
 			case_sensitive = !case_sensitive;
 		}
 
 		/* Try showing */
-		if (ch == '&')
+		if (ch.code == '&')
 		{
 			/* Get "shower" */
 			prt("Show: ", hgt - 1, 0);
@@ -1516,7 +1531,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 		}
 
 		/* Try finding */
-		if (ch == '/')
+		if (ch.code == '/')
 		{
 			/* Get "finder" */
 			prt("Find: ", hgt - 1, 0);
@@ -1536,7 +1551,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 		}
 
 		/* Go to a specific line */
-		if (ch == '#')
+		if (ch.code == '#')
 		{
 			char tmp[80] = "0";
 
@@ -1546,7 +1561,7 @@ bool show_file(cptr name, cptr what, int line, int mode)
 		}
 
 		/* Go to a specific file */
-		if (ch == '%')
+		if (ch.code == '%')
 		{
 			char ftmp[80] = "help.hlp";
 
@@ -1554,69 +1569,63 @@ bool show_file(cptr name, cptr what, int line, int mode)
 			if (askfor_aux(ftmp, sizeof(ftmp), NULL))
 			{
 				if (!show_file(ftmp, NULL, 0, mode))
-					ch = ESCAPE;
+					ch.code = ESCAPE;
 			}
 		}
 
-		/* Back up one line */
-		if (ch == ARROW_UP || ch == '8')
-		{
-			line = line - 1;
-		}
+		switch (ch.code) {
+			/* up a line */
+			case ARROW_UP:
+			case '8': line--; break;
 
-		/* Back up one full page */
-		if ((ch == '9') || (ch == '-'))
-		{
-			line = line - (hgt - 4);
-		}
+			/* up a page */
+			case KC_PGUP:
+			case '9':
+			case '-': line -= (hgt - 4); break;
 
-		/* Back to the top */
-		if (ch == '7')
-		{
-			line = 0;
-		}
+			/* home */
+			case KC_HOME:
+			case '7': line = 0; break;
 
-		/* Advance one line */
-		if ((ch == ARROW_DOWN) || (ch == '2') || (ch == '\n') || (ch == '\r'))
-		{
-			line = line + 1;
-		}
+			/* down a line */
+			case ARROW_DOWN:
+			case '2':
+			case '\n':
+			case '\r': line++; break;
 
-		/* Advance one full page */
-		if ((ch == '3') || (ch == ' '))
-		{
-			line = line + (hgt - 4);
-		}
+			/* down a page */
+			case KC_PGDOWN:
+			case '3':
+			case ' ': line += hgt - 4; break;
 
-		/* Advance to the bottom */
-		if (ch == '1')
-		{
-			line = size;
+			/* end */
+			case KC_END:
+			case '1': line = size; break;
 		}
 
 		/* Recurse on letters */
-		if (menu && isalpha((unsigned char)ch))
+		if (menu && isalpha((unsigned char)ch.code))
 		{
 			/* Extract the requested menu item */
-			k = A2I(ch);
+			k = A2I(ch.code);
 
 			/* Verify the menu item */
 			if ((k >= 0) && (k <= 25) && hook[k][0])
 			{
 				/* Recurse on that file */
-				if (!show_file(hook[k], NULL, 0, mode)) ch = ESCAPE;
+				if (!show_file(hook[k], NULL, 0, mode)) ch.code = ESCAPE;
 			}
 		}
 
 		/* Exit on escape */
-		if (ch == ESCAPE) break;
+		if (ch.code == ESCAPE) break;
 	}
 
 	/* Close the file */
 	file_close(fff);
 
 	/* Done */
-	return (ch != '?');
+	return (ch.code != '?');
 }
 
 
@@ -1714,13 +1723,13 @@ void process_player_name(bool sf)
 void save_game(void)
 {
 	/* Disturb the player */
-	disturb(1, 0);
+	disturb(p_ptr, 1, 0);
 
 	/* Clear messages */
 	message_flush();
 
 	/* Handle stuff */
-	handle_stuff();
+	handle_stuff(p_ptr);
 
 	/* Message */
 	prt("Saving game...", 0, 0);
@@ -1735,16 +1744,10 @@ void save_game(void)
 	signals_ignore_tstp();
 
 	/* Save the player */
-	if (old_save())
-	{
+	if (savefile_save(savefile))
 		prt("Saving game... done.", 0, 0);
-	}
-
-	/* Save failed (oops) */
 	else
-	{
 		prt("Saving game... failed!", 0, 0);
-	}
 
 	/* Allow suspend again */
 	signals_handle_tstp();
@@ -1769,7 +1772,7 @@ void save_game(void)
 void close_game(void)
 {
 	/* Handle stuff */
-	handle_stuff();
+	handle_stuff(p_ptr);
 
 	/* Flush the messages */
 	message_flush();
@@ -1800,11 +1803,12 @@ void close_game(void)
 
 		if (Term->mapped_flag)
 		{
-			/* Prompt for scores XXX XXX XXX */
-			prt("Press Return (or Escape).", 0, 40);
+			struct keypress ch;
 
-			/* Predict score (or ESCAPE) */
-			if (inkey() != ESCAPE) predict_score();
+			prt("Press Return (or Escape).", 0, 40);
+			ch = inkey();
+			if (ch.code != ESCAPE)
+				predict_score();
 		}
 	}
 
@@ -1817,53 +1821,6 @@ void close_game(void)
 	signals_handle_tstp();
 }
 
-
-/*
- * Handle abrupt death of the visual system
- *
- * This routine is called only in very rare situations, and only
- * by certain visual systems, when they experience fatal errors.
- *
- * XXX XXX Hack -- clear the death flag when creating a HANGUP
- * save file so that player can see tombstone when restart.
- */
-void exit_game_panic(void)
-{
-	/* If nothing important has happened, just quit */
-	if (!character_generated || character_saved) quit("panic");
-
-	/* Mega-Hack -- see "msg_print()" */
-	msg_flag = FALSE;
-
-	/* Clear the top line */
-	prt("", 0, 0);
-
-	/* Hack -- turn off some things */
-	disturb(1, 0);
-
-	/* Hack -- Delay death XXX XXX XXX */
-	if (p_ptr->chp < 0) p_ptr->is_dead = FALSE;
-
-	/* Hardcode panic save */
-	p_ptr->panic_save = 1;
-
-	/* Forbid suspend */
-	signals_ignore_tstp();
-
-	/* Indicate panic save */
-	my_strcpy(p_ptr->died_from, "(panic save)", sizeof(p_ptr->died_from));
-
-	/* Panic save, or get worried */
-	if (!old_save()) quit("panic save failed!");
-
-
-	/* Successful panic save */
-	quit("panic save succeeded!");
-}
-
-
-
-
 static void write_html_escape_char(ang_file *fp, char c)
 {
 	switch (c)
@@ -1885,7 +1842,7 @@ static void write_html_escape_char(ang_file *fp, char c)
 
 
 /* Take an html screenshot */
-void html_screenshot(cptr name, int mode)
+void html_screenshot(const char *name, int mode)
 {
 	int y, x;
 	int wid, hgt;
@@ -1922,8 +1879,7 @@ void html_screenshot(cptr name, int mode)
 	if (mode == 0)
 	{
 		file_putf(fp, "<!DOCTYPE html><html><head>\n");
-		file_putf(fp, "  <meta='generator' content='%s %s'>\n",
-	            	VERSION_NAME, VERSION_STRING);
+		file_putf(fp, "  <meta='generator' content='%s'>\n", buildid);
 		file_putf(fp, "  <title>%s</title>\n", name);
 		file_putf(fp, "</head>\n\n");
 		file_putf(fp, "<body style='color: #fff; background: #000;'>\n");
