@@ -3,13 +3,13 @@
 #ifndef FILES_H
 #define FILES_H
 
-extern void html_screenshot(cptr name, int mode);
+extern void html_screenshot(const char *name, int mode);
 extern void player_flags(bitflag f[OF_SIZE]);
 extern void display_player(int mode);
 extern void display_player_stat_info(void);
 extern void display_player_xtra_info(void);
-extern errr file_character(cptr name, bool full);
-extern bool show_file(cptr name, cptr what, int line, int mode);
+extern errr file_character(const char *name, bool full);
+extern bool show_file(const char *name, const char *what, int line, int mode);
 extern void do_cmd_help(void);
 extern void process_player_name(bool sf);
 extern bool get_name(char *buf, size_t buflen);
