@@ -27,9 +27,9 @@ EFFECT(CURE_CONFUSION, FALSE,  4,	"cures confusion")
 EFFECT(CURE_MIND,      FALSE,  8,	"cures confusion and hallucination, removes fear and grants you temporary resistance to confusion")
 EFFECT(CURE_BODY,      FALSE,  7,	"heals cut damage, and cures stunning, poison and blindness")
 
-EFFECT(CURE_LIGHT,     FALSE,  5,	"heals you a small amount (15% of your wounds, minimum 15HP), heals some cut damage, makes you a little less confused, and cures blindness")
-EFFECT(CURE_SERIOUS,   FALSE,  8,	"heals you a fair amount (1/5 of your wounds, minimum 25HP), heals cut damage, and cures blindness and confusion")
-EFFECT(CURE_CRITICAL,  FALSE, 10,	"heals you a large amount (1/4 of your wounds, minimum 30HP), heals cut damage, and cures stunning, poisoning, blindness, and confusion")
+EFFECT(CURE_LIGHT,     FALSE,  3,	"heals 20 hitpoints, some cut damage, makes you a little less confused, and cures blindness")
+EFFECT(CURE_SERIOUS,   FALSE,  6,	"heals 40 hitpoints, cut damage, and cures blindness and confusion")
+EFFECT(CURE_CRITICAL,  FALSE,  9,	"heals 60 hitpoints, cut damage, and cures stunning, poisoning, blindness, and confusion")
 EFFECT(CURE_FULL,      FALSE, 12,	"heals you a really large amount (35% of max HP, minimum 300HP), heals cut damage, and cures stunning, poisoning, blindness, and confusion")
 EFFECT(CURE_FULL2,     FALSE, 18,	"restores 1200 hit points, heals cut damage, and cures stunning, poisoning, blindness, and confusion")
 EFFECT(CURE_NONORLYBIG,FALSE, 21,	"restores 5000 hit points, restores experience and stats, heals cut damage, and cures stunning, poison, blindness, and confusion")
@@ -103,6 +103,7 @@ EFFECT(ENCHANT_TODAM,  FALSE, 20,	"attempts to magically enhance a weapon's to-d
 EFFECT(ENCHANT_WEAPON, FALSE, 22,	"attempts to magically enhance a weapon both to-hit and to-dam")
 EFFECT(ENCHANT_ARMOR,  FALSE, 12,	"attempts to magically enhance a piece of armour")
 EFFECT(ENCHANT_ARMOR2, FALSE, 15,	"attempts to magically enhance a piece of armour with high chance of success")
+EFFECT(RESTORE_ITEM,   FALSE, 10,	"restores an item after disenchantment or damage")
 EFFECT(IDENTIFY,       FALSE,  9,	"reveals to you the extent of an item's magical powers")
 EFFECT(REMOVE_CURSE,   FALSE,  8,	"removes all ordinary curses from all equipped items")
 EFFECT(REMOVE_CURSE2,  FALSE, 20,	"removes all curses from all equipped items")
@@ -241,7 +242,7 @@ EFFECT(DRAGON_BLUE,    TRUE,   18,	"allows you to breathe lightning for 100 dama
 EFFECT(DRAGON_GREEN,   TRUE,   19,	"allows you to breathe poison gas for 150 damage")
 EFFECT(DRAGON_RED,     TRUE,   20,	"allows you to breathe fire for 200 damage")
 EFFECT(DRAGON_MULTIHUED, TRUE, 20,	"allows you to breathe the elements for 250 damage")
-EFFECT(DRAGON_BRONZE,  TRUE,   19,	"allows you to breathe confusion for 120 damage")
+EFFECT(DRAGON_BRONZE,  TRUE,   INHIBIT_POWER,	"allows you to breathe confusion for 120 damage")
 EFFECT(DRAGON_GOLD,    TRUE,   19,	"allows you to breathe sound for 130 damage")
 EFFECT(DRAGON_CHAOS,   TRUE,   23,	"allows you to breathe chaos or disenchantment for 220 damage")
 EFFECT(DRAGON_LAW,     TRUE,   23,	"allows you to breathe sound/shards for 230 damage")
