@@ -3,6 +3,7 @@
  * Purpose: Support for "curses" systems
  *
  * Copyright (c) 1997 Ben Harrison, and others
+ * Copyright (c) 2009-2011 Erik Osheim
  *
  * This work is free software; you can redistribute it and/or modify it
  * under the terms of either:
@@ -15,7 +16,10 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
+
 #include "angband.h"
+#include "buildid.h"
+#include "cmds.h"
 
 #ifdef USE_GCU
 #include "main.h"
@@ -25,9 +29,6 @@
 #include "locale.h"
 #include "langinfo.h"
 
-/* included for redrawing code, to prevent warnings */
-#include "cmds.h"
-
 /* Avoid 'struct term' name conflict with <curses.h> (via <term.h>) on AIX */
 #define term System_term
 
@@ -48,8 +49,6 @@
 
 #undef term
 
-
-
 /*
  * Use POSIX terminal I/O
  */
@@ -74,14 +73,11 @@ _stdcall void Sleep(int);
 # include <termios.h>
 #endif
 
-
-
 /*
  * If you have errors relating to curs_set(), comment out the following line
- */ 
+ */
 #define USE_CURS_SET
 
-
 /*
  * If you have errors with any of the functions mentioned below, try
  * uncommenting the line it's mentioned on.
@@ -90,8 +86,6 @@ _stdcall void Sleep(int);
 /* #define nonl() */
 /* #define nl() */
 
-
-
 /*
  * Save the "normal" and "angband" terminal settings
  */
@@ -112,8 +106,7 @@ static bool loaded_terminfo;
 /*
  * Information about a term
  */
-typedef struct term_data
-{
+typedef struct term_data {
 	term t;                 /* All term info */
 	WINDOW *win;            /* Pointer to the curses window */
 } term_data;
@@ -127,13 +120,21 @@ static term_data data[MAX_TERM_DATA];
 /* Number of initialized "term" structures */
 static int active = 0;
 
-#ifdef A_ALTCHARSET
+#define CTRL_ORE 1
+#define CTRL_WALL 2
+#define CTRL_ROCK 3
 
-/* Whether or not to use the special ACS characters */
-static int use_alt_charset = 1;
-
-#endif
+static char ctrl_char[32] = {
+	'\0', '*', '#', '%', '?', '?', '?', '\'', '+', '?', '?', '+',
+	'+', '+', '+', '+', '~', '-', '-', '-', '_', '+', '+', '+',
+	'+', '|', '?', '?', '?', '?', '?', '.'
+};
 
+static int ctrl_attr[32] = {
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+	0, 0, 0, 0, 0, 0, 0, 0
+};
 
 #ifdef A_COLOR
 
@@ -150,36 +151,20 @@ static int use_alt_charset = 1;
  */
 static int can_use_color = FALSE;
 
-/*
- * Software flag -- we are allowed to change the colors
- */
-static int can_fix_color = FALSE;
-
 /*
  * Simple Angband to Curses color conversion table
  */
 static int colortable[BASIC_COLORS];
 
 /* Screen info: use one big Term 0, or other subwindows? */
-static bool use_big_screen;
+static bool use_big_screen = FALSE;
+static bool bold_extended = FALSE;
 
 /*
  * Background color we should draw with; either BLACK or DEFAULT
  */
 static int bg_color = COLOR_BLACK;
 
-/*
- * Lookup table for the "alternate character set".
- *
- * It's worth noting that curses already has this, as an undocumented
- * (but exported) internal variable named acs_map.  I wish I could use
- * it.
- */
-static unsigned int acs_table[32] = {
-	0, '*', '#', '?', '?', '?', '?', '\'', '+', '?', '?', '+',
-	'+', '+', '+', '+', '~', '-', '-', '-', '_', '+', '+', '+',
-	'+', '|', '?', '?', '?', '?', '?', '.'
-};
 
 #define PAIR_WHITE 0
 #define PAIR_RED 1
@@ -192,36 +177,23 @@ static unsigned int acs_table[32] = {
 
 #endif
 
-
-
 /*
  * Place the "keymap" into its "normal" state
  */
-static void keymap_norm(void)
-{
-
+static void keymap_norm(void) {
 #ifdef USE_TPOSIX
-
-	/* restore the saved values of the special chars */
 	(void)tcsetattr(0, TCSAFLUSH, &norm_termios);
-
 #endif
-
-
 }
 
 
 /*
  * Place the "keymap" into the "game" state
  */
-static void keymap_game(void)
-{
-
+static void keymap_game(void) {
 #ifdef USE_TPOSIX
-
-	/* restore the saved values of the special chars */
+	/* Set the game's termios settings */
 	(void)tcsetattr(0, TCSAFLUSH, &game_termios);
-
 #endif
 }
 
@@ -229,28 +201,20 @@ static void keymap_game(void)
 /*
  * Save the normal keymap
  */
-static void keymap_norm_prepare(void)
-{
-
+static void keymap_norm_prepare(void) {
 #ifdef USE_TPOSIX
-
-	/* Get the normal keymap */
+	/* Restore the normal termios settings */
 	tcgetattr(0, &norm_termios);
-
 #endif
-
 }
 
 
 /*
  * Save the keymaps (normal and game)
  */
-static void keymap_game_prepare(void)
-{
-
+static void keymap_game_prepare(void) {
 #ifdef USE_TPOSIX
-
-	/* Acquire the current mapping */
+	/* Save the current termios settings */
 	tcgetattr(0, &game_termios);
 
 	/* Force "Ctrl-C" to interupt */
@@ -264,8 +228,6 @@ static void keymap_game_prepare(void)
 	game_termios.c_cc[VDSUSP] = (char)-1;
 #endif
 
-	/* Hack -- Leave "VSTART/VSTOP" alone */
-
 	/* Disable the standard control characters */
 	game_termios.c_cc[VQUIT] = (char)-1;
 	game_termios.c_cc[VERASE] = (char)-1;
@@ -279,26 +241,18 @@ static void keymap_game_prepare(void)
 
 	/* Turn off flow control (enable ^S) */
 	game_termios.c_iflag &= ~IXON;
-
 #endif
-
-
 }
 
 
-
-
 /*
  * Suspend/Resume
  */
-static errr Term_xtra_gcu_alive(int v)
-{
-	int x, y;
+static errr Term_xtra_gcu_alive(int v) {
+	if (!v) {
+		/* Suspend */
+		int x, y;
 
-
-	/* Suspend */
-	if (!v)
-	{
 		/* Go to normal keymap mode */
 		keymap_norm();
 
@@ -311,7 +265,7 @@ static errr Term_xtra_gcu_alive(int v)
 		Term_xtra(TERM_XTRA_SHAPE, 1);
 
 		/* Flush the curses buffer */
-		(void)refresh();
+		refresh();
 
 		/* Get current cursor position */
 		getyx(stdscr, y, x);
@@ -323,15 +277,10 @@ static errr Term_xtra_gcu_alive(int v)
 		endwin();
 
 		/* Flush the output */
-		(void)fflush(stdout);
-	}
+		fflush(stdout);
 
-	/* Resume */
-	else
-	{
-		/* Refresh */
-		/* (void)touchwin(curscr); */
-		/* (void)wrefresh(curscr); */
+	} else {
+		/* Resume */
 
 		/* Restore the settings */
 		cbreak();
@@ -343,22 +292,15 @@ static errr Term_xtra_gcu_alive(int v)
 	}
 
 	/* Success */
-	return (0);
+	return 0;
 }
 
-
-#ifdef A_ALTCHARSET
-const char help_gcu[] = "Text mode, subopts -b(ig screen) -a(scii) -g(raphics)";
-#else
-const char help_gcu[] = "Text mode, subopts -b(ig screen)";
-#endif
-
+const char help_gcu[] = "Text mode, subopts -b(ig screen) -a(scii) -B(old)";
 
 /*
  * Init the "curses" system
  */
-static void Term_init_gcu(term *t)
-{
+static void Term_init_gcu(term *t) {
 	term_data *td = (term_data *)(t->data);
 
 	/*
@@ -371,13 +313,13 @@ static void Term_init_gcu(term *t)
 	if (active++ != 0) return;
 
 	/* Erase the window */
-	(void)wclear(td->win);
+	wclear(td->win);
 
 	/* Reset the cursor */
-	(void)wmove(td->win, 0, 0);
+	wmove(td->win, 0, 0);
 
 	/* Flush changes */
-	(void)wrefresh(td->win);
+	wrefresh(td->win);
 
 	/* Game keymap */
 	keymap_game();
@@ -387,8 +329,7 @@ static void Term_init_gcu(term *t)
 /*
  * Nuke the "curses" system
  */
-static void Term_nuke_gcu(term *t)
-{
+static void Term_nuke_gcu(term *t) {
 	int x, y;
 	term_data *td = (term_data *)(t->data);
 
@@ -413,13 +354,13 @@ static void Term_nuke_gcu(term *t)
 	mvcur(y, x, LINES - 1, 0);
 
 	/* Flush the curses buffer */
-	(void)refresh();
+	refresh();
 
 	/* Exit curses */
 	endwin();
 
 	/* Flush the output */
-	(void)fflush(stdout);
+	fflush(stdout);
 
 	/* Normal keymap */
 	keymap_norm();
@@ -431,47 +372,33 @@ static void Term_nuke_gcu(term *t)
  * correct dimensions. Terminal layout: 0|2
  *                                      1|3
  */
-void get_gcu_term_size(int i, int *rows, int *cols, int *y, int *x)
-{
-	if (use_big_screen && i == 0)
-	{
+void get_gcu_term_size(int i, int *rows, int *cols, int *y, int *x) {
+	if (use_big_screen && i == 0) {
 		*rows = LINES;
 		*cols = COLS;
 		*y = *x = 0;
-	}
-	else if (use_big_screen)
-	{
+	} else if (use_big_screen) {
 		*rows = *cols = *y = *x = 0;
-	}
-	else if (i == 0)
-	{
+	} else if (i == 0) {
 		*rows = 24;
 		*cols = 80;
 		*y = *x = 0;
-	}
-	else if (i == 1)
-	{
+	} else if (i == 1) {
 		*rows = LINES - 25;
 		*cols = 80;
 		*y = 25;
 		*x = 0;
-	}
-	else if (i == 2)
-	{
+	} else if (i == 2) {
 		*rows = 24;
 		*cols = COLS - 81;
 		*y = 0;
 		*x = 81;
-	}
-	else if (i == 3)
-	{
+	} else if (i == 3) {
 		*rows = LINES - 25;
 		*cols = COLS - 81;
 		*y = 25;
 		*x = 81;
-	}
-	else
-	{
+	} else {
 		*rows = *cols = *y = *x = 0;
 	}
 }
@@ -480,13 +407,11 @@ void get_gcu_term_size(int i, int *rows, int *cols, int *y, int *x)
 /*
  * Query ncurses for new screen size and try to resize the GCU terms.
  */
-void do_gcu_resize(void)
-{
+void do_gcu_resize(void) {
 	int i, rows, cols, y, x;
 	term *old_t = Term;
 	
-	for (i = 0; i < MAX_TERM_DATA; i++)
-	{
+	for (i = 0; i < MAX_TERM_DATA; i++) {
 		/* If we're using a big screen, we only care about Term-0 */
 		if (use_big_screen && i > 0) break;
 		
@@ -508,35 +433,20 @@ void do_gcu_resize(void)
 /*
  * Process events, with optional wait
  */
-static errr Term_xtra_gcu_event(int v)
-{
+static errr Term_xtra_gcu_event(int v) {
 	int i, j, k;
 
-	/* Wait */
-	if (v)
-	{
-		/* Get a keypress; we use halfdelay(1) so if the user takes more */
-		/* than 0.1 seconds we get a chance to do updates. */
+	if (v) {
+		/* Wait for a keypress; use halfdelay(1) so if the user takes more */
+		/* than 0.2 seconds we get a chance to do updates. */
 		halfdelay(2);
 		i = getch();
-		while (i == ERR)
-		{
+		while (i == ERR) {
 			i = getch();
 			idle_update();
 		}
 		cbreak();
-
-		/* Mega-Hack -- allow graceful "suspend" */
-		for (k = 0; (k < 10) && (i == ERR); k++) i = getch();
-
-		/* Broken input is special */
-		if (i == ERR) exit_game_panic();
-		if (i == EOF) exit_game_panic();
-	}
-
-	/* Do not wait */
-	else
-	{
+	} else {
 		/* Do not wait for it */
 		nodelay(stdscr, TRUE);
 
@@ -553,8 +463,7 @@ static errr Term_xtra_gcu_event(int v)
 
 	/* Not sure if this is portable to non-ncurses platforms */
 	#ifdef USE_NCURSES
-	if (i == KEY_RESIZE)
-	{
+	if (i == KEY_RESIZE) {
 		/* wait until we go one second (10 deci-seconds) before actually
 		 * doing the resizing. users often end up triggering multiple
 		 * KEY_RESIZE events while changing window size. */
@@ -586,17 +495,13 @@ static errr Term_xtra_gcu_event(int v)
 	 * available; this seems like an acceptable risk to fix problems associated
 	 * with various terminal emulators (I'm looking at you PuTTY).
 	 */
-	if (i == 27) /* ESC */
-	{
+	if (i == 27) { /* ESC */
 		nodelay(stdscr, TRUE);
 		j = getch();
-		switch (j)
-		{
-			case 'O':
-			{
+		switch (j) {
+			case 'O': {
 				k = getch();
-				switch (k)
-				{
+				switch (k) {
 					/* PuTTY number pad */
 					case 'q': i = '1'; break;
 					case 'r': i = '2'; break;
@@ -624,13 +529,26 @@ static errr Term_xtra_gcu_event(int v)
 
 #ifdef KEY_DOWN
 	/* Handle arrow keys */
-	switch (i)
-	{
+	switch (i) {
 		case KEY_DOWN:  i = ARROW_DOWN;  break;
 		case KEY_UP:    i = ARROW_UP;    break;
 		case KEY_LEFT:  i = ARROW_LEFT;  break;
 		case KEY_RIGHT: i = ARROW_RIGHT; break;
-		default:
+
+		/* keypad keys */
+		case 0xFC: i = '0'; break;
+		case 0xFD: i = '.'; break;
+		case 0xC0: i = '\b'; break;
+		case 0xDF: i = '1'; break;
+		case 0xF5: i = '3'; break;
+		case 0xE9: i = '5'; break;
+		case 0xC1: i = '7'; break;
+		case 0xF4: i = '9'; break;
+
+		/* try to compensate for inadequate terminfo */
+		case 263: i = '\b'; break;
+
+		default: {
 			if (i < KEY_MIN) break;
 
 			/* Mega-Hack -- Fold, spindle, and mutilate
@@ -642,173 +560,122 @@ static errr Term_xtra_gcu_event(int v)
 
 			i = 128 + (i & 127);
 			break;
+		}
 	}
 #endif
 
 	/* Enqueue the keypress */
-	Term_keypress(i);
+	Term_keypress(i, 0);
 
 	/* Success */
 	return (0);
 }
 
+int scale_color(int i, int j, int scale) {
+	return (angband_color_table[i][j] * (scale - 1) + 127) / 255;
+}
 
-void set_256color_table(int i, int fg)
-{
-	init_pair(i + 1, fg, bg_color);
-	colortable[i] = COLOR_PAIR(i + 1) | A_BRIGHT;
+int create_color(int i, int scale) {
+	int r = scale_color(i, 1, scale);
+	int g = scale_color(i, 2, scale);
+	int b = scale_color(i, 3, scale);
+	int rgb = 16 + scale * scale * r + scale * g + b;
+
+	/* In the case of white and black we need to use the ANSI colors */
+	if (r == g && g == b) {
+		if (b == 0) rgb = 0;
+		if (b == scale) rgb = 15;
+	}
+
+	return rgb;
 }
 
 
 /*
  * React to changes
  */
-static errr Term_xtra_gcu_react(void)
-{
+static errr Term_xtra_gcu_react(void) {
 
 #ifdef A_COLOR
+	if (COLORS == 256 || COLORS == 88) {
+		/* If we have more than 16 colors, find the best matches. These numbers
+		 * correspond to xterm/rxvt's builtin color numbers--they do not
+		 * correspond to curses' constants OR with curses' color pairs.
+		 *
+		 * XTerm has 216 (6*6*6) RGB colors, with each RGB setting 0-5.
+		 * RXVT has 64 (4*4*4) RGB colors, with each RGB setting 0-3.
+		 *
+		 * Both also have the basic 16 ANSI colors, plus some extra grayscale
+		 * colors which we do not use.
+		 */
+		int i;
+		int scale = COLORS == 256 ? 6 : 4;
 
-	int i;
-
-	if (can_fix_color)
-	{
-		/* Initialize the curses colors to our own RGB definitions */
-		for (i = 0; i < BASIC_COLORS; i++)
-		{
-			/* Set one color (note scaling) */
-			init_color(i, angband_color_table[i][1] * 1000 / 255,
-						  angband_color_table[i][2] * 1000 / 255,
-						  angband_color_table[i][3] * 1000 / 255);
+		for (i = 0; i < BASIC_COLORS; i++) {
+			int fg = create_color(i, scale);
+			init_pair(i + 1, fg, bg_color);
+			if (bold_extended)
+				colortable[i] = COLOR_PAIR(i + 1) | A_BRIGHT;
+			else
+				colortable[i] = COLOR_PAIR(i + 1);
 		}
 	}
-	else if (COLORS == 256)
-	{
-		/* If we have 256 colors, find the best matches. These numbers
-		 * correspond to xterm's builtin color numbers--they do not correspond
-		 * to curses' constants OR with curses' color pairs.
-		 *
-		 * XTerm has 216 (6*6*6) colors, with each RGB setting 0-5. So, to find
-		 * the "closest" match, I just multiply Angband's RGB setting by 5 and
-		 * then divide by 255. I used true rounding because using the floor
-		 * produces too dark a feel, and using the ceiling kind of washes all
-		 * the colors out.
-		 */
-		set_256color_table(TERM_DARK, 0);
-		set_256color_table(TERM_WHITE, 15);
-		set_256color_table(TERM_SLATE, 145);
-		set_256color_table(TERM_ORANGE, 214);
-		set_256color_table(TERM_RED, 160);
-		set_256color_table(TERM_GREEN, 35);
-		set_256color_table(TERM_BLUE, 27);
-		set_256color_table(TERM_UMBER, 130);
-		set_256color_table(TERM_L_DARK, 102);
-		set_256color_table(TERM_L_WHITE, 188);
-		set_256color_table(TERM_L_PURPLE, 201);
-		set_256color_table(TERM_YELLOW, 226);
-		set_256color_table(TERM_L_RED, 203);
-		set_256color_table(TERM_L_GREEN, 46);
-		set_256color_table(TERM_L_BLUE, 51);
-		set_256color_table(TERM_L_UMBER, 179);
-		set_256color_table(TERM_PURPLE, 127);
-		set_256color_table(TERM_VIOLET, 135);
-		set_256color_table(TERM_TEAL, 37);
-		set_256color_table(TERM_MUD, 101);
-		set_256color_table(TERM_L_YELLOW, 229);
-		set_256color_table(TERM_MAGENTA, 199);
-		set_256color_table(TERM_L_TEAL, 86);
-		set_256color_table(TERM_L_VIOLET, 183);
-		set_256color_table(TERM_L_PINK, 217);
-		set_256color_table(TERM_MUSTARD, 184);
-		set_256color_table(TERM_BLUE_SLATE, 152);
-		set_256color_table(TERM_DEEP_L_BLUE, 39);
-	}
-	/* TODO: figure out how 88-color terminals (e.g. urxvt) work */
-
 #endif
 
-	/* Success */
-	return (0);
+	return 0;
 }
 
 
 /*
  * Handle a "special request"
  */
-static errr Term_xtra_gcu(int n, int v)
-{
+static errr Term_xtra_gcu(int n, int v) {
 	term_data *td = (term_data *)(Term->data);
 
 	/* Analyze the request */
-	switch (n)
-	{
+	switch (n) {
 		/* Clear screen */
-		case TERM_XTRA_CLEAR:
-		touchwin(td->win);
-		(void)wclear(td->win);
-		return (0);
+		case TERM_XTRA_CLEAR: touchwin(td->win); wclear(td->win); return 0;
 
 		/* Make a noise */
-		case TERM_XTRA_NOISE:
-		if (write(1, "\007", 1) != 1)
-			return (1);
-		return (0);
+		case TERM_XTRA_NOISE: write(1, "\007", 1); return 0;
 
 		/* Flush the Curses buffer */
-		case TERM_XTRA_FRESH:
-		(void)wrefresh(td->win);
-		return (0);
+		case TERM_XTRA_FRESH: wrefresh(td->win); return 0;
 
 #ifdef USE_CURS_SET
-
 		/* Change the cursor visibility */
-		case TERM_XTRA_SHAPE:
-		curs_set(v);
-		return (0);
-
+		case TERM_XTRA_SHAPE: curs_set(v); return 0;
 #endif
 
 		/* Suspend/Resume curses */
-		case TERM_XTRA_ALIVE:
-		return (Term_xtra_gcu_alive(v));
+		case TERM_XTRA_ALIVE: return Term_xtra_gcu_alive(v);
 
 		/* Process events */
-		case TERM_XTRA_EVENT:
-		return (Term_xtra_gcu_event(v));
+		case TERM_XTRA_EVENT: return Term_xtra_gcu_event(v);
 
 		/* Flush events */
-		case TERM_XTRA_FLUSH:
-		while (!Term_xtra_gcu_event(FALSE));
-		return (0);
+		case TERM_XTRA_FLUSH: while (!Term_xtra_gcu_event(FALSE)); return 0;
 
 		/* Delay */
-		case TERM_XTRA_DELAY:
-		if (v > 0)
-			usleep(1000 * v);
-		return (0);
+		case TERM_XTRA_DELAY: if (v > 0) usleep(1000 * v); return 0;
 
 		/* React to events */
-		case TERM_XTRA_REACT:
-		Term_xtra_gcu_react();
-		return (0);
+		case TERM_XTRA_REACT: Term_xtra_gcu_react(); return 0;
 	}
 
-	/* Unknown */
-	return (1);
+	/* Unknown event */
+	return 1;
 }
 
 
 /*
  * Actually MOVE the hardware cursor
  */
-static errr Term_curs_gcu(int x, int y)
-{
+static errr Term_curs_gcu(int x, int y) {
 	term_data *td = (term_data *)(Term->data);
-
-	/* Literally move the cursor */
 	wmove(td->win, y, x);
-
-	/* Success */
-	return (0);
+	return 0;
 }
 
 
@@ -816,11 +683,9 @@ static errr Term_curs_gcu(int x, int y)
  * Erase a grid of space
  * Hack -- try to be "semi-efficient".
  */
-static errr Term_wipe_gcu(int x, int y, int n)
-{
+static errr Term_wipe_gcu(int x, int y, int n) {
 	term_data *td = (term_data *)(Term->data);
 
-	/* Place cursor */
 	wmove(td->win, y, x);
 
 	if (x + n >= td->t.wid)
@@ -830,8 +695,7 @@ static errr Term_wipe_gcu(int x, int y, int n)
 		/* Clear some characters */
 		whline(td->win, ' ', n);
 
-	/* Success */
-	return (0);
+	return 0;
 }
 
 /*
@@ -840,18 +704,30 @@ static errr Term_wipe_gcu(int x, int y, int n)
  * Given a position in the ISO Latin-1 character set, return the correct
  * character on this system. Currently
  */
- static byte Term_xchar_gcu(byte c)
-{
-	return (c);
+ static byte Term_xchar_gcu(byte c) {
+	return c;
 }
 
 
+/* Hack - replace non-ASCII characters to
+ * avoid display glitches in selectors.
+ *
+ * Note that we do this after the ACS mapping,
+ * because the display glitches we are avoiding
+ * are in curses itself.
+ */
+char filter_char(char c) {
+	if (c < ' ' || c >= 127)
+		return '?';
+	else
+		return c;
+}
+
 
 /*
  * Place some text on the screen using an attribute
  */
-static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
-{
+static errr Term_text_gcu(int x, int y, int n, byte a, const char *s) {
 	term_data *td = (term_data *)(Term->data);
 
 #ifdef A_COLOR
@@ -864,25 +740,15 @@ static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
 
 	/* Write to screen */
 	while (n--) {
-		unsigned int c = (unsigned char) *(s++);
+		unsigned char c = *(s++);
 
-#ifdef A_ALTCHARSET
-		/* Map high-bit characters down using the $TERM-specific
-		 * alternate character set. */
-		if (c < 32) c = acs_table[c];
-#endif
-
-		if ((c & 255) < ' ' || (c & 255) == 127)
-			/* Hack - replace non-ASCII characters to
-			 * avoid display glitches in selectors.
-			 *
-			 * Note that we do this after the ACS mapping,
-			 * because the display glitches we are avoiding
-			 * are in curses itself.
-			 */
-			waddch(td->win, '?');
-		else
-			waddch(td->win, c);
+		if (c < 32) {
+			wattron(td->win, ctrl_attr[c]);
+			waddch(td->win, filter_char(ctrl_char[c]));
+			wattroff(td->win, ctrl_attr[c]);
+		} else {
+			waddch(td->win, filter_char(c));
+		}
 	}
 
 #if defined(A_COLOR)
@@ -900,8 +766,7 @@ static errr Term_text_gcu(int x, int y, int n, byte a, cptr s)
  *
  * Assumes legal arguments.
  */
-static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
-{
+static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x) {
 	term *t = &td->t;
 
 	/* Create new window */
@@ -909,10 +774,7 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 
 	/* Check for failure */
 	if (!td->win)
-	{
-		/* Error */
 		quit("Failed to setup curses window.");
-	}
 
 	/* Initialize the term */
 	term_init(t, cols, rows, 256);
@@ -951,17 +813,10 @@ static errr term_data_init_gcu(term_data *td, int rows, int cols, int y, int x)
 	return (0);
 }
 
-
-static void hook_quit(cptr str)
-{
-	/* Unused */
-	(void)str;
-
-	/* Exit curses */
+static void hook_quit(const char *str) {
 	endwin();
 }
 
-
 /*
  * Prepare "curses" for use by the file "z-term.c"
  *
@@ -970,36 +825,35 @@ static void hook_quit(cptr str)
  *
  * Someone should really check the semantics of "initscr()"
  */
-errr init_gcu(int argc, char **argv)
-{
+errr init_gcu(int argc, char **argv) {
 	int i;
 	int rows, cols, y, x;
 	int next_win = 0;
+	bool graphics = TRUE;
 
 	/* Initialize info about terminal capabilities */
 	termtype = getenv("TERM");
 	loaded_terminfo = termtype && tgetent(0, termtype) == 1;
 
-	/* Let's learn about our terminal */
-	use_alt_charset = loaded_terminfo && tgetstr("acs_chars", NULL);
-
 	/* Parse args */
-	for (i = 1; i < argc; i++)
-	{
+	for (i = 1; i < argc; i++) {
 		if (prefix(argv[i], "-b"))
 			use_big_screen = TRUE;
-
-#ifdef A_ALTCHARSET
+		else if (prefix(argv[i], "-B"))
+			bold_extended = TRUE;
 		else if (prefix(argv[i], "-a"))
-			use_alt_charset = 0;
-		else if (prefix(argv[i], "-g"))
-			use_alt_charset = 1;
-#endif
-
+			graphics = FALSE;
 		else
 			plog_fmt("Ignoring option: %s", argv[i]);
 	}
 
+	if (graphics) {
+		ctrl_char[CTRL_WALL] = ' ';
+		ctrl_attr[CTRL_ORE] = A_REVERSE;
+		ctrl_attr[CTRL_WALL] = A_REVERSE;
+		ctrl_attr[CTRL_ROCK] = A_REVERSE;
+	}
+
 	/* Extract the normal keymap */
 	keymap_norm_prepare();
 
@@ -1016,62 +870,21 @@ errr init_gcu(int argc, char **argv)
 	quit_aux = hook_quit;
 
 	/* Require standard size screen */
-	if ((LINES < 24) || (COLS < 80))
-	{
+	if (LINES < 24 || COLS < 80)
 		quit("Angband needs at least an 80x24 'curses' screen");
-	}
-
 
 #ifdef A_COLOR
-
-	/*** Init the Color-pairs and set up a translation table ***/
-
 	/* Do we have color, and enough color, available? */
 	can_use_color = ((start_color() != ERR) && has_colors() &&
 					 (COLORS >= 8) && (COLOR_PAIRS >= 8));
 
 #ifdef HAVE_USE_DEFAULT_COLORS
-
 	/* Should we use curses' "default color" */
-	if (use_default_colors() == OK)
-		bg_color = -1;
-
+	if (use_default_colors() == OK) bg_color = -1;
 #endif
 
-#ifdef HAVE_CAN_CHANGE_COLOR
-
-	/* Can we change colors? */
-	can_fix_color = (can_use_color && can_change_color() &&
-	                 orig_colors && (COLORS >= 16) && (COLOR_PAIRS > 8));
-
-#endif
-
-	/* Attempt to use customized colors */
-	if (can_fix_color)
-	{
-		/* Prepare the color pairs */
-		for (i = 0; i < (BASIC_COLORS / 2); i++)
-		{
-			/* Reset the color */
-			if (init_pair(i + 1, i, bg_color) == ERR)
-			{
-				quit("Color pair init failed");
-			}
-
-			/* Set up the colormap */
-			colortable[i] = (COLOR_PAIR(i + 1) | A_NORMAL);
-			colortable[i + (BASIC_COLORS / 2)] = (COLOR_PAIR(i + 1) | A_BRIGHT);
-		}
-
-		/* Take account of "gamma correction" XXX XXX XXX */
-
-		/* Prepare the "Angband Colors" */
-		Term_xtra_gcu_react();
-	}
-
 	/* Attempt to use colors */
-	else if (can_use_color)
-	{
+	if (can_use_color) {
 		/* Prepare the color pairs */
 		/* PAIR_WHITE (pair 0) is *always* WHITE on BLACK */
 		init_pair(PAIR_RED, COLOR_RED, bg_color);
@@ -1086,7 +899,7 @@ errr init_gcu(int argc, char **argv)
 		colortable[TERM_DARK]     = (COLOR_PAIR(PAIR_BLACK));
 		colortable[TERM_WHITE]    = (COLOR_PAIR(PAIR_WHITE) | A_BRIGHT);
 		colortable[TERM_SLATE]    = (COLOR_PAIR(PAIR_WHITE));
-		colortable[TERM_ORANGE]   = (COLOR_PAIR(PAIR_RED) | A_BRIGHT);
+		colortable[TERM_ORANGE]   = (COLOR_PAIR(PAIR_YELLOW) | A_BRIGHT);
 		colortable[TERM_RED]      = (COLOR_PAIR(PAIR_RED));
 		colortable[TERM_GREEN]    = (COLOR_PAIR(PAIR_GREEN));
 		colortable[TERM_BLUE]     = (COLOR_PAIR(PAIR_BLUE));
@@ -1113,28 +926,8 @@ errr init_gcu(int argc, char **argv)
 		colortable[TERM_BLUE_SLATE]  = (COLOR_PAIR(PAIR_BLUE));
 		colortable[TERM_DEEP_L_BLUE] = (COLOR_PAIR(PAIR_BLUE));
 	}
-
 #endif
 
-#ifdef A_ALTCHARSET
-	/* Build a quick access table for the "alternate character set". */
-	if (use_alt_charset)
-	{
-		acs_table[1] = ACS_DIAMOND;    acs_table[16] = ACS_S1;
-		acs_table[2] = ACS_CKBOARD;    acs_table[18] = ACS_HLINE;
-		acs_table[7] = ACS_DEGREE;     acs_table[20] = ACS_S9;
-		acs_table[8] = ACS_PLMINUS;    acs_table[21] = ACS_LTEE;
-		acs_table[11] = ACS_LRCORNER;  acs_table[22] = ACS_RTEE;
-		acs_table[12] = ACS_URCORNER;  acs_table[23] = ACS_BTEE;
-		acs_table[13] = ACS_ULCORNER;  acs_table[24] = ACS_TTEE;
-		acs_table[14] = ACS_LLCORNER;  acs_table[25] = ACS_VLINE;
-		acs_table[15] = ACS_PLUS;      acs_table[31] = ACS_BULLET;
-	}
-#endif
-
-
-	/*** Low level preparation ***/
-
 	/* Paranoia -- Assume no waiting */
 	nodelay(stdscr, FALSE);
 
@@ -1150,8 +943,7 @@ errr init_gcu(int argc, char **argv)
 	keymap_game_prepare();
 
 	/*** Now prepare the term(s) ***/
-	for (i = 0; i < MAX_TERM_DATA; i++)
-	{
+	for (i = 0; i < MAX_TERM_DATA; i++) {
 		if (use_big_screen && i > 0) break;
 
 		/* Get the terminal dimensions; if the user asked for a big screen
@@ -1182,5 +974,4 @@ errr init_gcu(int argc, char **argv)
 	return (0);
 }
 
-
 #endif /* USE_GCU */
