@@ -1,4 +1,4 @@
-/*
+/**
  * File: obj-info.c
  * Purpose: Object description code.
  *
@@ -21,8 +21,10 @@
 #include "attack.h"
 #include "effects.h"
 #include "cmds.h"
-#include "tvalsval.h"
+#include "object/tvalsval.h"
 #include "z-textblock.h"
+#include "object/slays.h"
+#include "object/pval.h"
 
 /*
  * Describes a flag-name pair.
@@ -117,11 +119,8 @@ static const flag_type resist_flags[] =
 	{ OF_RES_FIRE,  "fire" },
 	{ OF_RES_COLD,  "cold" },
 	{ OF_RES_POIS,  "poison" },
-	{ OF_RES_FEAR,  "fear" },
 	{ OF_RES_LIGHT, "light" },
 	{ OF_RES_DARK,  "dark" },
-	{ OF_RES_BLIND, "blindness" },
-	{ OF_RES_CONFU, "confusion" },
 	{ OF_RES_SOUND, "sound" },
 	{ OF_RES_SHARD, "shards" },
 	{ OF_RES_NEXUS, "nexus"  },
@@ -130,6 +129,14 @@ static const flag_type resist_flags[] =
 	{ OF_RES_DISEN, "disenchantment" },
 };
 
+static const flag_type protect_flags[] =
+{
+	{ OF_RES_FEAR,  "fear" },
+	{ OF_RES_BLIND, "blindness" },
+	{ OF_RES_CONFU, "confusion" },
+	{ OF_RES_STUN,  "stunning" },
+};
+
 static const flag_type ignore_flags[] =
 {
 	{ OF_IGNORE_ACID, "acid" },
@@ -167,75 +174,6 @@ static const flag_type misc_flags[] =
 };
 
 
-/** Slays **/
-/*
- * Entries in this table should be in ascending order of multiplier, to
- * ensure that the highest one takes precedence
- * object flag, vulnerable flag, resist flag, multiplier, ranged verb,
- * melee verb, verb describing what the thing branded does when it is active,
- * description of affected creatures, brand
- */
-const slay_t slay_table[] =
-{
-	{ OF_SLAY_ANIMAL, RF_ANIMAL, FLAG_END,   2,
-	  "pierces",  "smite",   "glows",      "animals", NULL },
-	{ OF_SLAY_EVIL,   RF_EVIL,   FLAG_END,   2,
-	  "pierces",  "smite",   "glows",      "evil creatures", NULL },
-	{ OF_SLAY_UNDEAD, RF_UNDEAD, FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "undead", NULL },
-	{ OF_SLAY_DEMON,  RF_DEMON,  FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "demons", NULL },
-	{ OF_SLAY_ORC,    RF_ORC,    FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "orcs", NULL },
-	{ OF_SLAY_TROLL,  RF_TROLL,  FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "trolls", NULL },
-	{ OF_SLAY_GIANT,  RF_GIANT,  FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "giants", NULL },
-	{ OF_SLAY_DRAGON, RF_DRAGON, FLAG_END,   3,
-	  "pierces",  "smite",   "glows",      "dragons", NULL },
-	{ OF_BRAND_ACID,  FLAG_END,  RF_IM_ACID, 3,
-	  "corrodes", "corrode", "spits",      "creatures not resistant to acid", "acid" },
-	{ OF_BRAND_ELEC,  FLAG_END,  RF_IM_ELEC, 3,
-	  "zaps",     "zap",     "crackles",   "creatures not resistant to electricity", "lightning" },
-	{ OF_BRAND_FIRE,  FLAG_END,  RF_IM_FIRE, 3,
-	  "burns",    "burn",    "flares",     "creatures not resistant to fire", "flames" },
-	{ OF_BRAND_COLD,  FLAG_END,  RF_IM_COLD, 3,
-	  "freezes" , "freeze",  "grows cold", "creatures not resistant to cold", "frost" },
-	{ OF_BRAND_POIS,  FLAG_END,  RF_IM_POIS, 3,
-	  "poisons",  "poison",  "seethes",    "creatures not resistant to poison", "venom" },
-	{ OF_KILL_DRAGON, RF_DRAGON, FLAG_END,   5, 
-	 "deeply pierces",  "fiercely smite", "glows brightly", "dragons", NULL },
-	{ OF_KILL_DEMON,  RF_DEMON,  FLAG_END,   5,
-	  "deeply pierces", "fiercely smite", "glows brightly", "demons", NULL },
-	{ OF_KILL_UNDEAD, RF_UNDEAD, FLAG_END,   5,
-	  "deeply pierces", "fiercely smite", "glows brightly", "undead", NULL },
-	{ FLAG_END,       FLAG_END,  FLAG_END,   0, NULL, NULL, NULL, NULL, NULL }
-};
-
-/*
- * Slays which are in some sense duplicates. *Slay* dragon supercedes slay
- * dragon, for example.
- */
-const struct {
-	u16b minor;
-	u16b major;
-} slay_dups[] =
-{
-	{ OF_SLAY_DRAGON, OF_KILL_DRAGON },
-	{ OF_SLAY_DEMON, OF_KILL_DEMON },
-	{ OF_SLAY_UNDEAD, OF_KILL_UNDEAD },
-};
-
-
-/*
- * Helper function to externalise N_ELEMENTS(slay_table), which itself is not
- * available outside this compilation unit
- */
-size_t num_slays(void)
-{
-	return N_ELEMENTS(slay_table);
-}
-
 /*** Code that makes use of the data tables ***/
 
 /*
@@ -261,35 +199,40 @@ static bool describe_curses(textblock *tb, const object_type *o_ptr,
  * Describe stat modifications.
  */
 static bool describe_stats(textblock *tb, const object_type *o_ptr,
-		const bitflag flags[OF_SIZE], oinfo_detail_t mode)
+		bitflag flags[MAX_PVALS][OF_SIZE], oinfo_detail_t mode)
 {
-	cptr descs[N_ELEMENTS(pval_flags)];
-	size_t count;
+	const char *descs[N_ELEMENTS(pval_flags)];
+	size_t count, i;
 	bool full = mode & OINFO_FULL;
 	bool dummy = mode & OINFO_DUMMY;
+	bool search = FALSE;
 
-	if (!o_ptr->pval && !dummy)
+	if (!o_ptr->num_pvals && !dummy)
 		return FALSE;
 
-	count = info_collect(tb, pval_flags, N_ELEMENTS(pval_flags), flags, descs);
+	for (i = 0; i < o_ptr->num_pvals; i++) {
+		count = info_collect(tb, pval_flags, N_ELEMENTS(pval_flags), flags[i], descs);
 
-	if (count)
-	{
-		if ((object_pval_is_visible(o_ptr) || full) && !dummy)
-			textblock_append_c(tb, (o_ptr->pval > 0) ? TERM_L_GREEN : TERM_RED,
-					"%+i ", o_ptr->pval);
-		else
-			textblock_append(tb, "Affects your ");
+		if (count)
+		{
+			if ((object_this_pval_is_visible(o_ptr, i) || full) && !dummy)
+				textblock_append_c(tb, (o_ptr->pval[i] > 0) ? TERM_L_GREEN : TERM_RED,
+					"%+i ", o_ptr->pval[i]);
+			else
+				textblock_append(tb, "Affects your ");
 
-		info_out_list(tb, descs, count);
+			info_out_list(tb, descs, count);
+		}
+		if (of_has(flags[i], OF_SEARCH))
+			search = TRUE;
 	}
 
-	if (of_has(flags, OF_SEARCH))
+	if (search)
 	{
-		if ((object_pval_is_visible(o_ptr) || full) && !dummy)
+		if ((object_this_pval_is_visible(o_ptr, which_pval(o_ptr, OF_SEARCH)) || full) && !dummy)
 		{
-			textblock_append_c(tb, (o_ptr->pval > 0) ? TERM_L_GREEN : TERM_RED,
-				"%+i%% ", o_ptr->pval * 5);
+			textblock_append_c(tb, (o_ptr->pval[which_pval(o_ptr, OF_SEARCH)] > 0) ? TERM_L_GREEN : TERM_RED,
+				"%+i%% ", o_ptr->pval[which_pval(o_ptr, OF_SEARCH)] * 5);
 			textblock_append(tb, "to searching.\n");
 		}
 		else if (count)
@@ -309,6 +252,7 @@ static bool describe_immune(textblock *tb, const bitflag flags[OF_SIZE])
 {
 	const char *i_descs[N_ELEMENTS(immunity_flags)];
 	const char *r_descs[N_ELEMENTS(resist_flags)];
+	const char *p_descs[N_ELEMENTS(protect_flags)];
 	const char *v_descs[N_ELEMENTS(vuln_flags)];
 	size_t count;
 
@@ -334,6 +278,16 @@ static bool describe_immune(textblock *tb, const bitflag flags[OF_SIZE])
 		prev = TRUE;
 	}
 
+	/* Protections */
+	count = info_collect(tb, protect_flags, N_ELEMENTS(protect_flags),
+			flags, p_descs);
+	if (count)
+	{
+		textblock_append(tb, "Provides protection from ");
+		info_out_list(tb, p_descs, count);
+		prev = TRUE;
+	}
+
 	/* Vulnerabilities */
 	count = info_collect(tb, vuln_flags, N_ELEMENTS(vuln_flags),
 			flags, v_descs);
@@ -417,22 +371,14 @@ static bool describe_slays(textblock *tb, const bitflag flags[OF_SIZE],
 		int tval)
 {
 	bool printed = FALSE;
-
-	const char *slay_descs[N_ELEMENTS(slay_table)];
-	const char *kill_descs[N_ELEMENTS(slay_table)];
-	const char *brand_descs[N_ELEMENTS(slay_table)];
-	const slay_t *s_ptr;
+	const char *slay_descs[SL_MAX] = { 0 };
 	bitflag slay_mask[OF_SIZE], kill_mask[OF_SIZE], brand_mask[OF_SIZE];
-
-	size_t x = 0;
-	size_t y = 0;
-	size_t z = 0;
-
+	size_t count;
 	bool fulldesc;
 
-	flags_init(slay_mask, OF_SIZE, OF_SLAY_MASK, FLAG_END);
-	flags_init(kill_mask, OF_SIZE, OF_KILL_MASK, FLAG_END);
-	flags_init(brand_mask, OF_SIZE, OF_BRAND_MASK, FLAG_END);
+	create_mask(slay_mask, FALSE, OFT_SLAY, OFT_MAX);
+	create_mask(kill_mask, FALSE, OFT_KILL, OFT_MAX);
+	create_mask(brand_mask, FALSE, OFT_BRAND, OFT_MAX);
 
 	if (tval == TV_SWORD || tval == TV_HAFTED || tval == TV_POLEARM ||
 			tval == TV_DIGGING || tval == TV_BOW || tval == TV_SHOT ||
@@ -441,88 +387,45 @@ static bool describe_slays(textblock *tb, const bitflag flags[OF_SIZE],
 	else
 		fulldesc = TRUE;
 
-	for (s_ptr = slay_table; s_ptr->slay_flag; s_ptr++)
-	{
-		if (!of_has(flags, s_ptr->slay_flag)) continue;
-
-		if (of_has(slay_mask, s_ptr->slay_flag))
-			slay_descs[x++] = s_ptr->desc;
-		else if (of_has(kill_mask, s_ptr->slay_flag))
-			kill_descs[y++] = s_ptr->desc;
-		else if (of_has(brand_mask, s_ptr->slay_flag))
-			brand_descs[z++] = s_ptr->brand;
-	}
-
 	/* Slays */
-	if (x)
+	count = list_slays(flags, slay_mask, slay_descs, NULL, NULL, TRUE);
+	if (count)
 	{
 		if (fulldesc)
 			textblock_append(tb, "It causes your melee attacks to slay ");
 		else
 			textblock_append(tb, "Slays ");
-		info_out_list(tb, slay_descs, x);
+		info_out_list(tb, slay_descs, count);
 		printed = TRUE;
 	}
 
 	/* Kills */
-	if (y)
+	count = list_slays(flags, kill_mask, slay_descs, NULL, NULL, TRUE);
+	if (count)
 	{
 		if (fulldesc)
 			textblock_append(tb, "It causes your melee attacks to *slay* ");
 		else
 			textblock_append(tb, "*Slays* ");
-		info_out_list(tb, kill_descs, y);
+		info_out_list(tb, slay_descs, count);
 		printed = TRUE;
 	}
 
 	/* Brands */
-	if (z)
+	count = list_slays(flags, brand_mask, NULL, slay_descs, NULL, TRUE);
+	if (count)
 	{
 		if (fulldesc)
 			textblock_append(tb, "It brands your melee attacks with ");
 		else
 			textblock_append(tb, "Branded with ");
-		info_out_list(tb, brand_descs, z);
+		info_out_list(tb, slay_descs, count);
 		printed = TRUE;
 	}
 
 	return printed;
 }
 
-
-
-/*
- * list[] and mult[] must be > 16 in size
- */
-static int collect_slays(const char *desc[], int mult[], bitflag *flags)
-{
-	int cnt = 0;
-	u16b i;
-	const slay_t *s_ptr;
-
-	/* Remove "duplicate" flags e.g. *slay* and slay the same
-	 * monster type
-	 */
-	for (i = 0; i < N_ELEMENTS(slay_dups); i++) {
-		if (of_has(flags, slay_dups[i].minor) &&
-				of_has(flags, slay_dups[i].major)) {
-			of_off(flags, slay_dups[i].minor);
-		}
-	}
-
-	/* Collect slays */
-	for (s_ptr = slay_table; s_ptr->slay_flag; s_ptr++) {
-		if (of_has(flags, s_ptr->slay_flag)) {
-			mult[cnt] = s_ptr->mult;
-			desc[cnt++] = s_ptr->desc;
-		}
-	}
-
-	return cnt;
-}
-
-
-
 /*
  * Account for criticals in the calculation of melee prowess
  *
@@ -594,24 +497,26 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 {
 	bool full = mode & OINFO_FULL;
 
-	const char *desc[16];
+	const char *desc[SL_MAX] = { 0 };
 	int i;
-	int mult[16];
+	int mult[SL_MAX];
 	int cnt, dam, total_dam, plus = 0;
 	int xtra_postcrit = 0, xtra_precrit = 0;
 	int crit_mult, crit_div, crit_add;
 	int str_plus, dex_plus, old_blows = 0, new_blows, extra_blows;
-	int str_done = -1;
-	object_type *j_ptr = &p_ptr->inventory[INVEN_BOW];
+	int str_faster = -1, str_done = -1;
+	object_type *bow = &p_ptr->inventory[INVEN_BOW];
 
-	bitflag f[OF_SIZE];
-	bitflag tmp_f[OF_SIZE];
+	bitflag f[OF_SIZE], tmp_f[OF_SIZE], mask[OF_SIZE];
 
 	bool weapon = (wield_slot(o_ptr) == INVEN_WIELD);
 	bool ammo   = (p_ptr->state.ammo_tval == o_ptr->tval) &&
-	              (j_ptr->k_idx);
+	              (bow->kind);
 	int multiplier = 1;
 
+	/* Create the "all slays" mask */
+	create_mask(mask, FALSE, OFT_SLAY, OFT_KILL, OFT_BRAND, OFT_MAX);
+
 	/* Abort if we've nothing to say */
 	if (mode & OINFO_DUMMY) return FALSE;
 
@@ -627,10 +532,11 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 		return TRUE;
 	}
 
-	if (full)
+	if (full) {
 		object_flags(o_ptr, f);
-	else
+	} else {
 		object_flags_known(o_ptr, f);
+	}
 
 	textblock_append_c(tb, TERM_L_WHITE, "Combat info:\n");
 
@@ -672,26 +578,29 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 			textblock_append_c(tb, TERM_L_RED, "You are too weak to use this weapon.\n");
 
 		textblock_append_c(tb, TERM_L_GREEN, "%d.%d ",
-				state.num_blow / 100, (state.num_blow / 10) % 10);
+				state.num_blows / 100, (state.num_blows / 10) % 10);
 		textblock_append(tb, "blow%s/round.\n",
-				(state.num_blow > 100) ? "s" : "");
+				(state.num_blows > 100) ? "s" : "");
 
 		/* Check to see if extra STR or DEX would yield extra blows */
-		old_blows = state.num_blow;
+		old_blows = state.num_blows;
 		extra_blows = 0;
 
 		/* First we need to look for extra blows on other items, as
 		 * state does not track these */
 		for (i = INVEN_BOW; i < INVEN_TOTAL; i++)
 		{
+			if (!p_ptr->inventory[i].kind)
+				continue;
 			object_flags_known(&p_ptr->inventory[i], tmp_f);
 
 			if (of_has(tmp_f, OF_BLOWS))
-				extra_blows += p_ptr->inventory[i].pval;
+				extra_blows += p_ptr->inventory[i].pval[which_pval(&p_ptr->inventory[i], OF_BLOWS)];
 		}
 
 		/* Then we add blows from the weapon being examined */
-		if (of_has(f, OF_BLOWS)) extra_blows += o_ptr->pval;
+		if (of_has(f, OF_BLOWS))
+			extra_blows += o_ptr->pval[which_pval(o_ptr, OF_BLOWS)];
 
 		/* Then we check for extra "real" blows */
 		for (dex_plus = 0; dex_plus < dex_plus_bound; dex_plus++)
@@ -705,11 +614,11 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 				/* Test to make sure that this extra blow is a
 				 * new str/dex combination, not a repeat
 				 */
-				if (new_blows > old_blows &&
+				if ((new_blows - new_blows % 10) > (old_blows - old_blows % 10) &&
 					(str_plus < str_done ||
 					str_done == -1))
 				{
-					textblock_append(tb, "With an additional %d strength and %d dex you would get %d.%d blows\n",
+					textblock_append(tb, "With +%d STR and +%d DEX you would get %d.%d blows\n",
 						str_plus, dex_plus, (new_blows / 100),
 						(new_blows / 10) % 10);
 					state.stat_ind[A_STR] -= str_plus;
@@ -718,6 +627,24 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 					break;
 				}
 
+				/* If the combination doesn't increment
+				 * the displayed blows number, it might still
+				 * take a little less energy
+				 */
+				if (new_blows > old_blows &&
+					(str_plus < str_faster ||
+					str_faster == -1) &&
+					(str_plus < str_done ||
+					str_done == -1))
+				{
+					textblock_append(tb, "With +%d STR and +%d DEX you would attack a bit faster\n",
+						str_plus, dex_plus);
+					state.stat_ind[A_STR] -= str_plus;
+					state.stat_ind[A_DEX] -= dex_plus;
+					str_faster = str_plus;
+					continue;
+				}
+
 				state.stat_ind[A_STR] -= str_plus;
 				state.stat_ind[A_DEX] -= dex_plus;
 			}
@@ -738,13 +665,13 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 
 		if (object_attack_plusses_are_visible(o_ptr))
 			dam += (o_ptr->to_d * 10);
-		if (object_attack_plusses_are_visible(j_ptr))
-			dam += (j_ptr->to_d * 10);
+		if (object_attack_plusses_are_visible(bow))
+			dam += (bow->to_d * 10);
 
 		/* Apply brands/slays from the shooter to the ammo, but only if known
 		 * Note that this is not dependent on mode, so that viewing shop-held
 		 * ammo (fully known) does not leak information about launcher */
-		object_flags_known(j_ptr, tmp_f);
+		object_flags_known(bow, tmp_f);
 		of_union(f, tmp_f);
 
 		textblock_append(tb, "Hits targets up to ");
@@ -760,9 +687,11 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 
 		for (i = INVEN_LEFT; i < INVEN_TOTAL; i++)
 		{
+			if (!p_ptr->inventory[i].kind)
+				continue;
 			object_flags_known(&p_ptr->inventory[i], tmp_f);
 
-			flags_mask(tmp_f, OF_SIZE, OF_ALL_SLAY_MASK, FLAG_END);
+			of_inter(tmp_f, mask); /* strip out non-slays */
 
 			if (of_union(f, tmp_f))
 				nonweap = TRUE;
@@ -776,7 +705,7 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 
 	if (ammo) multiplier = p_ptr->state.ammo_mult;
 
-	cnt = collect_slays(desc, mult, f);
+	cnt = list_slays(f, mask, desc, NULL, mult, TRUE);
 	for (i = 0; i < cnt; i++)
 	{
 		int melee_adj_mult = ammo ? 0 : 1; /* ammo mult adds fully, melee mult is times 1, so adds 1 less */
@@ -787,7 +716,7 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 		if (weapon) 
 			total_dam = (total_dam * old_blows) / 100;
 		else
-			total_dam *= p_ptr->state.num_fire;
+			total_dam *= p_ptr->state.num_shots;
 		
 
 		if (total_dam <= 0)
@@ -811,7 +740,7 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 	if (weapon)
 		total_dam = (total_dam * old_blows) / 100;
 	else
-		total_dam *= p_ptr->state.num_fire;
+		total_dam *= p_ptr->state.num_shots;
 
 	if (total_dam <= 0)
 		textblock_append_c(tb, TERM_L_RED, "%d", 0);
@@ -831,7 +760,8 @@ static bool describe_combat(textblock *tb, const object_type *o_ptr,
 	/* Add breakage chance */
 	if (ammo)
 	{
-		textblock_append_c(tb, TERM_L_GREEN, "%d%%", breakage_chance(o_ptr));
+		int chance = breakage_chance(o_ptr, TRUE);
+		textblock_append_c(tb, TERM_L_GREEN, "%d%%", chance);
 		textblock_append(tb, " chance of breaking upon contact.\n");
 	}
 
@@ -932,7 +862,7 @@ static bool describe_food(textblock *tb, const object_type *o_ptr,
 {
 	/* Describe boring bits */
 	if ((o_ptr->tval == TV_FOOD || o_ptr->tval == TV_POTION) &&
-		o_ptr->pval)
+		o_ptr->pval[DEFAULT_PVAL])
 	{
 		/* Sometimes adjust for player speed */
 		int multiplier = extract_energy[p_ptr->state.speed];
@@ -940,7 +870,7 @@ static bool describe_food(textblock *tb, const object_type *o_ptr,
 
 		if (object_is_known(o_ptr) || full) {
 			textblock_append(tb, "Nourishes for around ");
-			textblock_append_c(tb, TERM_L_GREEN, "%d", (o_ptr->pval / 2) *
+			textblock_append_c(tb, TERM_L_GREEN, "%d", (o_ptr->pval[DEFAULT_PVAL] / 2) *
 				multiplier / 10);
 			textblock_append(tb, " turns.\n");
 		} else {
@@ -962,7 +892,7 @@ static bool describe_light(textblock *tb, const object_type *o_ptr,
 {
 	int rad = 0;
 
-	bool artifact = artifact_p(o_ptr);
+	bool artifact = o_ptr->artifact ? TRUE : FALSE;
 	bool no_fuel = of_has(flags, OF_NO_FUEL) ? TRUE : FALSE;
 	bool is_light = (o_ptr->tval == TV_LIGHT) ? TRUE : FALSE;
 
@@ -1008,20 +938,17 @@ static bool describe_light(textblock *tb, const object_type *o_ptr,
 static bool describe_effect(textblock *tb, const object_type *o_ptr, bool full,
 		bool only_artifacts, bool subjective)
 {
-	const object_kind *k_ptr = &k_info[o_ptr->k_idx];
 	const char *desc;
 	random_value timeout = {0, 0, 0, 0};
 
 	int effect = 0, fail;
 
-	if (o_ptr->name1)
+	if (o_ptr->artifact)
 	{
-		const artifact_type *a_ptr = &a_info[o_ptr->name1];
-
 		if (object_effect_is_known(o_ptr) || full)
 		{
-			effect = a_ptr->effect;
-			timeout = a_ptr->time;
+			effect = o_ptr->artifact->effect;
+			timeout = o_ptr->artifact->time;
 		}
 		else if (object_effect(o_ptr))
 		{
@@ -1036,12 +963,12 @@ static bool describe_effect(textblock *tb, const object_type *o_ptr, bool full,
 
 		if (object_effect_is_known(o_ptr) || full)
 		{
-			effect = k_ptr->effect;
-			timeout = k_ptr->time;
+			effect = o_ptr->kind->effect;
+			timeout = o_ptr->kind->time;
 		}
 		else if (object_effect(o_ptr) != 0)
 		{
-			if (effect_aim(k_ptr->effect))
+			if (effect_aim(o_ptr->kind->effect))
 				textblock_append(tb, "It can be aimed.\n");
 			else if (o_ptr->tval == TV_FOOD)
 				textblock_append(tb, "It can be eaten.\n");
@@ -1129,7 +1056,7 @@ static bool describe_effect(textblock *tb, const object_type *o_ptr, bool full,
 }
 
 
-bool describe_origin(textblock *tb, const object_type *o_ptr)
+static bool describe_origin(textblock *tb, const object_type *o_ptr)
 {
 	char origin_text[80];
 
@@ -1143,6 +1070,7 @@ bool describe_origin(textblock *tb, const object_type *o_ptr)
 	{
 		case ORIGIN_NONE:
 		case ORIGIN_MIXED:
+		case ORIGIN_STOLEN:
 			return FALSE;
 
 		case ORIGIN_BIRTH:
@@ -1159,9 +1087,51 @@ bool describe_origin(textblock *tb, const object_type *o_ptr)
 			         origin_text);
  			break;
 
+		case ORIGIN_PIT:
+			textblock_append(tb, "Found lying on the floor in a pit at %s.\n",
+			         origin_text);
+ 			break;
+
+		case ORIGIN_VAULT:
+			textblock_append(tb, "Found lying on the floor in a vault at %s.\n",
+			         origin_text);
+ 			break;
+
+		case ORIGIN_SPECIAL:
+			textblock_append(tb, "Found lying on the floor of a special room at %s.\n",
+			         origin_text);
+ 			break;
+
+		case ORIGIN_LABYRINTH:
+			textblock_append(tb, "Found lying on the floor of a labyrinth at %s.\n",
+			         origin_text);
+ 			break;
+
+		case ORIGIN_CAVERN:
+			textblock_append(tb, "Found lying on the floor of a cavern at %s.\n",
+			         origin_text);
+ 			break;
+
+		case ORIGIN_RUBBLE:
+			textblock_append(tb, "Found under some rubble at %s.\n",
+			         origin_text);
+ 			break;
+
 		case ORIGIN_DROP:
+		case ORIGIN_DROP_SPECIAL:
+		case ORIGIN_DROP_PIT:
+		case ORIGIN_DROP_VAULT:
+		case ORIGIN_DROP_SUMMON:
+		case ORIGIN_DROP_BREED:
+		case ORIGIN_DROP_POLY:
+		case ORIGIN_DROP_WIZARD:
 		{
-			const char *name = r_info[o_ptr->origin_xtra].name;
+			const char *name;
+
+			if (r_info[o_ptr->origin_xtra].ridx)
+				name = r_info[o_ptr->origin_xtra].name;
+			else
+				name = "monster lost to history";
 
 			textblock_append(tb, "Dropped by ");
 
@@ -1204,31 +1174,38 @@ bool describe_origin(textblock *tb, const object_type *o_ptr)
 	return TRUE;
 }
 
-static void describe_flavor_text(textblock *tb, const object_type *o_ptr)
+/*
+ * Print an item's flavour text.
+ *
+ * \param tb is the textblock to which we are adding.
+ * \param o_ptr is the object we are describing.
+ * \param ego is whether we're describing an ego template (as opposed to a
+ * real object)
+ */
+static void describe_flavor_text(textblock *tb, const object_type *o_ptr,
+	bool ego)
 {
 	/* Display the known artifact description */
-	if (!OPT(adult_randarts) && o_ptr->name1 &&
-			object_is_known(o_ptr) && a_info[o_ptr->name1].text)
-	{
-		textblock_append(tb, "%s\n\n", a_info[o_ptr->name1].text);
-	}
+	if (!OPT(birth_randarts) && o_ptr->artifact &&
+			object_is_known(o_ptr) && o_ptr->artifact->text)
+		textblock_append(tb, "%s\n\n", o_ptr->artifact->text);
 
 	/* Display the known object description */
-	else if (object_flavor_is_aware(o_ptr) || object_is_known(o_ptr))
+	else if (object_flavor_is_aware(o_ptr) || object_is_known(o_ptr) || ego)
 	{
 		bool did_desc = FALSE;
 
-		if (k_info[o_ptr->k_idx].text)
+		if (!ego && o_ptr->kind->text)
 		{
-			textblock_append(tb, "%s", k_info[o_ptr->k_idx].text);
+			textblock_append(tb, "%s", o_ptr->kind->text);
 			did_desc = TRUE;
 		}
 
 		/* Display an additional ego-item description */
-		if (object_ego_is_visible(o_ptr) && e_info[o_ptr->name2].text)
+		if ((ego || object_ego_is_visible(o_ptr)) && o_ptr->ego->text)
 		{
 			if (did_desc) textblock_append(tb, "  ");
-			textblock_append(tb, "%s\n\n", e_info[o_ptr->name2].text);
+			textblock_append(tb, "%s\n\n", o_ptr->ego->text);
 		}
 		else if (did_desc)
 		{
@@ -1238,11 +1215,9 @@ static void describe_flavor_text(textblock *tb, const object_type *o_ptr)
 }
 
 
-bool describe_ego(textblock *tb, const object_type *o_ptr)
+static bool describe_ego(textblock *tb, const struct ego_item *ego)
 {
-	struct ego_item *ego = &e_info[o_ptr->name2];
-
-	if (ego->name && ego->xtra)
+	if (ego && ego->xtra)
 	{
 		const char *xtra[] = { "sustain", "higher resistance", "ability" };
 		textblock_append(tb, "It provides one random %s.  ",
@@ -1261,6 +1236,7 @@ bool describe_ego(textblock *tb, const object_type *o_ptr)
 static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
 {
 	bitflag flags[OF_SIZE];
+	bitflag pval_flags[MAX_PVALS][OF_SIZE];
 	bool something = FALSE;
 	bool known = object_is_known(o_ptr);
 
@@ -1272,13 +1248,16 @@ static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
 	textblock *tb = textblock_new();
 
 	/* Grab the object flags */
-	if (full)
+	if (full) {
 		object_flags(o_ptr, flags);
-	else
+		object_pval_flags(o_ptr, pval_flags);
+	} else {
 		object_flags_known(o_ptr, flags);
+		object_pval_flags_known(o_ptr, pval_flags);
+	}
 
 	if (subjective) describe_origin(tb, o_ptr);
-	if (!terse) describe_flavor_text(tb, o_ptr);
+	if (!terse) describe_flavor_text(tb, o_ptr, ego);
 
 	if (!full && !known)
 	{
@@ -1287,16 +1266,16 @@ static textblock *object_info_out(const object_type *o_ptr, oinfo_detail_t mode)
 	}
 
 	if (describe_curses(tb, o_ptr, flags)) something = TRUE;
-	if (describe_stats(tb, o_ptr, flags, mode)) something = TRUE;
+	if (describe_stats(tb, o_ptr, pval_flags, mode)) something = TRUE;
 	if (describe_slays(tb, flags, o_ptr->tval)) something = TRUE;
 	if (describe_immune(tb, flags)) something = TRUE;
 	if (describe_ignores(tb, flags)) something = TRUE;
 	if (describe_sustains(tb, flags)) something = TRUE;
 	if (describe_misc_magic(tb, flags)) something = TRUE;
-	if (ego && describe_ego(tb, o_ptr)) something = TRUE;
+	if (ego && describe_ego(tb, o_ptr->ego)) something = TRUE;
 	if (something) textblock_append(tb, "\n");
 
-	if (describe_effect(tb, o_ptr, full, terse, subjective))
+	if (!ego && describe_effect(tb, o_ptr, full, terse, subjective))
 	{
 		something = TRUE;
 		textblock_append(tb, "\n");
@@ -1354,8 +1333,8 @@ textblock *object_info_ego(struct ego_item *ego)
 	obj.kind = kind;
 	obj.tval = kind->tval;
 	obj.sval = kind->sval;
-	obj.name2 = ego->eidx;
-	of_union(obj.flags, ego->flags);
+	obj.ego = ego;
+	ego_apply_magic(&obj, 0);
 
 	return object_info_out(&obj, OINFO_FULL | OINFO_EGO | OINFO_DUMMY);
 }
