@@ -1,3 +1,7 @@
+/* player/util.c
+ * Copyright (c) 2011 The Angband Developers. See COPYING.
+ */
+
 #include "angband.h"
 #include "cave.h"
 #include "object/tvalsval.h"
@@ -53,21 +57,21 @@ s16b modify_stat_value(int value, int amount)
 /* Is the player capable of casting a spell? */
 bool player_can_cast(void)
 {
-	if (!cp_ptr->spell_book)
+	if (!p_ptr->class->spell_book)
 	{
-		msg_print("You cannot pray or produce magics.");
+		msg("You cannot pray or produce magics.");
 		return FALSE;
 	}
 
 	if (p_ptr->timed[TMD_BLIND] || no_light())
 	{
-		msg_print("You cannot see!");
+		msg("You cannot see!");
 		return FALSE;
 	}
 
 	if (p_ptr->timed[TMD_CONFUSED])
 	{
-		msg_print("You are too confused!");
+		msg("You are too confused!");
 		return FALSE;
 	}
 
@@ -82,8 +86,8 @@ bool player_can_study(void)
 
 	if (!p_ptr->new_spells)
 	{
-		cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
-		msg_format("You cannot learn any new %ss!", p);
+		const char *p = ((p_ptr->class->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+		msg("You cannot learn any new %ss!", p);
 		return FALSE;
 	}
 
@@ -95,25 +99,25 @@ bool player_can_read(void)
 {
 	if (p_ptr->timed[TMD_BLIND])
 	{
-		msg_print("You can't see anything.");
+		msg("You can't see anything.");
 		return FALSE;
 	}
 
 	if (no_light())
 	{
-		msg_print("You have no light to read by.");
+		msg("You have no light to read by.");
 		return FALSE;
 	}
 
 	if (p_ptr->timed[TMD_CONFUSED])
 	{
-		msg_print("You are too confused to read!");
+		msg("You are too confused to read!");
 		return FALSE;
 	}
 
 	if (p_ptr->timed[TMD_AMNESIA])
 	{
-		msg_print("You can't remember how to read!");
+		msg("You can't remember how to read!");
 		return FALSE;
 	}
 
@@ -128,9 +132,36 @@ bool player_can_fire(void)
 	/* Require a usable launcher */
 	if (!o_ptr->tval || !p_ptr->state.ammo_tval)
 	{
-		msg_print("You have nothing to fire with.");
+		msg("You have nothing to fire with.");
 		return FALSE;
 	}
 
 	return TRUE;
 }
+
+/*
+ * Apply confusion, if needed, to a direction
+ *
+ * Display a message and return TRUE if direction changes.
+ */
+bool player_confuse_dir(struct player *p, int *dp, bool too)
+{
+	int dir = *dp;
+
+	if (p->timed[TMD_CONFUSED])
+		if ((dir == 5) || (randint0(100) < 75))
+			/* Random direction */
+			dir = ddd[randint0(8)];
+
+	if (*dp != dir) {
+		if (too)
+			msg("You are too confused.");
+		else
+			msg("You are confused.");
+
+		*dp = dir;
+		return TRUE;
+	}
+
+	return FALSE;
+}
