@@ -17,7 +17,9 @@
  */
 
 #include "angband.h"
+#include "cave.h"
 #include "history.h"
+#include "monster/mon-make.h"
 #include "monster/monster.h"
 #include "option.h"
 #include "savefile.h"
@@ -28,12 +30,12 @@
  */
 static void wr_item(const object_type *o_ptr)
 {
-	size_t i;
+	size_t i, j;
 
 	wr_u16b(0xffff);
 	wr_byte(ITEM_VERSION);
 
-	wr_s16b(o_ptr->k_idx);
+	wr_s16b(0);
 
 	/* Location */
 	wr_byte(o_ptr->iy);
@@ -41,15 +43,22 @@ static void wr_item(const object_type *o_ptr)
 
 	wr_byte(o_ptr->tval);
 	wr_byte(o_ptr->sval);
-	wr_s16b(o_ptr->pval);
+
+        for (i = 0; i < MAX_PVALS; i++) {
+		wr_s16b(o_ptr->pval[i]);
+        }
+        wr_byte(o_ptr->num_pvals);
 
 	wr_byte(0);
 
 	wr_byte(o_ptr->number);
 	wr_s16b(o_ptr->weight);
 
-	wr_byte(o_ptr->name1);
-	wr_byte(o_ptr->name2);
+	if (o_ptr->artifact) wr_byte(o_ptr->artifact->aidx);
+	else wr_byte(0);
+
+	if (o_ptr->ego) wr_byte(o_ptr->ego->eidx);
+	else wr_byte(0);
 
 	wr_s16b(o_ptr->timeout);
 
@@ -68,19 +77,24 @@ static void wr_item(const object_type *o_ptr)
 	wr_byte(o_ptr->origin_depth);
 	wr_u16b(o_ptr->origin_xtra);
 
-
-	/* Hack - XXX - MarbleDice - Maximum saveable flags = 96 */
-	for (i = 0; i < 12 && i < OF_SIZE; i++)
+	for (i = 0; i < OF_BYTES && i < OF_SIZE; i++)
 		wr_byte(o_ptr->flags[i]);
-	if (i < 12) pad_bytes(12 - i);
+	if (i < OF_BYTES) pad_bytes(OF_BYTES - i);
 
-	/* Hack - XXX - MarbleDice - Maximum saveable flags = 96 */
-	for (i = 0; i < 12 && i < OF_SIZE; i++)
+	for (i = 0; i < OF_BYTES && i < OF_SIZE; i++)
 		wr_byte(o_ptr->known_flags[i]);
-	if (i < 12) pad_bytes(12 - i);
+	if (i < OF_BYTES) pad_bytes(OF_BYTES - i);
+
+	for (j = 0; j < MAX_PVALS; j++) {
+		for (i = 0; i < OF_BYTES && i < OF_SIZE; i++)
+			wr_byte(o_ptr->pval_flags[j][i]);
+		if (i < OF_BYTES) pad_bytes(OF_BYTES - i);
+	}
 
 	/* Held by monster index */
 	wr_s16b(o_ptr->held_m_idx);
+	
+	wr_s16b(o_ptr->mimicking_m_idx);
 
 	/* Save the inscription (if any) */
 	if (o_ptr->note)
@@ -98,7 +112,7 @@ static void wr_item(const object_type *o_ptr)
  * Write RNG state
  *
  * There were originally 64 bytes of randomizer saved. Now we only need
- * 32 + 4 bytes saved, so we'll write an extra 28 bytes at the end which won't
+ * 32 + 5 bytes saved, so we'll write an extra 27 bytes at the end which won't
  * be used.
  */
 void wr_randomizer(void)
@@ -121,7 +135,7 @@ void wr_randomizer(void)
 		wr_u32b(STATE[i]);
 
 	/* NULL padding */
-	for (i = 0; i < 60 - RAND_DEG; i++)
+	for (i = 0; i < 59 - RAND_DEG; i++)
 		wr_u32b(0);
 }
 
@@ -133,55 +147,27 @@ void wr_options(void)
 {
 	int i, k;
 
-	u32b flag[8];
-	u32b mask[8];
 	u32b window_flag[ANGBAND_TERM_MAX];
 	u32b window_mask[ANGBAND_TERM_MAX];
 
 
-	/* XXX */
-	for (i = 0; i < 4; i++) wr_u32b(0L);
-
-
-	/*** Special Options ***/
-
+	/* Special Options */
 	wr_byte(op_ptr->delay_factor);
 	wr_byte(op_ptr->hitpoint_warn);
 	wr_u16b(lazymove_delay);
 
+	/* Normal options */
+	for (i = 0; i < OPT_MAX; i++) {
+		const char *name = option_name(i);
+		if (!name)
+			continue;
 
-	/*** Normal options ***/
-
-	/* Reset */
-	for (i = 0; i < 8; i++)
-	{
-		flag[i] = 0L;
-		mask[i] = 0L;
-	}
-
-	/* Analyze the options */
-	for (i = 0; i < OPT_MAX; i++)
-	{
-		int os = i / 32;
-		int ob = i % 32;
-
-		/* Process real entries */
-		if (!option_name(i)) continue;
-
-		/* Set flag */
-		if (op_ptr->opt[i])
-			flag[os] |= (1L << ob);
-
-		/* Set mask */
-		mask[os] |= (1L << ob);
-	}
-
-	/* Dump the flags */
-	for (i = 0; i < 8; i++) wr_u32b(flag[i]);
-
-	/* Dump the masks */
-	for (i = 0; i < 8; i++) wr_u32b(mask[i]);
+		wr_string(name);
+		wr_byte(op_ptr->opt[i]);
+   }
 
+	/* Sentinel */
+	wr_byte(0);
 
 	/*** Window options ***/
 
@@ -265,16 +251,13 @@ void wr_monster_memory(void)
 			wr_byte(l_ptr->blows[i]);
 
 		/* Memorize flags */
-
-		/* Hack - XXX - MarbleDice - Maximum saveable flags = 96 */
-		for (i = 0; i < 12 && i < RF_SIZE; i++)
+		for (i = 0; i < RF_BYTES && i < RF_SIZE; i++)
 			wr_byte(l_ptr->flags[i]);
-		if (i < 12) pad_bytes(12 - i);
+		if (i < RF_BYTES) pad_bytes(RF_BYTES - i);
 
-		/* Hack - XXX - MarbleDice - Maximum saveable flags = 96 */
-		for (i = 0; i < 12 && i < RSF_SIZE; i++)
+		for (i = 0; i < RF_BYTES && i < RSF_SIZE; i++)
 			wr_byte(l_ptr->spell_flags[i]);
-		if (i < 12) pad_bytes(12 - i);
+		if (i < RF_BYTES) pad_bytes(RF_BYTES - i);
 
 		/* Monster limit per level */
 		wr_byte(r_ptr->max_num);
@@ -356,8 +339,8 @@ void wr_player(void)
 	wr_string(p_ptr->history);
 
 	/* Race/Class/Gender/Spells */
-	wr_byte(p_ptr->prace);
-	wr_byte(p_ptr->pclass);
+	wr_byte(p_ptr->race->ridx);
+	wr_byte(p_ptr->class->cidx);
 	wr_byte(p_ptr->psex);
 	wr_byte(op_ptr->name_suffix);
 
@@ -435,7 +418,7 @@ void wr_player(void)
 
 void wr_squelch(void)
 {
-	size_t i;
+	size_t i, n;
 
 	/* Write number of squelch bytes */
 	wr_byte(squelch_size);
@@ -453,14 +436,20 @@ void wr_squelch(void)
 		wr_byte(flags);
 	}
 
+	n = 0;
+	for (i = 0; i < z_info->k_max; i++)
+		if (k_info[i].note)
+			n++;
+
 	/* Write the current number of auto-inscriptions */
-	wr_u16b(inscriptions_count);
+	wr_u16b(n);
 
 	/* Write the autoinscriptions array */
-	for (i = 0; i < inscriptions_count; i++)
-	{
-		wr_s16b(inscriptions[i].kind_idx);
-		wr_string(quark_str(inscriptions[i].inscription_idx));
+	for (i = 0; i < z_info->k_max; i++) {
+		if (!k_info[i].note)
+			continue;
+		wr_s16b(i);
+		wr_string(quark_str(k_info[i].note));
 	}
 
 	return;
@@ -470,8 +459,8 @@ void wr_squelch(void)
 void wr_misc(void)
 {
 
-	/* Random artifact version */
-	wr_u32b(RANDART_VERSION);
+	/* XXX Old random artifact version, remove after 3.3 */
+	wr_u32b(63);
 
 	/* Random artifact seed */
 	wr_u32b(seed_randart);
@@ -498,10 +487,9 @@ void wr_misc(void)
 	wr_byte(p_ptr->is_dead);
 
 	/* Write feeling */
-	wr_byte(feeling);
-
-	/* Turn of last "feeling" */
-	wr_s32b(old_turn);
+	wr_byte(cave->feeling);
+	wr_u16b(cave->feeling_squares);
+	wr_s32b(cave->created_at);
 
 	/* Current turn */
 	wr_s32b(turn);
@@ -537,9 +525,9 @@ void wr_player_spells(void)
  */
 void wr_randarts(void)
 {
-	size_t i, j;
+	size_t i, j, k;
 
-	if (!OPT(adult_randarts))
+	if (!OPT(birth_randarts))
 		return;
 
 	wr_u16b(z_info->a_max);
@@ -550,7 +538,9 @@ void wr_randarts(void)
 
 		wr_byte(a_ptr->tval);
 		wr_byte(a_ptr->sval);
-		wr_s16b(a_ptr->pval);
+		for (j = 0; j < MAX_PVALS; j++)
+			wr_s16b(a_ptr->pval[j]);
+		wr_byte(a_ptr->num_pvals);
 
 		wr_s16b(a_ptr->to_h);
 		wr_s16b(a_ptr->to_d);
@@ -564,10 +554,15 @@ void wr_randarts(void)
 
 		wr_s32b(a_ptr->cost);
 
-		/* Hack - XXX - MarbleDice - Maximum saveable flags = 96 */
-		for (j = 0; j < 12 && j < OF_SIZE; j++)
+		for (j = 0; j < OF_BYTES && j < OF_SIZE; j++)
 			wr_byte(a_ptr->flags[j]);
-		if (j < 12) pad_bytes(OF_SIZE - j);
+		if (j < OF_BYTES) pad_bytes(OF_BYTES - j);
+
+		for (k = 0; k < MAX_PVALS; k++) {
+			for (j = 0; j < OF_BYTES && j < OF_SIZE; j++)
+				wr_byte(a_ptr->pval_flags[k][j]);
+			if (j < OF_BYTES) pad_bytes(OF_BYTES - j);
+		}
 
 		wr_byte(a_ptr->level);
 		wr_byte(a_ptr->rarity);
@@ -593,7 +588,7 @@ void wr_inventory(void)
 		object_type *o_ptr = &p_ptr->inventory[i];
 
 		/* Skip non-objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 
 		/* Dump index */
 		wr_u16b((u16b)i);
@@ -614,7 +609,7 @@ void wr_stores(void)
 	wr_u16b(MAX_STORES);
 	for (i = 0; i < MAX_STORES; i++)
 	{
-		const store_type *st_ptr = &store[i];
+		const struct store *st_ptr = &stores[i];
 		int j;
 
 		/* XXX Old values */
@@ -668,8 +663,8 @@ void wr_dungeon(void)
 	wr_u16b(daycount);
 	wr_u16b(p_ptr->py);
 	wr_u16b(p_ptr->px);
-	wr_u16b(DUNGEON_HGT);
-	wr_u16b(DUNGEON_WID);
+	wr_u16b(cave->height);
+	wr_u16b(cave->width);
 	wr_u16b(0);
 	wr_u16b(0);
 
@@ -685,8 +680,8 @@ void wr_dungeon(void)
 	{
 		for (x = 0; x < DUNGEON_WID; x++)
 		{
-			/* Extract the important cave_info flags */
-			tmp8u = (cave_info[y][x] & (IMPORTANT_FLAGS));
+			/* Extract the important cave->info flags */
+			tmp8u = (cave->info[y][x] & (IMPORTANT_FLAGS));
 
 			/* If the run is broken, or too full, flush it */
 			if ((tmp8u != prev_char) || (count == MAX_UCHAR))
@@ -712,7 +707,7 @@ void wr_dungeon(void)
 		wr_byte((byte)prev_char);
 	}
 
-	/** Now dump the cave_info2[][] stuff **/
+	/** Now dump the cave->info2[][] stuff **/
 
 	/* Note that this will induce two wasted bytes */
 	count = 0;
@@ -724,7 +719,7 @@ void wr_dungeon(void)
 		for (x = 0; x < DUNGEON_WID; x++)
 		{
 			/* Keep all the information from info2 */
-			tmp8u = cave_info2[y][x];
+			tmp8u = cave->info2[y][x];
 
 			/* If the run is broken, or too full, flush it */
 			if ((tmp8u != prev_char) || (count == MAX_UCHAR))
@@ -763,7 +758,7 @@ void wr_dungeon(void)
 		for (x = 0; x < DUNGEON_WID; x++)
 		{
 			/* Extract a byte */
-			tmp8u = cave_feat[y][x];
+			tmp8u = cave->feat[y][x];
 
 			/* If the run is broken, or too full, flush it */
 			if ((tmp8u != prev_char) || (count == MAX_UCHAR))
@@ -813,7 +808,7 @@ void wr_objects(void)
 	/* Dump the objects */
 	for (i = 1; i < o_max; i++)
 	{
-		object_type *o_ptr = &o_list[i];
+		object_type *o_ptr = object_byid(i);
 
 		/* Dump it */
 		wr_item(o_ptr);
@@ -824,29 +819,39 @@ void wr_objects(void)
 void wr_monsters(void)
 {
 	int i;
+	size_t j;
 
 	if (p_ptr->is_dead)
 		return;
 
 	/* Total monsters */
-	wr_u16b(mon_max);
+	wr_u16b(cave_monster_max(cave));
 
 	/* Dump the monsters */
-	for (i = 1; i < mon_max; i++)
-	{
-		const monster_type *m_ptr = &mon_list[i];
+	for (i = 1; i < cave_monster_max(cave); i++) {
+		byte unaware = 0;
+	
+		const monster_type *m_ptr = cave_monster(cave, i);
 
 		wr_s16b(m_ptr->r_idx);
 		wr_byte(m_ptr->fy);
 		wr_byte(m_ptr->fx);
 		wr_s16b(m_ptr->hp);
 		wr_s16b(m_ptr->maxhp);
-		wr_s16b(m_ptr->csleep);
 		wr_byte(m_ptr->mspeed);
 		wr_byte(m_ptr->energy);
-		wr_byte(m_ptr->stunned);
-		wr_byte(m_ptr->confused);
-		wr_byte(m_ptr->monfear);
+		wr_byte(MON_TMD_MAX);
+
+		for (j = 0; j < MON_TMD_MAX; j++)
+			wr_s16b(m_ptr->m_timed[j]);
+
+		if (m_ptr->unaware) unaware |= 0x01;
+		wr_byte(unaware);
+
+		for (j = 0; j < OF_BYTES && j < OF_SIZE; j++)
+			wr_byte(m_ptr->known_pflags[j]);
+		if (j < OF_BYTES) pad_bytes(OF_BYTES - j);
+		
 		wr_byte(0);
 	}
 }
