@@ -20,12 +20,10 @@
 #include "cmds.h"
 #include "ui-menu.h"
 #include "object/tvalsval.h"
+#include "object/pval.h"
 #include "squelch.h"
 
 
-
-
-
 typedef struct
 {
 	squelch_type_t squelch_type;
@@ -36,34 +34,70 @@ typedef struct
 
 static quality_squelch_struct quality_mapping[] =
 {
-	{ TYPE_WEAPON_POINTY,	TV_SWORD,	0,		SV_UNKNOWN },
-	{ TYPE_WEAPON_POINTY,	TV_POLEARM,	0,		SV_UNKNOWN },
-	{ TYPE_WEAPON_BLUNT,	TV_HAFTED,	0,		SV_UNKNOWN },
-	{ TYPE_SHOOTER,		TV_BOW,		0,		SV_UNKNOWN },
-	{ TYPE_MISSILE_SLING,	TV_SHOT,	0,		SV_UNKNOWN },
-	{ TYPE_MISSILE_BOW,	TV_ARROW,	0,		SV_UNKNOWN },
-	{ TYPE_MISSILE_XBOW,	TV_BOLT,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_ROBE,	TV_SOFT_ARMOR,	SV_ROBE,	SV_ROBE },
-	{ TYPE_ARMOR_BODY,	TV_DRAG_ARMOR,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_BODY,	TV_HARD_ARMOR,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_BODY,	TV_SOFT_ARMOR,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_CLOAK,	TV_CLOAK,	SV_CLOAK, 	SV_FUR_CLOAK },
-	{ TYPE_ARMOR_CLOAK,	TV_CLOAK,	SV_ETHEREAL_CLOAK, 	SV_ETHEREAL_CLOAK },
+	{ TYPE_WEAPON_POINTY,	TV_SWORD,		0,		SV_UNKNOWN },
+	{ TYPE_WEAPON_POINTY,	TV_POLEARM,		0,		SV_UNKNOWN },
+	{ TYPE_WEAPON_BLUNT,	TV_HAFTED,		0,		SV_UNKNOWN },
+	{ TYPE_SHOOTER,			TV_BOW,			0,		SV_UNKNOWN },
+	{ TYPE_MISSILE_SLING,	TV_SHOT,		0,		SV_UNKNOWN },
+	{ TYPE_MISSILE_BOW,		TV_ARROW,		0,		SV_UNKNOWN },
+	{ TYPE_MISSILE_XBOW,	TV_BOLT,		0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_ROBE,		TV_SOFT_ARMOR,	SV_ROBE,SV_ROBE },
+	{ TYPE_ARMOR_BODY,		TV_DRAG_ARMOR,	0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_BODY,		TV_HARD_ARMOR,	0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_BODY,		TV_SOFT_ARMOR,	0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_CLOAK,		TV_CLOAK,		SV_CLOAK, SV_FUR_CLOAK },
+	{ TYPE_ARMOR_CLOAK,		TV_CLOAK,		SV_ETHEREAL_CLOAK, SV_ETHEREAL_CLOAK },
 /* XXX Eddie need to assert SV_CLOAK < SV_FUR_CLOAK < SV_ELVEN_CLOAK */
-	{ TYPE_ARMOR_ELVEN_CLOAK,	TV_CLOAK,	SV_ELVEN_CLOAK, 	SV_ELVEN_CLOAK },
-	{ TYPE_ARMOR_SHIELD,	TV_SHIELD,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_HEAD,	TV_HELM,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_HEAD,	TV_CROWN,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_HANDS,	TV_GLOVES,	0,		SV_UNKNOWN },
-	{ TYPE_ARMOR_FEET,	TV_BOOTS,	0,		SV_UNKNOWN },
-	{ TYPE_DIGGER,		TV_DIGGING,	0,		SV_UNKNOWN },
-	{ TYPE_RING,		TV_RING,	0,		SV_UNKNOWN },
-	{ TYPE_AMULET,		TV_AMULET,	0,		SV_UNKNOWN },
-	{ TYPE_LIGHT, 		TV_LIGHT, 	0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_ELVEN_CLOAK, TV_CLOAK,		SV_ELVEN_CLOAK,	SV_ELVEN_CLOAK },
+	{ TYPE_ARMOR_SHIELD,	TV_SHIELD,		0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_HEAD,		TV_HELM,		0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_HEAD,		TV_CROWN,		0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_HANDS,		TV_GLOVES,		0,		SV_UNKNOWN },
+	{ TYPE_ARMOR_FEET,		TV_BOOTS,		0,		SV_UNKNOWN },
+	{ TYPE_DIGGER,			TV_DIGGING,		0,		SV_UNKNOWN },
+	{ TYPE_RING,			TV_RING,		0,		SV_UNKNOWN },
+	{ TYPE_AMULET,			TV_AMULET,		0,		SV_UNKNOWN },
+	{ TYPE_LIGHT, 			TV_LIGHT, 		0,		SV_UNKNOWN },
 };
 
 
 
+quality_name_struct quality_choices[TYPE_MAX] =
+{
+	{ TYPE_WEAPON_POINTY,		"Pointy Melee Weapons" },
+	{ TYPE_WEAPON_BLUNT,		"Blunt Melee Weapons" },
+	{ TYPE_SHOOTER,				"Missile weapons" },
+	{ TYPE_MISSILE_SLING,		"Shots and Pebbles" },
+	{ TYPE_MISSILE_BOW,			"Arrows" },
+	{ TYPE_MISSILE_XBOW,		"Bolts" },
+	{ TYPE_ARMOR_ROBE,			"Robes" },
+	{ TYPE_ARMOR_BODY,			"Body Armor" },
+	{ TYPE_ARMOR_CLOAK,			"Cloaks" },
+	{ TYPE_ARMOR_ELVEN_CLOAK,	"Elven Cloaks" },
+	{ TYPE_ARMOR_SHIELD,		"Shields" },
+	{ TYPE_ARMOR_HEAD,			"Headgear" },
+	{ TYPE_ARMOR_HANDS,			"Handgear" },
+	{ TYPE_ARMOR_FEET,			"Footgear" },
+	{ TYPE_DIGGER,				"Diggers" },
+	{ TYPE_RING,				"Rings" },
+	{ TYPE_AMULET,				"Amulets" },
+	{ TYPE_LIGHT, 				"Lights" },
+};
+
+/*
+ * The names for the various kinds of quality
+ */
+quality_name_struct quality_values[SQUELCH_MAX] =
+{
+	{ SQUELCH_NONE,				"no squelch" },
+	{ SQUELCH_BAD,				"bad" },
+	{ SQUELCH_AVERAGE,			"average" },
+	{ SQUELCH_GOOD,				"good" },
+	{ SQUELCH_EXCELLENT_NO_HI,	"excellent with no high resists" },
+	{ SQUELCH_EXCELLENT_NO_SPL,	"excellent but not splendid" },
+	{ SQUELCH_ALL,				"non-artifact" },
+};
+
 byte squelch_level[TYPE_MAX];
 const size_t squelch_size = TYPE_MAX;
 
@@ -90,58 +124,30 @@ void squelch_birth_init(void)
 
 	/* Clear the squelch bytes */
 	for (i = 0; i < TYPE_MAX; i++)
-		squelch_level[i] = 0;
+		squelch_level[i] = SQUELCH_NONE;
 }
 
 
 
 /*** Autoinscription stuff ***/
 
-/*
- * This code needs documenting.
- */
-int get_autoinscription_index(s16b k_idx)
+const char *get_autoinscription(object_kind *kind)
 {
-	int i;
-
-	for (i = 0; i < inscriptions_count; i++)
-	{
-		if (k_idx == inscriptions[i].kind_idx)
-			return i;
-	}
-
-	return -1;
-}
-
-/*
- * DOCUMENT ME!
- */
-const char *get_autoinscription(s16b kind_idx)
-{
-	int i;
-
-	for (i = 0; i < inscriptions_count; i++)
-	{
-		if (kind_idx == inscriptions[i].kind_idx)
-			return quark_str(inscriptions[i].inscription_idx);
-	}
-
-	return 0;
+	return kind ? quark_str(kind->note) : NULL;
 }
 
 /* Put the autoinscription on an object */
 int apply_autoinscription(object_type *o_ptr)
 {
 	char o_name[80];
-	cptr note = get_autoinscription(o_ptr->k_idx);
-	cptr existing_inscription = quark_str(o_ptr->note);
+	const char *note = get_autoinscription(o_ptr->kind);
 
 	/* Don't inscribe unaware objects */
 	if (!note || !object_flavor_is_aware(o_ptr))
 		return 0;
 
 	/* Don't re-inscribe if it's already inscribed */
-	if (existing_inscription)
+	if (o_ptr->note)
 		return 0;
 
 	/* Get an object description */
@@ -152,7 +158,7 @@ int apply_autoinscription(object_type *o_ptr)
 	else
 		o_ptr->note = 0;
 
-	msg_format("You autoinscribe %s.", o_name);
+	msg("You autoinscribe %s.", o_name);
 
 	return 1;
 }
@@ -160,52 +166,22 @@ int apply_autoinscription(object_type *o_ptr)
 
 int remove_autoinscription(s16b kind)
 {
-	int i = get_autoinscription_index(kind);
-
-	/* It's not here. */
-	if (i == -1) return 0;
-
-	while (i < inscriptions_count - 1)
-	{
-		inscriptions[i] = inscriptions[i+1];
-		i++;
-	}
-
-	inscriptions_count--;
-
+	struct object_kind *k = objkind_byid(kind);
+	if (!k || !k->note)
+		return 0;
+	k->note = 0;
 	return 1;
 }
 
 
-int add_autoinscription(s16b kind, cptr inscription)
+int add_autoinscription(s16b kind, const char *inscription)
 {
-	int index;
-
-	/* Paranoia */
-	if (kind == 0) return 0;
-
-	/* If there's no inscription, remove it */
-	if (!inscription || (inscription[0] == 0))
-		return remove_autoinscription(kind);
-
-	index = get_autoinscription_index(kind);
-
-	if (index == -1)
-		index = inscriptions_count;
-
-	if (index >= AUTOINSCRIPTIONS_MAX)
-	{
-		msg_format("This inscription (%s) cannot be added because the inscription array is full!", inscription);
+	struct object_kind *k = objkind_byid(kind);
+	if (!k)
 		return 0;
-	}
-
-	inscriptions[index].kind_idx = kind;
-	inscriptions[index].inscription_idx = quark_add(inscription);
-
-	/* Only increment count if inscription added to end of array */
-	if (index == inscriptions_count)
-		inscriptions_count++;
-
+	if (!inscription)
+		return remove_autoinscription(kind);
+	k->note = quark_add(inscription);
 	return 1;
 }
 
@@ -217,13 +193,13 @@ void autoinscribe_ground(void)
 	s16b this_o_idx, next_o_idx = 0;
 
 	/* Scan the pile of objects */
-	for (this_o_idx = cave_o_idx[py][px]; this_o_idx; this_o_idx = next_o_idx)
+	for (this_o_idx = cave->o_idx[py][px]; this_o_idx; this_o_idx = next_o_idx)
 	{
 		/* Get the next object */
-		next_o_idx = o_list[this_o_idx].next_o_idx;
+		next_o_idx = object_byid(this_o_idx)->next_o_idx;
 
 		/* Apply an autoinscription */
-		apply_autoinscription(&o_list[this_o_idx]);
+		apply_autoinscription(object_byid(this_o_idx));
 	}
 }
 
@@ -235,7 +211,7 @@ void autoinscribe_pack(void)
 	for (i = INVEN_PACK; i >= 0; i--)
 	{
 		/* Skip empty items */
-		if (!p_ptr->inventory[i].k_idx) continue;
+		if (!p_ptr->inventory[i].kind) continue;
 
 		/* Apply the inscription */
 		apply_autoinscription(&p_ptr->inventory[i]);
@@ -255,9 +231,9 @@ void autoinscribe_pack(void)
 void object_squelch_flavor_of(const object_type *o_ptr)
 {
 	if (object_flavor_is_aware(o_ptr))
-		k_info[o_ptr->k_idx].squelch |= SQUELCH_IF_AWARE;
+		o_ptr->kind->squelch |= SQUELCH_IF_AWARE;
 	else
-		k_info[o_ptr->k_idx].squelch |= SQUELCH_IF_UNAWARE;
+		o_ptr->kind->squelch |= SQUELCH_IF_UNAWARE;
 }
 
 
@@ -280,6 +256,37 @@ squelch_type_t squelch_type_of(const object_type *o_ptr)
 	return TYPE_MAX;
 }
 
+/**
+ * Small helper function to see how an object trait compares to the one
+ * in its base type.
+ *
+ * If the base type provides a positive bonus, we'll use that. Otherwise, we'll
+ * use zero (players don't consider an item with a positive bonus to be bad
+ * even if the base kind has a higher positive bonus).
+ */
+static int cmp_object_trait(int bonus, random_value base)
+{
+	int amt = randcalc(base, 0, MINIMISE);
+	if (amt > 0) amt = 0;
+	return CMP(bonus, amt);
+}
+
+/**
+ * Small helper function to see if an item seems good, bad or average based on
+ * to_h, to_d and to_a.
+ *
+ * The sign of the return value announces if the object is bad (negative),
+ * good (positive) or average (zero).
+ */
+static int is_object_good(const object_type *o_ptr)
+{
+	int good = 0;
+	good += 4 * cmp_object_trait(o_ptr->to_d, o_ptr->kind->to_d);
+	good += 2 * cmp_object_trait(o_ptr->to_h, o_ptr->kind->to_h);
+	good += 1 * cmp_object_trait(o_ptr->to_a, o_ptr->kind->to_a);
+	return good;
+}
+
 
 /*
  * Determine the squelch level of an object, which is similar to its pseudo.
@@ -289,24 +296,25 @@ squelch_type_t squelch_type_of(const object_type *o_ptr)
  */
 byte squelch_level_of(const object_type *o_ptr)
 {
-	object_kind *k_ptr = &k_info[o_ptr->k_idx];
 	byte value;
-	bitflag f[OF_SIZE];
+	bitflag f[OF_SIZE], f2[OF_SIZE];
+	int i;
 
 	object_flags_known(o_ptr, f);
 
-	if ((object_pval_is_visible(o_ptr)) && (o_ptr->pval < 0))
-		return SQUELCH_BAD;
-
 	/* Deal with jewelry specially. */
 	if (object_is_jewelry(o_ptr))
 	{
-		if ((object_pval_is_visible(o_ptr)) && (o_ptr->pval > 0))
-			return SQUELCH_AVERAGE;
+		/* CC: average jewelry has at least one known positive pval */
+		for (i = 0; i < o_ptr->num_pvals; i++)
+			if ((object_this_pval_is_visible(o_ptr, i)) && (o_ptr->pval[i] > 0))
+				return SQUELCH_AVERAGE;
+
 		if ((o_ptr->to_h > 0) || (o_ptr->to_d > 0) || (o_ptr->to_a > 0))
 			return SQUELCH_AVERAGE;
-		if ((object_attack_plusses_are_visible(o_ptr) && ((o_ptr->to_h < 0) || (o_ptr->to_d < 0))) ||
-		    (object_defence_plusses_are_visible(o_ptr) && (o_ptr->to_a < 0)))
+		if ((object_attack_plusses_are_visible(o_ptr) &&
+				((o_ptr->to_h < 0) || (o_ptr->to_d < 0))) ||
+		    	(object_defence_plusses_are_visible(o_ptr) && o_ptr->to_a < 0))
 			return SQUELCH_BAD;
 
 		return SQUELCH_AVERAGE;
@@ -315,7 +323,8 @@ byte squelch_level_of(const object_type *o_ptr)
 	/* And lights */
 	if (o_ptr->tval == TV_LIGHT)
 	{
-		if (flags_test(f, OF_SIZE, OF_OBVIOUS_MASK, FLAG_END))
+		create_mask(f2, TRUE, OFID_WIELD, OFT_MAX);
+		if (of_is_inter(f, f2))
 			return SQUELCH_ALL;
 		if ((o_ptr->to_h > 0) || (o_ptr->to_d > 0) || (o_ptr->to_a > 0))
 			return SQUELCH_GOOD;
@@ -325,32 +334,39 @@ byte squelch_level_of(const object_type *o_ptr)
 		return SQUELCH_AVERAGE;
 	}
 
-	if (object_was_sensed(o_ptr))
-	{
+	/* CC: we need to redefine "bad" with multiple pvals
+	 * At the moment we use "all pvals known and negative" */
+	for (i = 0; i < o_ptr->num_pvals; i++) {
+		if (!object_this_pval_is_visible(o_ptr, i) ||
+			(o_ptr->pval[i] > 0))
+			break;
+
+		if (i == (o_ptr->num_pvals - 1))
+			return SQUELCH_BAD;
+	}
+
+	if (object_was_sensed(o_ptr)) {
 		obj_pseudo_t pseudo = object_pseudo(o_ptr);
 
-		switch (pseudo)
-		{
-			case INSCRIP_AVERAGE:
+		switch (pseudo) {
+			case INSCRIP_AVERAGE: {
 				value = SQUELCH_AVERAGE;
 				break;
+			}
 
-			case INSCRIP_EXCELLENT:
+			case INSCRIP_EXCELLENT: {
 				/* have to assume splendid until you have tested it */
-				if (object_was_worn(o_ptr))
-				{
+				if (object_was_worn(o_ptr)) {
 					if (object_high_resist_is_possible(o_ptr))
 						value = SQUELCH_EXCELLENT_NO_SPL;
 					else
 						value = SQUELCH_EXCELLENT_NO_HI;
-				}
-				else
-				{
+				} else {
 					value = SQUELCH_ALL;
 				}
 				break;
+			}
 
-			case INSCRIP_STRANGE: /* XXX Eddie perhaps some strange count as something else */
 			case INSCRIP_SPLENDID:
 				value = SQUELCH_ALL;
 				break;
@@ -360,16 +376,26 @@ byte squelch_level_of(const object_type *o_ptr)
 				break;
 
 			/* This is the interesting case */
-			case INSCRIP_MAGICAL:
+			case INSCRIP_STRANGE:
+			case INSCRIP_MAGICAL: {
 				value = SQUELCH_GOOD;
-				if ((object_attack_plusses_are_visible(o_ptr) || (randcalc_valid(k_ptr->to_h, o_ptr->to_h) && randcalc_valid(k_ptr->to_d, o_ptr->to_d))) &&
-				    (object_defence_plusses_are_visible(o_ptr) || (randcalc_valid(k_ptr->to_a, o_ptr->to_a))) &&
-				    (o_ptr->to_h <= randcalc(k_ptr->to_h, 0, MINIMISE)) &&
-				    (o_ptr->to_d <= randcalc(k_ptr->to_d, 0, MINIMISE)) &&
-				    (o_ptr->to_a <= randcalc(k_ptr->to_a, 0, MINIMISE)))
-					value = SQUELCH_BAD;
-				break;
 
+				if ((object_attack_plusses_are_visible(o_ptr) ||
+						randcalc_valid(o_ptr->kind->to_h, o_ptr->to_h) ||
+						randcalc_valid(o_ptr->kind->to_d, o_ptr->to_d)) &&
+				    	(object_defence_plusses_are_visible(o_ptr) ||
+						randcalc_valid(o_ptr->kind->to_a, o_ptr->to_a))) {
+					int isgood = is_object_good(o_ptr);
+					if (isgood > 0) {
+						value = SQUELCH_GOOD;
+					} else if (isgood < 0) {
+						value = SQUELCH_BAD;
+					} else {
+						value = SQUELCH_AVERAGE;
+					}
+				}
+				break;
+			}
 
 			default:
 				/* do not handle any other possible pseudo values */
@@ -421,40 +447,33 @@ void kind_squelch_when_unaware(object_kind *k_ptr)
 }
 
 
-
 /*
  * Determines if an object is eligible for squelching.
  */
 bool squelch_item_ok(const object_type *o_ptr)
 {
-	object_kind *k_ptr = &k_info[o_ptr->k_idx];
 	byte type;
 
+	if (p_ptr->unignoring)
+		return FALSE;
+
 	/* Don't squelch artifacts unless marked to be squelched */
-	if (artifact_p(o_ptr))
+	if (o_ptr->artifact ||
+			check_for_inscrip(o_ptr, "!k") || check_for_inscrip(o_ptr, "!*"))
 		return FALSE;
 
-	/* Don't squelch stuff inscribed not to be destroyed (!k) */
-	if (check_for_inscrip(o_ptr, "!k") || check_for_inscrip(o_ptr, "!*"))
-		return FALSE;
-
-	/* Auto-squelch dead chests */
-	if (o_ptr->tval == TV_CHEST && o_ptr->pval == 0)
+	/* Do squelch individual objects that marked ignore */
+	if (o_ptr->ignore)
 		return TRUE;
 
-	/* check option for worthless kinds */
-	if (OPT(squelch_worthless) && o_ptr->tval != TV_GOLD)
-	{
-		if (object_flavor_is_aware(o_ptr) && k_ptr->cost == 0)
-			return TRUE;
-		if (object_is_known_cursed(o_ptr))
-			return TRUE;
-	}
+	/* Auto-squelch dead chests */
+	if (o_ptr->tval == TV_CHEST && o_ptr->pval[DEFAULT_PVAL] == 0)
+		return TRUE;
 
 	/* Do squelching by kind */
 	if (object_flavor_is_aware(o_ptr) ?
-		 kind_is_squelched_aware(k_ptr) :
-		 kind_is_squelched_unaware(k_ptr))
+		 kind_is_squelched_aware(o_ptr->kind) :
+		 kind_is_squelched_unaware(o_ptr->kind))
 		return TRUE;
 
 	type = squelch_type_of(o_ptr);
@@ -472,85 +491,6 @@ bool squelch_item_ok(const object_type *o_ptr)
 		return FALSE;
 }
 
-
-/*
- * Returns TRUE if an item should be hidden due to the player's
- * current settings.
- */
-bool squelch_hide_item(object_type *o_ptr)
-{
-	return (OPT(hide_squelchable) ? squelch_item_ok(o_ptr) : FALSE);
-}
-
-
-/*
- * Destroy all {squelch}able items.
- *
- * Imported, with thanks, from Ey... much cleaner than the original.
- */
-void squelch_items(void)
-{
-	int floor_list[MAX_FLOOR_STACK];
-	int floor_num, n;
-	int count = 0;
-
-	object_type *o_ptr;
-
-	/* Set the hook and scan the floor */
-	item_tester_hook = squelch_item_ok;
-	floor_num = scan_floor(floor_list, N_ELEMENTS(floor_list), p_ptr->py, p_ptr->px, 0x01);
-
-	if (floor_num)
-	{
-		for (n = 0; n < floor_num; n++)
-		{
-			o_ptr = &o_list[floor_list[n]];
-
-			/* Avoid artifacts */
-			if (artifact_p(o_ptr)) continue;
-
-			if (item_tester_okay(o_ptr))
-			{
-				/* Destroy item */
-				floor_item_increase(floor_list[n], -o_ptr->number);
-				floor_item_optimize(floor_list[n]);
-				count++;
-			}
-		}
-	}
-
-	/* Scan through the slots backwards */
-	for (n = INVEN_PACK - 1; n >= 0; n--)
-	{
-		o_ptr = &p_ptr->inventory[n];
-
-		/* Skip non-objects and artifacts */
-		if (!o_ptr->k_idx) continue;
-		if (artifact_p(o_ptr)) continue;
-
-		if (item_tester_okay(o_ptr))
-		{
-			/* Destroy item */
-			inven_item_increase(n, -o_ptr->number);
-			inven_item_optimize(n);
-			count++;
-		}
-	}
-
-	item_tester_hook = NULL;
-
-	/* Mention casualties */
-	if (count > 0)
-	{
-		message_format(MSG_DESTROY, 0, "%d item%s squelched.",
-		               count, ((count > 1) ? "s" : ""));
-
-		/* Combine/reorder the pack */
-		p_ptr->notice |= (PN_COMBINE | PN_REORDER | PN_SORT_QUIVER);
-	}
-}
-
-
 /*
  * Drop all {squelch}able items.
  */
@@ -564,7 +504,7 @@ void squelch_drop(void)
 		object_type *o_ptr = &p_ptr->inventory[n];
 
 		/* Skip non-objects and unsquelchable objects */
-		if (!o_ptr->k_idx) continue;
+		if (!o_ptr->kind) continue;
 		if (!squelch_item_ok(o_ptr)) continue;
 
 		/* Check for !d (no drop) inscription */
