@@ -6,17 +6,17 @@
 #include "init.h"
 #include "types.h"
 
-static int setup(void **state) {
+int setup_tests(void **state) {
 	*state = init_parse_k();
 	return !*state;
 }
 
-static int teardown(void *state) {
+int teardown_tests(void *state) {
 	parser_destroy(state);
 	return 0;
 }
 
-static int test_n0(void *state) {
+int test_n0(void *state) {
 	errr r = parser_parse(state, "N:3:Test Object Kind");
 	struct object_kind *k;
 
@@ -28,7 +28,7 @@ static int test_n0(void *state) {
 	ok;
 }
 
-static int test_g0(void *state) {
+int test_g0(void *state) {
 	errr r = parser_parse(state, "G:~:red");
 	struct object_kind *k;
 
@@ -40,7 +40,7 @@ static int test_g0(void *state) {
 	ok;
 }
 
-static int test_g1(void *state) {
+int test_g1(void *state) {
 	errr r = parser_parse(state, "G:!:W");
 	struct object_kind *k;
 
@@ -52,8 +52,8 @@ static int test_g1(void *state) {
 	ok;
 }
 
-static int test_i0(void *state) {
-	errr r = parser_parse(state, "I:4:2:3d6");
+int test_i0(void *state) {
+	errr r = parser_parse(state, "I:4:2");
 	struct object_kind *k;
 
 	eq(r, 0);
@@ -61,13 +61,11 @@ static int test_i0(void *state) {
 	require(k);
 	eq(k->tval, 4);
 	eq(k->sval, 2);
-	eq(k->pval.dice, 3);
-	eq(k->pval.sides, 6);
 	ok;
 }
 
-static int test_i1(void *state) {
-	errr r = parser_parse(state, "I:food:2:0");
+int test_i1(void *state) {
+	errr r = parser_parse(state, "I:food:2");
 	struct object_kind *k;
 
 	eq(r, 0);
@@ -78,7 +76,7 @@ static int test_i1(void *state) {
 	ok;
 }
 
-static int test_w0(void *state) {
+int test_w0(void *state) {
 	errr r = parser_parse(state, "W:10:0:5:120");
 	struct object_kind *k;
 
@@ -91,7 +89,7 @@ static int test_w0(void *state) {
 	ok;
 }
 
-static int test_a0(void *state) {
+int test_a0(void *state) {
 	errr r = parser_parse(state, "A:3:4 to 6");
 	struct object_kind *k;
 
@@ -104,7 +102,7 @@ static int test_a0(void *state) {
 	ok;
 }
 
-static int test_p0(void *state) {
+int test_p0(void *state) {
 	errr r = parser_parse(state, "P:3:4d8:1d4:2d5:7d6");
 	struct object_kind *k;
 
@@ -123,7 +121,7 @@ static int test_p0(void *state) {
 	ok;
 }
 
-static int test_c0(void *state) {
+int test_c0(void *state) {
 	errr r = parser_parse(state, "C:2d8");
 	struct object_kind *k;
 
@@ -135,7 +133,7 @@ static int test_c0(void *state) {
 	ok;
 }
 
-static int test_m0(void *state) {
+int test_m0(void *state) {
 	errr r = parser_parse(state, "M:4:3d6");
 	struct object_kind *k;
 
@@ -148,7 +146,7 @@ static int test_m0(void *state) {
 	ok;
 }
 
-static int test_f0(void *state) {
+int test_f0(void *state) {
 	errr r = parser_parse(state, "F:STR");
 	struct object_kind *k;
 
@@ -159,7 +157,22 @@ static int test_f0(void *state) {
 	ok;
 }
 
-static int test_e0(void *state) {
+int test_l0(void *state) {
+	errr r = parser_parse(state, "L:1+2d3M4:STR | INT");
+	struct object_kind *k;
+
+	eq(r, 0);
+	k = parser_priv(state);
+	require(k);
+	eq(k->pval[0].base, 1);
+	eq(k->pval[0].dice, 2);
+	eq(k->pval[0].sides, 3);
+	eq(k->pval[0].m_bonus, 4);
+	require(k->pval_flags[0]);
+	ok;
+}
+
+int test_e0(void *state) {
 	errr r = parser_parse(state, "E:POISON:4d5");
 	struct object_kind *k;
 
@@ -172,7 +185,7 @@ static int test_e0(void *state) {
 	ok;
 }
 
-static int test_d0(void *state) {
+int test_d0(void *state) {
 	errr r = parser_parse(state, "D:foo bar");
 	struct object_kind *k;
 
@@ -188,8 +201,8 @@ static int test_d0(void *state) {
 	ok;
 }
 
-static const char *suite_name = "parse/k-info";
-static struct test tests[] = {
+const char *suite_name = "parse/k-info";
+struct test tests[] = {
 	{ "n0", test_n0 },
 	{ "g0", test_g0 },
 	{ "g1", test_g1 },
@@ -203,5 +216,6 @@ static struct test tests[] = {
 	{ "f0", test_f0 },
 	{ "e0", test_e0 },
 	{ "d0", test_d0 },
+	{ "l0", test_l0 },
 	{ NULL, NULL }
 };
