@@ -5,17 +5,17 @@
 #include "init.h"
 #include "types.h"
 
-static int setup(void **state) {
+int setup_tests(void **state) {
 	*state = init_parse_s();
 	return !*state;
 }
 
-static int teardown(void *state) {
+int teardown_tests(void *state) {
 	parser_destroy(state);
 	return 0;
 }
 
-static int test_n0(void *state) {
+int test_n0(void *state) {
 	enum parser_error r = parser_parse(state, "N:1:Detect Monsters");
 	struct spell *s;
 
@@ -27,7 +27,7 @@ static int test_n0(void *state) {
 	ok;
 }
 
-static int test_i0(void *state) {
+int test_i0(void *state) {
 	enum parser_error r = parser_parse(state, "I:90:0:1");
 	struct spell *s;
 
@@ -40,7 +40,7 @@ static int test_i0(void *state) {
 	ok;
 }
 
-static int test_d0(void *state) {
+int test_d0(void *state) {
 	enum parser_error r = parser_parse(state, "D:Teleports you randomly.");
 	struct spell *s;
 
@@ -51,8 +51,8 @@ static int test_d0(void *state) {
 	ok;
 }
 
-static const char *suite_name = "parse/s-info";
-static struct test tests[] = {
+const char *suite_name = "parse/s-info";
+struct test tests[] = {
 	{ "n0", test_n0 },
 	{ "i0", test_i0 },
 	{ "d0", test_d0 },
