@@ -5,17 +5,17 @@
 #include "init.h"
 #include "types.h"
 
-static int setup(void **state) {
+int setup_tests(void **state) {
 	*state = init_parse_v();
 	return !*state;
 }
 
-static int teardown(void *state) {
+int teardown_tests(void *state) {
 	parser_destroy(state);
 	return 0;
 }
 
-static int test_n0(void *state) {
+int test_n0(void *state) {
 	enum parser_error r = parser_parse(state, "N:1:round");
 	struct vault *v;
 
@@ -27,7 +27,7 @@ static int test_n0(void *state) {
 	ok;
 }
 
-static int test_x0(void *state) {
+int test_x0(void *state) {
 	enum parser_error r = parser_parse(state, "X:6:5:12:20");
 	struct vault *v;
 
@@ -41,7 +41,7 @@ static int test_x0(void *state) {
 	ok;
 }
 
-static int test_d0(void *state) {
+int test_d0(void *state) {
 	enum parser_error r0 = parser_parse(state, "D:  %%  ");
 	enum parser_error r1 = parser_parse(state, "D: %  % ");
 	struct vault *v;
@@ -54,8 +54,8 @@ static int test_d0(void *state) {
 	ok;
 }
 
-static const char *suite_name = "parse/v-info";
-static struct test tests[] = {
+const char *suite_name = "parse/v-info";
+struct test tests[] = {
 	{ "n0", test_n0 },
 	{ "x0", test_x0 },
 	{ "d0", test_d0 },
