@@ -1,6 +1,6 @@
 /*
  * File: util.c
- * Purpose: Macro code, gamma correction, some high-level UI functions, inkey()
+ * Purpose: Gamma correction, some high-level UI functions, inkey()
  *
  * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
  *
@@ -18,456 +18,9 @@
 
 #include "angband.h"
 #include "button.h"
-#include "game-event.h"
-#include "macro.h"
 #include "cmds.h"
-
-/*
- * Convert a decimal to a single digit hex number
- */
-static char hexify(int i)
-{
-	return (hexsym[i % 16]);
-}
-
-/*
- * Convert a hexidecimal-digit into a decimal
- */
-static int dehex(char c)
-{
-	if (isdigit((unsigned char)c)) return (D2I(c));
-	if (isalpha((unsigned char)c)) return (A2I(tolower((unsigned char)c)) + 10);
-	return (0);
-}
-
-
-/*
- * Transform macro trigger name ('\[alt-D]' etc..)
- * into macro trigger key code ('^_O_64\r' or etc..)
- */
-static size_t trigger_text_to_ascii(char *buf, size_t max, cptr *strptr)
-{
-	cptr str = *strptr;
-	bool mod_status[MAX_MACRO_MOD];
-
-	int i, len = 0;
-	int shiftstatus = 0;
-	cptr key_code;
-	
-	size_t current_len = strlen(buf);
-
-	/* No definition of trigger names */
-	if (macro_template == NULL) return 0;
-
-	/* Initialize modifier key status */	
-	for (i = 0; macro_modifier_chr[i]; i++)
-		mod_status[i] = FALSE;
-
-	str++;
-
-	/* Examine modifier keys */
-	while (1)
-	{
-		/* Look for modifier key name */
-		for (i = 0; macro_modifier_chr[i]; i++)
-		{
-			len = strlen(macro_modifier_name[i]);
-
-			if (!my_strnicmp(str, macro_modifier_name[i], len))
-				break;
-		}
-
-		/* None found? */
-		if (!macro_modifier_chr[i]) break;
-
-		/* Proceed */
-		str += len;
-
-		/* This modifier key is pressed */
-		mod_status[i] = TRUE;
-
-		/* Shift key might be going to change keycode */
-		if (macro_modifier_chr[i] == 'S')
-			shiftstatus = 1;
-	}
-
-	/* Look for trigger name */
-	for (i = 0; i < max_macrotrigger; i++)
-	{
-		len = strlen(macro_trigger_name[i]);
-
-		/* Found it and it is ending with ']' */
-		if (!my_strnicmp(str, macro_trigger_name[i], len) && (']' == str[len]))
-			break;
-	}
-
-	/* Invalid trigger name? */
-	if (i == max_macrotrigger)
-	{
-		/*
-		 * If this invalid trigger name is ending with ']',
-		 * skip whole of it to avoid defining strange macro trigger
-		 */
-		str = strchr(str, ']');
-
-		if (str)
-		{
-			strnfcat(buf, max, &current_len, "\x1F\r");
-
-			*strptr = str; /* where **strptr == ']' */
-		}
-
-		return current_len;
-	}
-
-	/* Get keycode for this trigger name */
-	key_code = macro_trigger_keycode[shiftstatus][i];
-
-	/* Proceed */
-	str += len;
-
-	/* Begin with '^_' */
-	strnfcat(buf, max, &current_len, "\x1F");
-
-	/* Write key code style trigger using template */
-	for (i = 0; macro_template[i]; i++)
-	{
-		char ch = macro_template[i];
-
-		switch (ch)
-		{
-			/* Modifier key character */
-			case '&':
-			{
-				size_t j;
-				for (j = 0; macro_modifier_chr[j]; j++)
-				{
-					if (mod_status[j])
-						strnfcat(buf, max, &current_len, "%c", macro_modifier_chr[j]);
-				}
-				break;
-			}
-
-			/* Key code */
-			case '#':
-			{
-				strnfcat(buf, max, &current_len, "%s", key_code);
-				break;
-			}
-
-			/* Fixed string */
-			default:
-			{
-				strnfcat(buf, max, &current_len, "%c", ch);
-				break;
-			}
-		}
-	}
-
-	/* End with '\r' */
-	strnfcat(buf, max, &current_len, "\r");
-
-	/* Succeed */
-	*strptr = str; /* where **strptr == ']' */
-	
-	return current_len;
-}
-
-
-/*
- * Hack -- convert a printable string into real ascii
- *
- * This function will not work on non-ascii systems.
- *
- * To be safe, "buf" should be at least as large as "str".
- */
-void text_to_ascii(char *buf, size_t len, cptr str)
-{
-	char *s = buf;
-
-	/* Analyze the "ascii" string */
-	while (*str)
-	{
-		/* Check if the buffer is long enough */
-		if (s >= buf + len - 1) break;
-
-		/* Backslash codes */
-		if (*str == '\\')
-		{
-			str++;
-			if (*str == '\0') break;
-
-			switch (*str)
-			{
-				/* Macro trigger */
-				case '[':
-				{
-					/* Terminate before appending the trigger */
-					*s = '\0';
-					s += trigger_text_to_ascii(buf, len, &str);
-					break;
-				}
-
-				/* Hex-mode */
-				case 'x':
-				{
-					if (isxdigit((unsigned char)(*(str + 1))) &&
-					    isxdigit((unsigned char)(*(str + 2))))
-					{
-						*s = 16 * dehex(*++str);
-						*s++ += dehex(*++str);
-					}
-					else
-					{
-						/* HACK - Invalid hex number */
-						*s++ = '?';
-					}
-					break;
-				}
-
-				case 'e':
-					*s++ = ESCAPE;
-					break;
-				case 's':
-					*s++ = ' ';
-					break;
-				case 'b':
-					*s++ = '\b';
-					break;
-				case 'n':
-					*s++ = '\n';
-					break;
-				case 'r':
-					*s++ = '\r';
-					break;
-				case 't':
-					*s++ = '\t';
-					break;
-				case 'a':
-					*s++ = '\a';
-					break;
-				case '\\':
-					*s++ = '\\';
-					break;
-				case '^':
-					*s++ = '^';
-					break;
-
-				default:
-					*s = *str;
-					break;
-			}
-
-			/* Skip the final char */
-			str++;
-		}
-
-		/* Normal Control codes */
-		else if (*str == '^')
-		{
-			str++;
-			if (*str == '\0') break;
-
-			*s++ = KTRL(*str);
-			str++;
-		}
-
-		/* Normal chars */
-		else
-		{
-			*s++ = *str++;
-		}
-	}
-
-	/* Terminate */
-	*s = '\0';
-}
-
-
-/*
- * Transform macro trigger key code ('^_O_64\r' or etc..) 
- * into macro trigger name ('\[alt-D]' etc..)
- */
-static size_t trigger_ascii_to_text(char *buf, size_t max, cptr *strptr)
-{
-	cptr str = *strptr;
-	char key_code[100];
-	int i;
-	cptr tmp;
-	size_t current_len = strlen(buf);
-	
-
-	/* No definition of trigger names */
-	if (macro_template == NULL) return 0;
-
-	/* Trigger name will be written as '\[name]' */
-	strnfcat(buf, max, &current_len, "\\[");
-
-	/* Use template to read key-code style trigger */
-	for (i = 0; macro_template[i]; i++)
-	{
-		char ch = macro_template[i];
-
-		switch (ch)
-		{
-			/* Read modifier */
-			case '&':
-			{
-				size_t j;
-				while ((tmp = strchr(macro_modifier_chr, *str)) != 0)
-				{
-					j = tmp - macro_modifier_chr;
-					strnfcat(buf, max, &current_len, "%s", macro_modifier_name[j]);
-					str++;
-				}
-				break;
-			}
-
-			/* Read key code */
-			case '#':
-			{
-				size_t j;
-				for (j = 0; *str && (*str != '\r') && (j < sizeof(key_code) - 1); j++)
-					key_code[j] = *str++;
-				key_code[j] = '\0';
-				break;
-			}
-
-			/* Skip fixed strings */
-			default:
-			{
-				if (ch != *str) return 0;
-				str++;
-			}
-		}
-	}
-
-	/* Key code style triggers always end with '\r' */
-	if (*str++ != '\r') return 0;
-
-	/* Look for trigger name with given keycode (normal or shifted keycode) */
-	for (i = 0; i < max_macrotrigger; i++)
-	{
-		if (!my_stricmp(key_code, macro_trigger_keycode[0][i]) ||
-		    !my_stricmp(key_code, macro_trigger_keycode[1][i]))
-			break;
-	}
-
-	/* Not found? */
-	if (i == max_macrotrigger) return 0;
-
-	/* Write trigger name + "]" */
-	strnfcat(buf, max, &current_len, "%s]", macro_trigger_name[i]);
-
-	/* Succeed */
-	*strptr = str;
-	return current_len;
-}
-
-
-/*
- * Hack -- convert a string into a printable form
- *
- * This function will not work on non-ascii systems.
- */
-void ascii_to_text(char *buf, size_t len, cptr str)
-{
-	char *s = buf;
-
-	/* Analyze the "ascii" string */
-	while (*str)
-	{
-		byte i = (byte)(*str++);
-
-		/* Check if the buffer is long enough */
-		/* HACK - always assume worst case (hex-value + '\0') */
-		if (s >= buf + len - 5) break;
-
-		if (i == ESCAPE)
-		{
-			*s++ = '\\';
-			*s++ = 'e';
-		}
-		else if (i == ' ')
-		{
-			*s++ = '\\';
-			*s++ = 's';
-		}
-		else if (i == '\b')
-		{
-			*s++ = '\\';
-			*s++ = 'b';
-		}
-		else if (i == '\t')
-		{
-			*s++ = '\\';
-			*s++ = 't';
-		}
-		else if (i == '\a')
-		{
-			*s++ = '\\';
-			*s++ = 'a';
-		}
-		else if (i == '\n')
-		{
-			*s++ = '\\';
-			*s++ = 'n';
-		}
-		else if (i == '\r')
-		{
-			*s++ = '\\';
-			*s++ = 'r';
-		}
-		else if (i == '\\')
-		{
-			*s++ = '\\';
-			*s++ = '\\';
-		}
-		else if (i == '^')
-		{
-			*s++ = '\\';
-			*s++ = '^';
-		}
-		/* Macro Trigger */
-		else if (i == 31)
-		{
-			size_t offset;
-
-			/* Terminate before appending the trigger */
-			*s = '\0';
-
-			offset = trigger_ascii_to_text(buf, len, &str);
-			
-			if (offset == 0)
-			{
-				/* No trigger found */
-				*s++ = '^';
-				*s++ = '_';
-			}
-			else
-				s += offset;
-		}
-		else if (i < 32)
-		{
-			*s++ = '^';
-			*s++ = UN_KTRL(i);
-		}
-		else if (i < 127)
-		{
-			*s++ = i;
-		}
-		else
-		{
-			*s++ = '\\';
-			*s++ = 'x';
-			*s++ = hexify((int)i / 16);
-			*s++ = hexify((int)i % 16);
-		}
-	}
-
-	/* Terminate */
-	*s = '\0';
-}
+#include "game-event.h"
+#include "randname.h"
 
 
 /*
@@ -477,7 +30,7 @@ void ascii_to_text(char *buf, size_t len, cptr str)
  * 
  * Return the start position, or NULL if there isn't a valid suffix. 
  */
-char *find_roman_suffix_start(cptr buf)
+char *find_roman_suffix_start(const char *buf)
 {
 	const char *start = strrchr(buf, ' ');
 	const char *p;
@@ -583,7 +136,7 @@ int roman_to_int(const char *roman)
 	char *p;
 
 	char roman_token_chr1[] = "MDCLXVI";
-	char* roman_token_chr2[] = {0, 0, "DM", 0, "LC", 0, "VX"};
+	const char *roman_token_chr2[] = {0, 0, "DM", 0, "LC", 0, "VX"};
 
 	int roman_token_vals[7][3] = {{1000},
 	                              {500},
@@ -650,72 +203,21 @@ void flush(void)
 }
 
 
-/*
- * Flush all pending input if the OPT(flush_failure) option is set.
- */
-void flush_fail(void)
-{
-	if (OPT(flush_failure)) flush();
-}
-
-
-/*
- * Local variable -- we are inside a "macro action"
- *
- * Do not match any macros until "ascii 30" is found.
- */
-static bool parse_macro = FALSE;
-
-
-/*
- * Local variable -- we are inside a "macro trigger"
- *
- * Strip all keypresses until a low ascii value is found.
- */
-static bool parse_under = FALSE;
-
 
 
 /*
  * Helper function called only from "inkey()"
- *
- * This function does almost all of the "macro" processing.
- *
- * We use the "Term_key_push()" function to handle "failed" macros, as well
- * as "extra" keys read in while choosing the proper macro, and also to hold
- * the action for the macro, plus a special "ascii 30" character indicating
- * that any macro action in progress is complete.  Embedded macros are thus
- * illegal, unless a macro action includes an explicit "ascii 30" character,
- * which would probably be a massive hack, and might break things.
- *
- * Only 500 (0+1+2+...+29+30) milliseconds may elapse between each key in
- * the macro trigger sequence.  If a key sequence forms the "prefix" of a
- * macro trigger, 500 milliseconds must pass before the key sequence is
- * known not to be that macro trigger.  XXX XXX XXX
  */
-static ui_event_data inkey_aux(int scan_cutoff)
+static ui_event inkey_aux(int scan_cutoff)
 {
-	int k = 0, n, p = 0, w = 0;
-	
-	ui_event_data ke, ke0;
-	char ch;
-	
-	cptr pat, act;
-	
-	char buf[1024];
+	int w = 0;	
 
-	/* Initialize the no return */
-	ke0.type = EVT_NONE;
-	ke0.key = 0;
-	ke0.index = 0; /* To fix GCC warnings on X11 */
-	ke0.mousey = 0;
-	ke0.mousex = 0;
- 
+	ui_event ke;
+	
 	/* Wait for a keypress */
 	if (scan_cutoff == SCAN_OFF)
 	{
 		(void)(Term_inkey(&ke, TRUE, TRUE));
-		ch = ke.key;
 	}
 	else
 	{
@@ -730,142 +232,16 @@ static ui_event_data inkey_aux(int scan_cutoff)
 			/* Excessive delay */
 			if (w >= scan_cutoff)
 			{
-				ke0.type = EVT_KBRD;
-				return ke0;
+				ui_event empty = EVENT_EMPTY;
+				return empty;
 			}
 
 			/* Delay */
 			Term_xtra(TERM_XTRA_DELAY, 10);
 		}
-		ch = ke.key;
 	}
 
-	
-	/* End "macro action" */
-	if (ke.key == 30 || ke.type == EVT_MOUSE)
-	{
-		parse_macro = FALSE;
-		return (ke);
-	}
-	
-	/* Inside "macro action" */
-	if (parse_macro) return (ke);
-	
-	/* Inside "macro trigger" */
-	if (parse_under) return (ke);
-	
-
-	/* Save the first key, advance */
-	buf[p++] = ke.key;
-	buf[p] = '\0';
-	
-	
-	/* Check for possible macro */
-	k = macro_find_check(buf);
-	
-	/* No macro pending */
-	if (k < 0) return (ke);
-	
-	
-	/* Wait for a macro, or a timeout */
-	while (TRUE)
-	{
-		/* Check for pending macro */
-		k = macro_find_maybe(buf);
-		
-		/* No macro pending */
-		if (k < 0) break;
-		
-		/* Check for (and remove) a pending key */
-		if (0 == Term_inkey(&ke, FALSE, TRUE))
-		{
-			/* Append the key */
-			buf[p++] = ke.key;
-			buf[p] = '\0';
-		
-			/* Restart wait */
-			w = 0;
-		}
-		
-		/* No key ready */
-		else
-		{
-			/* Increase "wait" */
-			w ++;
-		
-			/* Excessive delay */
-			if (w >= SCAN_MACRO) break;
-		
-			/* Delay */
-			Term_xtra(TERM_XTRA_DELAY, 10);
-		}
-	}
-	
-	
-	/* Check for available macro */
-	k = macro_find_ready(buf);
-
-	/* No macro available */
-	if (k < 0)
-	{
-		/* Push all the "keys" back on the queue */
-		/* The most recent event may not be a keypress. */
-		if(p)
-		{
-			if(Term_event_push(&ke)) return (ke0);
-			p--;
-		}
-		while (p > 0)
-		{
-			/* Push the key, notice over-flow */
-			if (Term_key_push(buf[--p])) return (ke0);
-		}
-		
-		/* Wait for (and remove) a pending key */
-		(void)Term_inkey(&ke, TRUE, TRUE);
-		
-		/* Return the key */
-		return (ke);
-	}
-	
-	
-	/* Get the pattern */
-	pat = macro__pat[k];
-	
-	/* Get the length of the pattern */
-	n = strlen(pat);
-	
-	/* Push the "extra" keys back on the queue */
-	while (p > n)
-	{
-		/* Push the key, notice over-flow */
-		if (Term_key_push(buf[--p])) return (ke0);
-	}
-	
-	
-	/* Begin "macro action" */
-	parse_macro = TRUE;
-	
-	/* Push the "end of macro action" key */
-	if (Term_key_push(30)) return (ke0);
-	
-	
-	/* Access the macro action */
-	act = macro__act[k];
-	
-	/* Get the length of the action */
-	n = strlen(act);
-	
-	/* Push the macro "action" onto the key queue */
-	while (n > 0)
-	{
-		/* Push the key, notice over-flow */
-		if (Term_key_push(act[--n])) return (ke0);
-	}
-	
-	
-	/* Hack -- Force "inkey()" to call us again */
-	return (ke0);
+	return (ke);
 }
 
 
@@ -874,11 +250,10 @@ static ui_event_data inkey_aux(int scan_cutoff)
  * Mega-Hack -- special "inkey_next" pointer.  XXX XXX XXX
  *
  * This special pointer allows a sequence of keys to be "inserted" into
- * the stream of keys returned by "inkey()".  This key sequence will not
- * trigger any macros, and cannot be bypassed by the Borg.  It is used
- * in Angband to handle "keymaps".
+ * the stream of keys returned by "inkey()".  This key sequence cannot be
+ * bypassed by the Borg.  We use it to implement keymaps.
  */
-cptr inkey_next = NULL;
+struct keypress *inkey_next = NULL;
 
 
 #ifdef ALLOW_BORG
@@ -889,7 +264,7 @@ cptr inkey_next = NULL;
  * This special function hook allows the "Borg" (see elsewhere) to take
  * control of the "inkey()" function, and substitute in fake keypresses.
  */
-char (*inkey_hack)(int flush_first) = NULL;
+struct keypress (*inkey_hack)(int flush_first) = NULL;
 
 #endif /* ALLOW_BORG */
 
@@ -903,46 +278,21 @@ char (*inkey_hack)(int flush_first) = NULL;
  * before this function returns.  Thus they function just like normal
  * parameters, except that most calls to this function can ignore them.
  *
- * If "inkey_xtra" is TRUE, then all pending keypresses will be flushed,
- * and any macro processing in progress will be aborted.  This flag is
- * set by the "flush()" function, which does not actually flush anything
- * itself, but rather, triggers delayed input flushing via "inkey_xtra".
+ * If "inkey_xtra" is TRUE, then all pending keypresses will be flushed.
+ * This is set by flush(), which doesn't actually flush anything itself
+ * but uses that flag to trigger delayed flushing.
  *
  * If "inkey_scan" is TRUE, then we will immediately return "zero" if no
  * keypress is available, instead of waiting for a keypress.
  *
- * If "inkey_base" is TRUE, then all macro processing will be bypassed.
- * If "inkey_base" and "inkey_scan" are both TRUE, then this function will
- * not return immediately, but will wait for a keypress for as long as the
- * normal macro matching code would, allowing the direct entry of macro
- * triggers.  The "inkey_base" flag is extremely dangerous!
- *
- * If "inkey_flag" is TRUE, then we will assume that we are waiting for a
- * normal command, and we will only show the cursor if "OPT(highlight_player)" is
- * TRUE (or if the player is in a store), instead of always showing the
- * cursor.  The various "main-xxx.c" files should avoid saving the game
- * in response to a "menu item" request unless "inkey_flag" is TRUE, to
- * prevent savefile corruption.
+ * If "inkey_flag" is TRUE, then we are waiting for a command in the main
+ * map interface, and we shouldn't show a cursor.
  *
  * If we are waiting for a keypress, and no keypress is ready, then we will
  * refresh (once) the window which was active when this function was called.
  *
  * Note that "back-quote" is automatically converted into "escape" for
- * convenience on machines with no "escape" key.  This is done after the
- * macro matching, so the user can still make a macro for "backquote".
- *
- * Note the special handling of "ascii 30" (ctrl-caret, aka ctrl-shift-six)
- * and "ascii 31" (ctrl-underscore, aka ctrl-shift-minus), which are used to
- * provide support for simple keyboard "macros".  These keys are so strange
- * that their loss as normal keys will probably be noticed by nobody.  The
- * "ascii 30" key is used to indicate the "end" of a macro action, which
- * allows recursive macros to be avoided.  The "ascii 31" key is used by
- * some of the "main-xxx.c" files to introduce macro trigger sequences.
- *
- * Hack -- we use "ascii 29" (ctrl-right-bracket) as a special "magic" key,
- * which can be used to give a variety of "sub-commands" which can be used
- * any time.  These sub-commands could include commands to take a picture of
- * the current screen, to start/stop recording a macro action, etc.
+ * convenience on machines with no "escape" key.
  *
  * If "angband_term[0]" is not active, we will make it active during this
  * function, so that the various "main-xxx.c" files can assume that input
@@ -951,34 +301,34 @@ char (*inkey_hack)(int flush_first) = NULL;
  * Mega-Hack -- This function is used as the entry point for clearing the
  * "signal_count" variable, and of the "character_saved" variable.
  *
- * Hack -- Note the use of "inkey_next" to allow "keymaps" to be processed.
- *
  * Mega-Hack -- Note the use of "inkey_hack" to allow the "Borg" to steal
  * control of the keyboard from the user.
  */
-ui_event_data inkey_ex(void)
+ui_event inkey_ex(void)
 {
 	bool cursor_state;
-	ui_event_data kk;
-	ui_event_data ke;
+	ui_event kk;
+	ui_event ke = EVENT_EMPTY;
 
 	bool done = FALSE;
 
 	term *old = Term;
 
-	/* Initialise keypress */
-	ke.key = 0;
-	ke.type = EVT_NONE;
+	/* Delayed flush */
+	if (inkey_xtra) {
+		Term_flush();
+		inkey_next = NULL;
+		inkey_xtra = FALSE;
+	}
 
 	/* Hack -- Use the "inkey_next" pointer */
-	if (inkey_next && *inkey_next && !inkey_xtra)
+	if (inkey_next && inkey_next->code)
 	{
 		/* Get next character, and advance */
 		ke.key = *inkey_next++;
-		ke.type = EVT_KBRD;
 
 		/* Cancel the various "global parameters" */
-		inkey_base = inkey_xtra = inkey_flag = FALSE;
+		inkey_flag = FALSE;
 		inkey_scan = 0;
 
 		/* Accept result */
@@ -994,7 +344,7 @@ ui_event_data inkey_ex(void)
 	if (inkey_hack && ((ke.key = (*inkey_hack)(inkey_xtra)) != 0))
 	{
 		/* Cancel the various "global parameters" */
-		inkey_base = inkey_xtra = inkey_flag = FALSE;
+		inkey_flag = FALSE;
 		inkey_scan = 0;
 		ke.type = EVT_KBRD;
 
@@ -1004,29 +354,13 @@ ui_event_data inkey_ex(void)
 
 #endif /* ALLOW_BORG */
 
-	/* Hack -- handle delayed "flush()" */
-	if (inkey_xtra)
-	{
-		/* End "macro action" */
-		parse_macro = FALSE;
-
-		/* End "macro trigger" */
-		parse_under = FALSE;
-
-		/* Forget old keypresses */
-		Term_flush();
-	}
-
 
 	/* Get the cursor state */
 	(void)Term_get_cursor(&cursor_state);
 
 	/* Show the cursor if waiting, except sometimes in "command" mode */
-	if (!inkey_scan && (!inkey_flag || OPT(highlight_player) || character_icky))
-	{
-		/* Show the cursor */
+	if (!inkey_scan && (!inkey_flag || character_icky))
 		(void)Term_set_cursor(TRUE);
-	}
 
 
 	/* Hack -- Activate main screen */
@@ -1037,18 +371,14 @@ ui_event_data inkey_ex(void)
 	while (ke.type == EVT_NONE)
 	{
 		/* Hack -- Handle "inkey_scan == SCAN_INSTANT */
-		if (!inkey_base && inkey_scan == SCAN_INSTANT &&
-		   (0 != Term_inkey(&kk, FALSE, FALSE)))
-		{
-			ke.type = EVT_KBRD;
+		if (inkey_scan == SCAN_INSTANT &&
+				(0 != Term_inkey(&kk, FALSE, FALSE)))
 			break;
-		}
 
 
 		/* Hack -- Flush output once when no key ready */
 		if (!done && (0 != Term_inkey(&kk, FALSE, FALSE)))
 		{
-
 			/* Hack -- activate proper term */
 			Term_activate(old);
 
@@ -1069,57 +399,6 @@ ui_event_data inkey_ex(void)
 		}
 
 
-		/* Hack -- Handle "inkey_base" */
-		if (inkey_base)
-		{
-			int w = 0;
-
-			/* Wait forever */
-			if (!inkey_scan)
-			{
-				/* Wait for (and remove) a pending key */
-				if (0 == Term_inkey(&ke, TRUE, TRUE))
-				{
-					/* Done */
-					ke.type = EVT_KBRD;
-					break;
-				}
-
-				/* Oops */
-				break;
-			}
-
-			/* Wait only as long as macro activation would wait*/
-			while (TRUE)
-			{
-				/* Check for (and remove) a pending key */
-				if (0 == Term_inkey(&ke, FALSE, TRUE))
-				{
-					/* Done */
-					ke.type = EVT_KBRD;
-					break;
-				}
-
-				/* No key ready */
-				else
-				{
-					/* Increase "wait" */
-					w ++;
-
-					/* Excessive delay */
-					if (w >= SCAN_MACRO) break;
-
-					/* Delay */
-					Term_xtra(TERM_XTRA_DELAY, 10);
-
-				}
-			}
-
-			/* Done */
-			ke.type = EVT_KBRD;
-			break;
-		}
-
 		/* Get a key (see above) */
 		ke = inkey_aux(inkey_scan);
 
@@ -1129,74 +408,24 @@ ui_event_data inkey_ex(void)
 			/* Check to see if we've hit a button */
 			/* Assuming text buttons here for now - this would have to
 			 * change for GUI buttons */
-			char key = button_get_key(ke.mousex, ke.mousey);
+			char key = button_get_key(ke.mouse.x, ke.mouse.y);
 
 			if (key)
 			{
 				/* Rewrite the event */
+				/* XXXmacro button implementation needs updating */
 				ke.type = EVT_BUTTON;
-				ke.key = key;
-				ke.index = 0;
-				ke.mousey = 0;
-				ke.mousex = 0;
+				ke.key.code = key;
+				ke.key.mods = 0;
 
 				/* Done */
 				break;
 			}
 		}
 
-		/* Handle "control-right-bracket" */
-		if (ke.key == 29)
-		{
-			/* Strip this key */
-			ke.type = EVT_NONE;
-
-			/* Continue */
-			continue;
-		}
-
-
 		/* Treat back-quote as escape */
-		if (ke.key == '`') ke.key = ESCAPE;
-
-
-		/* End "macro trigger" */
-		if (parse_under && (ke.key >=0 && ke.key <= 32))
-		{
-			/* Strip this key */
-			ke.type = EVT_NONE;
-			ke.key = 0;
-
-			/* End "macro trigger" */
-			parse_under = FALSE;
-		}
-
-		/* Handle "control-caret" */
-		if (ke.key == 30)
-		{
-			/* Strip this key */
-			ke.type = EVT_NONE;
-			ke.key = 0;
-		}
-
-		/* Handle "control-underscore" */
-		else if (ke.key == 31)
-		{
-			/* Strip this key */
-			ke.type = EVT_NONE;
-			ke.key = 0;
-
-			/* Begin "macro trigger" */
-			parse_under = TRUE;
-		}
-
-		/* Inside "macro trigger" */
-    	else if (parse_under)
-		{
-			/* Strip this key */
-			ke.type = EVT_NONE;
-			ke.key = 0;
-		}
+		if (ke.key.code == '`')
+			ke.key.code = ESCAPE;
 	}
 
 
@@ -1209,7 +438,7 @@ ui_event_data inkey_ex(void)
 
 
 	/* Cancel the various "global parameters" */
-	inkey_base = inkey_xtra = inkey_flag = FALSE;
+	inkey_flag = FALSE;
 	inkey_scan = 0;
 
 	/* Return the keypress */
@@ -1220,34 +449,33 @@ ui_event_data inkey_ex(void)
 /*
  * Get a keypress or mouse click from the user.
  */
-char anykey(void)
+void anykey(void)
 {
-	ui_event_data ke = EVENT_EMPTY;
+	ui_event ke = EVENT_EMPTY;
   
-	/* Only accept a keypress or mouse click*/
-	do
-	{
+	/* Only accept a keypress or mouse click */
+	while (ke.type != EVT_MOUSE && ke.type != EVT_KBRD)
 		ke = inkey_ex();
-	} while ((ke.type != EVT_MOUSE) && (ke.type != EVT_KBRD));
-
-	return ke.key;
 }
 
 /*
  * Get a "keypress" from the user.
  */
-char inkey(void)
+struct keypress inkey(void)
 {
-	ui_event_data ke = EVENT_EMPTY;
+	ui_event ke = EVENT_EMPTY;
 
 	/* Only accept a keypress */
-	do
-	{
+	while (ke.type != EVT_ESCAPE && ke.type != EVT_KBRD)
 		ke = inkey_ex();
-	} while ((ke.type != EVT_ESCAPE) && (ke.type != EVT_KBRD));
 
 	/* Paranoia */
-	if (ke.type == EVT_ESCAPE) ke.key = ESCAPE;
+	if (ke.type == EVT_ESCAPE) {
+		ke.type = EVT_KBRD;
+		ke.key.code = ESCAPE;
+		ke.key.mods = 0;
+	}
+
 	return ke.key;
 }
 
@@ -1256,7 +484,7 @@ char inkey(void)
 /*
  * Flush the screen, make a noise
  */
-void bell(cptr reason)
+void bell(const char *reason)
 {
 	/* Mega-Hack -- Flush the output */
 	Term_fresh();
@@ -1268,12 +496,9 @@ void bell(cptr reason)
 
 		/* Window stuff */
 		p_ptr->redraw |= (PR_MESSAGE);
-		redraw_stuff();
+		redraw_stuff(p_ptr);
 	}
 
-	/* Make a bell noise (if allowed) */
-	if (OPT(ring_bell)) Term_xtra(TERM_XTRA_NOISE, 0);
-
 	/* Flush the input (later!) */
 	flush();
 }
@@ -1304,18 +529,7 @@ static void msg_flush(int x)
 	Term_putstr(x, 0, -1, a, "-more-");
 
 	if (!OPT(auto_more))
-	{
-		/* Get an acceptable keypress */
-		while (1)
-		{
-			char ch;
-			ch = inkey();
-			if (OPT(quick_messages)) break;
-			if ((ch == ESCAPE) || (ch == ' ')) break;
-			if ((ch == '\n') || (ch == '\r')) break;
-			bell("Illegal response to a 'more' prompt!");
-		}
-	}
+		anykey();
 
 	/* Clear the line */
 	Term_erase(0, 0, 255);
@@ -1342,15 +556,15 @@ static int message_column = 0;
  * "pending" messages still on the screen, instead of using "msg_flush()".
  * This should only be done when the user is known to have read the message.
  *
- * We must be very careful about using the "msg_print()" functions without
- * explicitly calling the special "msg_print(NULL)" function, since this may
+ * We must be very careful about using the "msg("%s", )" functions without
+ * explicitly calling the special "msg("%s", NULL)" function, since this may
  * result in the loss of information if the screen is cleared, or if anything
  * is displayed on the top line.
  *
- * Hack -- Note that "msg_print(NULL)" will clear the top line even if no
+ * Hack -- Note that "msg("%s", NULL)" will clear the top line even if no
  * messages are pending.
  */
-static void msg_print_aux(u16b type, cptr msg)
+static void msg_print_aux(u16b type, const char *msg)
 {
 	int n;
 	char *t;
@@ -1358,6 +572,8 @@ static void msg_print_aux(u16b type, cptr msg)
 	byte color;
 	int w, h;
 
+	if (!Term)
+		return;
 
 	/* Obtain the size */
 	(void)Term_get_size(&w, &h);
@@ -1406,21 +622,18 @@ static void msg_print_aux(u16b type, cptr msg)
 	color = message_type_color(type);
 
 	/* Split message */
-	while (n > (w - 8))
+	while (n > w - 1)
 	{
 		char oops;
 
 		int check, split;
 
 		/* Default split */
-		split = (w - 8);
+		split = w - 8;
 
-		/* Find the "best" split point */
-		for (check = (w / 2); check < (w - 8); check++)
-		{
-			/* Found a valid split point */
+		/* Find the rightmost split point */
+		for (check = (w / 2); check < w - 8; check++)
 			if (t[check] == ' ') split = check;
-		}
 
 		/* Save the split character */
 		oops = t[split];
@@ -1457,20 +670,10 @@ static void msg_print_aux(u16b type, cptr msg)
 	event_signal(EVENT_MESSAGE);
 }
 
-
 /*
- * Print a message in the default color (white)
+ * Display a formatted message, using "vstrnfmt()" and "msg("%s", )".
  */
-void msg_print(cptr msg)
-{
-	msg_print_aux(MSG_GENERIC, msg);
-}
-
-
-/*
- * Display a formatted message, using "vstrnfmt()" and "msg_print()".
- */
-void msg_format(cptr fmt, ...)
+void msg(const char *fmt, ...)
 {
 	va_list vp;
 
@@ -1489,49 +692,17 @@ void msg_format(cptr fmt, ...)
 	msg_print_aux(MSG_GENERIC, buf);
 }
 
-
-/*
- * Display a message and play the associated sound.
- *
- * The "extra" parameter is currently unused.
- */
-void message(u16b message_type, s16b extra, cptr message)
-{
-	/* Unused parameter */
-	(void)extra;
-
-	sound(message_type);
-
-	msg_print_aux(message_type, message);
-}
-
-
-
-/*
- * Display a formatted message and play the associated sound.
- *
- * The "extra" parameter is currently unused.
- */
-void message_format(u16b message_type, s16b extra, cptr fmt, ...)
+void msgt(unsigned int type, const char *fmt, ...)
 {
 	va_list vp;
-
 	char buf[1024];
-
-	/* Begin the Varargs Stuff */
 	va_start(vp, fmt);
-
-	/* Format the args, save the length */
-	(void)vstrnfmt(buf, sizeof(buf), fmt, vp);
-
-	/* End the Varargs Stuff */
+	vstrnfmt(buf, sizeof(buf), fmt, vp);
 	va_end(vp);
-
-	/* Display */
-	message(message_type, extra, buf);
+	sound(type);
+	msg_print_aux(type, buf);
 }
 
-
 /*
  * Print the queued messages.
  */
@@ -1544,7 +715,8 @@ void message_flush(void)
 	if (message_column)
 	{
 		/* Print pending messages */
-		msg_flush(message_column);
+		if (Term)
+			msg_flush(message_column);
 
 		/* Forget it */
 		msg_flag = FALSE;
@@ -1602,7 +774,7 @@ void screen_load(void)
  * At the given location, using the given attribute, if allowed,
  * add the given string.  Do not clear the line.
  */
-void c_put_str(byte attr, cptr str, int row, int col)
+void c_put_str(byte attr, const char *str, int row, int col)
 {
 	/* Position cursor, Dump the attr/text */
 	Term_putstr(col, row, -1, attr, str);
@@ -1612,7 +784,7 @@ void c_put_str(byte attr, cptr str, int row, int col)
 /*
  * As above, but in "white"
  */
-void put_str(cptr str, int row, int col)
+void put_str(const char *str, int row, int col)
 {
 	/* Spawn */
 	Term_putstr(col, row, -1, TERM_WHITE, str);
@@ -1624,7 +796,7 @@ void put_str(cptr str, int row, int col)
  * Display a string on the screen using an attribute, and clear
  * to the end of the line.
  */
-void c_prt(byte attr, cptr str, int row, int col)
+void c_prt(byte attr, const char *str, int row, int col)
 {
 	/* Clear line, position cursor */
 	Term_erase(col, row, 255);
@@ -1637,7 +809,7 @@ void c_prt(byte attr, cptr str, int row, int col)
 /*
  * As above, but in "white"
  */
-void prt(cptr str, int row, int col)
+void prt(const char *str, int row, int col)
 {
 	/* Spawn */
 	c_prt(TERM_WHITE, str, row, col);
@@ -1658,7 +830,7 @@ void prt(cptr str, int row, int col)
  * This function will correctly handle any width up to the maximum legal
  * value of 256, though it works best for a standard 80 character width.
  */
-void text_out_to_screen(byte a, cptr str)
+void text_out_to_screen(byte a, const char *str)
 {
 	int x, y;
 
@@ -1666,7 +838,7 @@ void text_out_to_screen(byte a, cptr str)
 
 	int wrap;
 
-	cptr s;
+	const char *s;
 	char buf[1024];
 
 	/* We use either ascii or system-specific encoding */
@@ -1788,9 +960,9 @@ void text_out_to_screen(byte a, cptr str)
  * You must be careful to end all file output with a newline character
  * to "flush" the stored line position.
  */
-void text_out_to_file(byte a, cptr str)
+void text_out_to_file(byte a, const char *str)
 {
-	cptr s;
+	const char *s;
 	char buf[1024];
 
 	/* Current position on the line */
@@ -2131,9 +1303,9 @@ void clear_from(int row)
  * It should return TRUE when editing of the buffer is "complete" (e.g. on
  * the press of RETURN).
  */
-bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, char keypress, bool firsttime)
+bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, struct keypress keypress, bool firsttime)
 {
-	switch (keypress)
+	switch (keypress.code)
 	{
 		case ESCAPE:
 		{
@@ -2197,8 +1369,7 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, ch
 		{
 			bool atnull = (buf[*curs] == 0);
 
-
-			if (!my_isprint((unsigned char)keypress))
+			if (!my_isprint((unsigned char)keypress.code))
 			{
 				bell("Illegal edit key!");
 				break;
@@ -2228,7 +1399,7 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, ch
 			}
 
 			/* Insert the character */
-			buf[(*curs)++] = keypress;
+			buf[(*curs)++] = (char)keypress.code;
 			(*len)++;
 
 			/* Terminate */
@@ -2265,14 +1436,14 @@ bool askfor_aux_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, ch
  * 'askfor_aux_keypress' (the default handler if you supply NULL for
  * 'keypress_h') for an example.
  */
-bool askfor_aux(char *buf, size_t len, bool keypress_h(char *, size_t, size_t *, size_t *, char, bool))
+bool askfor_aux(char *buf, size_t len, bool keypress_h(char *, size_t, size_t *, size_t *, struct keypress, bool))
 {
 	int y, x;
 
 	size_t k = 0;		/* Cursor position */
 	size_t nul = 0;		/* Position of the null byte in the string */
 
-	char ch = '\0';
+	struct keypress ch = { 0 };
 
 	bool done = FALSE;
 	bool firsttime = TRUE;
@@ -2324,7 +1495,7 @@ bool askfor_aux(char *buf, size_t len, bool keypress_h(char *, size_t, size_t *,
 	}
 
 	/* Done */
-	return (ch != ESCAPE);
+	return (ch.code != ESCAPE);
 }
 
 
@@ -2333,11 +1504,11 @@ bool askfor_aux(char *buf, size_t len, bool keypress_h(char *, size_t, size_t *,
  * case of '*' for a new random "name" and passes any other "keypress"
  * through to the default "editing" handler.
  */
-static bool get_name_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, char keypress, bool firsttime)
+static bool get_name_keypress(char *buf, size_t buflen, size_t *curs, size_t *len, struct keypress keypress, bool firsttime)
 {
 	bool result;
 
-	switch (keypress)
+	switch (keypress.code)
 	{
 		case '*':
 		{
@@ -2347,8 +1518,7 @@ static bool get_name_keypress(char *buf, size_t buflen, size_t *curs, size_t *le
 			result = FALSE;
 			break;
 		}
-		
-		
+
 		default:
 		{
 			result = askfor_aux_keypress(buf, buflen, curs, len, keypress, firsttime);
@@ -2405,7 +1575,7 @@ bool get_name(char *buf, size_t buflen)
  * See "askfor_aux" for some notes about "buf" and "len", and about
  * the return value of this function.
  */
-bool get_string(cptr prompt, char *buf, size_t len)
+bool get_string(const char *prompt, char *buf, size_t len)
 {
 	bool res;
 
@@ -2435,7 +1605,7 @@ bool get_string(cptr prompt, char *buf, size_t len)
  *
  * Allow "p_ptr->command_arg" to specify a quantity
  */
-s16b get_quantity(cptr prompt, int max)
+s16b get_quantity(const char *prompt, int max)
 {
 	int amt = 1;
 
@@ -2498,9 +1668,9 @@ s16b get_quantity(cptr prompt, int max)
  *
  * Note that "[y/n]" is appended to the prompt.
  */
-bool get_check(cptr prompt)
+bool get_check(const char *prompt)
 {
-	ui_event_data ke;
+	struct keypress ke;
 
 	char buf[80];
 
@@ -2518,20 +1688,11 @@ bool get_check(cptr prompt)
 	/* Make some buttons */
 	button_add("[y]", 'y');
 	button_add("[n]", 'n');
-	redraw_stuff();
+	redraw_stuff(p_ptr);
   
 	/* Prompt for it */
 	prt(buf, 0, 0);
-
-	/* Get an acceptable answer */
-	while (TRUE)
-	{
-		ke = inkey_ex();
-		if (OPT(quick_messages)) break;
-		if (ke.key == ESCAPE) break;
-		if (strchr("YyNn", ke.key)) break;
-		bell("Illegal response to a 'yes/no' question!");
-	}
+	ke = inkey();
 
 	/* Kill the buttons */
 	button_kill('y');
@@ -2539,13 +1700,13 @@ bool get_check(cptr prompt)
 
 	/* Hack - restore the repeat button */
 	if (repeat) button_add("[Rpt]", 'n');
-	redraw_stuff();
+	redraw_stuff(p_ptr);
   
 	/* Erase the prompt */
 	prt("", 0, 0);
 
 	/* Normal negation */
-	if ((ke.key != 'Y') && (ke.key != 'y')) return (FALSE);
+	if ((ke.code != 'Y') && (ke.code != 'y')) return (FALSE);
 
 	/* Success */
 	return (TRUE);
@@ -2561,10 +1722,11 @@ bool get_check(cptr prompt)
  *     This prompts "Study? [yns]" and defaults to 'n'.
  *
  */
-char get_char(cptr prompt, const char *options, size_t len, char fallback)
+char get_char(const char *prompt, const char *options, size_t len, char fallback)
 {
 	size_t i;
-	char button[4], buf[80], key;
+	struct keypress key;
+	char button[4], buf[80];
 	bool repeat = FALSE;
   
 	/* Paranoia XXX XXX XXX */
@@ -2582,43 +1744,33 @@ char get_char(cptr prompt, const char *options, size_t len, char fallback)
 		strnfmt(button, 4, "[%c]", options[i]);
 		button_add(button, options[i]);
 	}
-	redraw_stuff();
+	redraw_stuff(p_ptr);
   
 	/* Prompt for it */
 	prt(buf, 0, 0);
 
 	/* Get an acceptable answer */
-	while (TRUE)
-	{
-		key = inkey_ex().key;
+	key = inkey();
 
-		/* Lowercase answer if necessary */
-		if (key >= 'A' && key <= 'Z') key += 32;
+	/* Lowercase answer if necessary */
+	if (key.code >= 'A' && key.code <= 'Z') key.code += 32;
 
-		/* See if key is in our options string */
-		if (strchr(options, key)) break;
-
-		/* If we want to escape, return the fallback */
-		if (key == ESCAPE || OPT(quick_messages)) 
-		{
-			key = fallback;
-			break;
-		}
-		bell("Illegal response!");
-	}
+	/* See if key is in our options string */
+	if (!strchr(options, (char)key.code))
+		key.code = fallback;
 
 	/* Kill the buttons */
 	for (i = 0; i < len; i++) button_kill(options[i]);
 
 	/* Hack - restore the repeat button */
 	if (repeat) button_add("[Rpt]", 'n');
-	redraw_stuff();
+	redraw_stuff(p_ptr);
   
 	/* Erase the prompt */
 	prt("", 0, 0);
 
 	/* Success */
-	return key;
+	return key.code;
 }
 
 
@@ -2671,9 +1823,9 @@ bool (*get_file)(const char *suggested_name, char *path, size_t len) = get_file_
  *
  * Returns TRUE unless the character is "Escape"
  */
-bool get_com(cptr prompt, char *command)
+bool get_com(const char *prompt, struct keypress *command)
 {
-	ui_event_data ke;
+	ui_event ke;
 	bool result;
 
 	result = get_com_ex(prompt, &ke);
@@ -2683,9 +1835,9 @@ bool get_com(cptr prompt, char *command)
 }
 
 
-bool get_com_ex(cptr prompt, ui_event_data *command)
+bool get_com_ex(const char *prompt, ui_event *command)
 {
-	ui_event_data ke;
+	ui_event ke;
 
 	/* Paranoia XXX XXX XXX */
 	message_flush();
@@ -2703,7 +1855,9 @@ bool get_com_ex(cptr prompt, ui_event_data *command)
 	*command = ke;
 
 	/* Done */
-	return (ke.key != ESCAPE);
+	if (ke.type == EVT_KBRD && ke.key.code == ESCAPE)
+		return FALSE;
+	return TRUE;
 }
 
 
@@ -2712,17 +1866,14 @@ bool get_com_ex(cptr prompt, ui_event_data *command)
  *
  * This function is stupid.  XXX XXX XXX
  */
-void pause_line(int row)
+void pause_line(struct term *term)
 {
-	prt("", row, 0);
-	put_str("[Press any key to continue]", row, 23);
+	prt("", term->hgt - 1, 0);
+	put_str("[Press any key to continue]", term->hgt - 1, 23);
 	(void)anykey();
-	prt("", row, 0);
+	prt("", term->hgt - 1, 0);
 }
 
-
-
-
 /*
  * Check a char for "vowel-hood"
  */
@@ -2778,7 +1929,7 @@ int color_char_to_attr(char c)
 /*
  * Converts a string to a terminal color byte.
  */
-int color_text_to_attr(cptr name)
+int color_text_to_attr(const char *name)
 {
 	int a;
 
@@ -2795,7 +1946,7 @@ int color_text_to_attr(cptr name)
 /*
  * Extract a textual representation of an attribute
  */
-cptr attr_to_text(byte a)
+const char *attr_to_text(byte a)
 {
 	if (a < BASIC_COLORS)
 		return (color_table[a].name);
