@@ -17,7 +17,9 @@
  */
 
 #include "angband.h"
+#include "buildid.h"
 #include "cmds.h"
+#include "monster/mon-lore.h"
 #include "monster/monster.h"
 #include "object/tvalsval.h"
 #include "ui-menu.h"
@@ -53,7 +55,7 @@ static void spoiler_blanklines(int n)
 /*
  * Write a line to the spoiler file and then "underline" it with hypens
  */
-static void spoiler_underline(cptr str, char c)
+static void spoiler_underline(const char *str, char c)
 {
 	text_out("%s", str);
 	text_out("\n");
@@ -137,7 +139,7 @@ static void kind_info(char *buf, size_t buf_len,
 
 	object_type *i_ptr;
 	object_type object_type_body;
-
+	int i;
 
 	/* Get local object */
 	i_ptr = &object_type_body;
@@ -146,10 +148,11 @@ static void kind_info(char *buf, size_t buf_len,
 	object_prep(i_ptr, &k_info[k], 0, MAXIMISE);
 
 	/* Obtain the "kind" info */
-	k_ptr = &k_info[i_ptr->k_idx];
+	k_ptr = i_ptr->kind;
 
 	/* Cancel bonuses */
-	i_ptr->pval = 0;
+	for (i = 0; i < MAX_PVALS; i++)
+		i_ptr->pval[i] = 0;
 	i_ptr->to_a = 0;
 	i_ptr->to_h = 0;
 	i_ptr->to_d = 0;
@@ -164,8 +167,6 @@ static void kind_info(char *buf, size_t buf_len,
 	/* Value */
 	(*val) = object_value(i_ptr, 1, FALSE);
 
-
-
 	/* Description (too brief) */
 	if (buf)
 		object_desc(buf, buf_len, i_ptr, ODESC_BASE | ODESC_SPOIL);
@@ -231,7 +232,7 @@ static void kind_info(char *buf, size_t buf_len,
 /*
  * Create a spoiler file for items
  */
-static void spoil_obj_desc(cptr fname)
+static void spoil_obj_desc(const char *fname)
 {
 	int i, k, s, t, n = 0;
 
@@ -242,7 +243,7 @@ static void spoil_obj_desc(cptr fname)
 	char wgt[80];
 	char dam[80];
 
-	cptr format = "%-51s  %7s%6s%4s%9s\n";
+	const char *format = "%-51s  %7s%6s%4s%9s\n";
 
 	/* We use either ascii or system-specific encoding */
  	int encoding = (OPT(xchars_to_file)) ? SYSTEM_SPECIFIC : ASCII;
@@ -254,13 +255,13 @@ static void spoil_obj_desc(cptr fname)
 	/* Oops */
 	if (!fh)
 	{
-		msg_print("Cannot create spoiler file.");
+		msg("Cannot create spoiler file.");
 		return;
 	}
 
 
 	/* Header */
-	file_putf(fh, "Spoiler File -- Basic Items (%s)\n\n\n", VERSION_STRING);
+	file_putf(fh, "Spoiler File -- Basic Items (%s)\n\n\n", buildid);
 
 	/* More Header */
 	file_putf(fh, format, "Description", "Dam/AC", "Wgt", "Lev", "Cost");
@@ -343,12 +344,12 @@ static void spoil_obj_desc(cptr fname)
 	/* Check for errors */
 	if (!file_close(fh))
 	{
-		msg_print("Cannot close spoiler file.");
+		msg("Cannot close spoiler file.");
 		return;
 	}
 
 	/* Message */
-	msg_print("Successfully created a spoiler file.");
+	msg("Successfully created a spoiler file.");
 }
 
 
@@ -393,47 +394,25 @@ static const grouper group_artifact[] =
 /*
  * Hack -- Create a "forged" artifact
  */
-bool make_fake_artifact(object_type *o_ptr, byte name1)
+bool make_fake_artifact(object_type *o_ptr, struct artifact *artifact)
 {
-	int i;
+	object_kind *kind;
 
-	artifact_type *a_ptr = &a_info[name1];
-
-
-	/* Ignore "empty" artifacts */
-	if (!a_ptr->name) return FALSE;
+	/* Don't bother with empty artifacts */
+	if (!artifact->tval) return FALSE;
 
 	/* Get the "kind" index */
-	i = lookup_kind(a_ptr->tval, a_ptr->sval);
-
-	/* Oops */
-	if (!i) return (FALSE);
+	kind = lookup_kind(artifact->tval, artifact->sval);
+	if (!kind) return FALSE;
 
 	/* Create the artifact */
-	object_prep(o_ptr, &k_info[i], 0, MAXIMISE);
+	object_prep(o_ptr, kind, 0, MAXIMISE);
 
 	/* Save the name */
-	o_ptr->name1 = name1;
+	o_ptr->artifact = artifact;
 
 	/* Extract the fields */
-	o_ptr->pval = a_ptr->pval;
-	o_ptr->ac = a_ptr->ac;
-	o_ptr->dd = a_ptr->dd;
-	o_ptr->ds = a_ptr->ds;
-	o_ptr->to_a = a_ptr->to_a;
-	o_ptr->to_h = a_ptr->to_h;
-	o_ptr->to_d = a_ptr->to_d;
-	o_ptr->weight = a_ptr->weight;
-
-	/* Hack -- extract the "cursed" flags */
-	if (of_has(a_ptr->flags, OF_LIGHT_CURSE))
-		of_on(o_ptr->flags, OF_LIGHT_CURSE);
-
-	if (of_has(a_ptr->flags, OF_HEAVY_CURSE))
-		of_on(o_ptr->flags, OF_HEAVY_CURSE);
-
-	if (of_has(a_ptr->flags, OF_PERMA_CURSE))
-		of_on(o_ptr->flags, OF_PERMA_CURSE);
+	copy_artifact_data(o_ptr, artifact);
 
 	/* Success */
 	o_ptr->ident |= IDENT_FAKE;
@@ -444,7 +423,7 @@ bool make_fake_artifact(object_type *o_ptr, byte name1)
 /*
  * Create a spoiler file for artifacts
  */
-static void spoil_artifact(cptr fname)
+static void spoil_artifact(const char *fname)
 {
 	int i, j;
 
@@ -461,7 +440,7 @@ static void spoil_artifact(cptr fname)
 	/* Oops */
 	if (!fh)
 	{
-		msg_print("Cannot create spoiler file.");
+		msg("Cannot create spoiler file.");
 		return;
 	}
 
@@ -470,8 +449,7 @@ static void spoil_artifact(cptr fname)
 	text_out_file = fh;
 
 	/* Dump the header */
-	spoiler_underline(format("Artifact Spoilers for %s %s",
-	                         VERSION_NAME, VERSION_STRING), '=');
+	spoiler_underline(format("Artifact Spoilers for %s", buildid), '=');
 
 	/* List the artifacts by tval */
 	for (i = 0; group_artifact[i].tval; i++)
@@ -500,11 +478,11 @@ static void spoil_artifact(cptr fname)
 			object_wipe(i_ptr);
 
 			/* Attempt to "forge" the artifact */
-			if (!make_fake_artifact(i_ptr, (byte)j)) continue;
+			if (!make_fake_artifact(i_ptr, a_ptr)) continue;
 
 			/* Grab artifact name */
-			object_desc(buf, sizeof(buf), i_ptr,
-						ODESC_PREFIX | ODESC_COMBAT | ODESC_SPOIL);
+			object_desc(buf, sizeof(buf), i_ptr, ODESC_PREFIX |
+				ODESC_COMBAT | ODESC_EXTRA | ODESC_SPOIL);
 
 			/* Print name and underline */
 			spoiler_underline(buf, '-');
@@ -523,6 +501,8 @@ static void spoil_artifact(cptr fname)
 				NULL, TRUE), (a_ptr->weight / 10),
 				(a_ptr->weight % 10));
 
+			if (OPT(birth_randarts)) text_out("%s.\n", a_ptr->text);
+
 			/* Terminate the entry */
 			spoiler_blanklines(2);
 		}
@@ -531,12 +511,12 @@ static void spoil_artifact(cptr fname)
 	/* Check for errors */
 	if (!file_close(fh))
 	{
-		msg_print("Cannot close spoiler file.");
+		msg("Cannot close spoiler file.");
 		return;
 	}
 
 	/* Message */
-	msg_print("Successfully created a spoiler file.");
+	msg("Successfully created a spoiler file.");
 }
 
 
@@ -546,7 +526,7 @@ static void spoil_artifact(cptr fname)
 /*
  * Create a spoiler file for monsters
  */
-static void spoil_mon_desc(cptr fname)
+static void spoil_mon_desc(const char *fname)
 {
 	int i, n = 0;
 
@@ -572,13 +552,12 @@ static void spoil_mon_desc(cptr fname)
 	/* Oops */
 	if (!fh)
 	{
-		msg_print("Cannot create spoiler file.");
+		msg("Cannot create spoiler file.");
 		return;
 	}
 
 	/* Dump the header */
-	x_file_putf(fh, encoding, "Monster Spoilers for %s Version %s\n",
-	        VERSION_NAME, VERSION_STRING);
+	x_file_putf(fh, encoding, "Monster Spoilers for %s\n", buildid);
 	x_file_putf(fh, encoding, "------------------------------------------\n\n");
 
 	/* Dump the header */
@@ -607,7 +586,7 @@ static void spoil_mon_desc(cptr fname)
 	{
 		monster_race *r_ptr = &r_info[who[i]];
 
-		cptr name = r_ptr->name;
+		const char *name = r_ptr->name;
 
 		/* Get the "name" */
 		if (rf_has(r_ptr->flags, RF_QUESTOR))
@@ -664,12 +643,12 @@ static void spoil_mon_desc(cptr fname)
 	/* Check for errors */
 	if (!file_close(fh))
 	{
-		msg_print("Cannot close spoiler file.");
+		msg("Cannot close spoiler file.");
 		return;
 	}
 
 	/* Worked */
-	msg_print("Successfully created a spoiler file.");
+	msg("Successfully created a spoiler file.");
 }
 
 
@@ -683,7 +662,7 @@ static void spoil_mon_desc(cptr fname)
 /*
  * Create a spoiler file for monsters (-SHAWN-)
  */
-static void spoil_mon_info(cptr fname)
+static void spoil_mon_info(const char *fname)
 {
 	char buf[1024];
 	int i, n;
@@ -698,7 +677,7 @@ static void spoil_mon_info(cptr fname)
 	/* Oops */
 	if (!fh)
 	{
-		msg_print("Cannot create spoiler file.");
+		msg("Cannot create spoiler file.");
 		return;
 	}
 
@@ -707,8 +686,7 @@ static void spoil_mon_info(cptr fname)
 	text_out_file = fh;
 
 	/* Dump the header */
-	text_out("Monster Spoilers for %s Version %s\n",
-	        VERSION_NAME, VERSION_STRING);
+	text_out("Monster Spoilers for %s\n", buildid);
 	text_out("------------------------------------------\n\n");
 
 	/* Allocate the "who" array */
@@ -801,11 +779,11 @@ static void spoil_mon_info(cptr fname)
 	/* Check for errors */
 	if (!file_close(fh))
 	{
-		msg_print("Cannot close spoiler file.");
+		msg("Cannot close spoiler file.");
 		return;
 	}
 
-	msg_print("Successfully created a spoiler file.");
+	msg("Successfully created a spoiler file.");
 }
 
 
@@ -848,7 +826,7 @@ void do_cmd_spoilers(void)
 	screen_save();
 	clear_from(0);
 	menu_layout(spoil_menu, &SCREEN_REGION);
-	menu_select(spoil_menu, 0);
+	menu_select(spoil_menu, 0, FALSE);
 	screen_load();
 }
 
