@@ -62,11 +62,6 @@
  *   Append the pointer "v" (implementation varies).
  *   No legal modifiers.
  *
- * format("%b", int b)
- *   Append the integer formatted as binary.
- *   If a modifier of 1, 2, 3 or 4 is provided, then only append 2**n bits, not
- *   all 32.
- *
  * Format("%E", double r)
  * Format("%F", double r)
  * Format("%G", double r)
@@ -107,7 +102,7 @@
  *   Append the character "c".
  *   Do not use the "+" or "0" flags.
  *
- * Format("%s", cptr s)
+ * Format("%s", const char *s)
  *   Append the string "s".
  *   Do not use the "+" or "0" flags.
  *   Note that a "NULL" value of "s" is converted to the empty string.
@@ -191,15 +186,15 @@
  * the given buffer to a length of zero, and return a "length" of zero.
  * The contents of "buf", except for "buf[0]", may then be undefined.
  */
-size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
+size_t vstrnfmt(char *buf, size_t max, const char *fmt, va_list vp)
 {
-	cptr s;
+	const char *s;
 
 	/* The argument is "long" */
 	bool do_long;
 
-	/* The argument needs "processing" */
-	bool do_xtra;
+	/* The argument needs to be uppercased */
+	bool titlecase;
 
 	/* Bytes used in buffer */
 	size_t n;
@@ -213,14 +208,8 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 	/* Resulting string */
 	char tmp[1024];
 
-
-	/* Fatal error - no buffer length */
-	if (!max) quit("Called vstrnfmt() with empty buffer!");
-
-
-	/* Mega-Hack -- treat "no format" as "empty string" */
-	if (!fmt) fmt = "";
-
+	assert(max);
+	assert(fmt);
 
 	/* Begin the buffer */
 	n = 0;
@@ -293,11 +282,8 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 		/* Save the "percent" */
 		aux[q++] = '%';
 
-		/* Assume no "long" argument */
 		do_long = FALSE;
-
-		/* Assume no "xtra" processing */
-		do_xtra = FALSE;
+		titlecase = FALSE;
 
 		/* Build the "aux" string */
 		while (TRUE)
@@ -335,16 +321,6 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 					do_long = TRUE;
 				}
 
-				/* Mega-Hack -- handle "extra-long" request */
-				else if (*s == 'L')
-				{
-					/* Error -- illegal format char */
-					buf[0] = '\0';
-
-					/* Return "error" */
-					return (0);
-				}
-
 				/* Handle normal end of format sequence */
 				else
 				{
@@ -376,14 +352,9 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 					/* Skip the "*" */
 					s++;
 				}
-
-				/* Mega-Hack -- Handle 'caret' (for "uppercase" request) */
 				else if (*s == '^')
 				{
-					/* Note the "xtra" flag */
-					do_xtra = TRUE;
-
-					/* Skip the "^" */
+					titlecase = TRUE;
 					s++;
 				}
 
@@ -538,7 +509,7 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 			/* String */
 			case 's':
 			{
-				cptr arg;
+				const char *arg;
 				char arg2[1024];
 
 				/* XXX There is a big bug here: if one
@@ -548,7 +519,7 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 				 */
 
 				/* Get the next argument */
-				arg = tval.t == T_END ? va_arg(vp, cptr) : tval.u.s;
+				arg = tval.t == T_END ? va_arg(vp, const char *) : tval.u.s;
 
 				/* Hack -- convert NULL to EMPTY */
 				if (!arg) arg = "";
@@ -566,46 +537,6 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 				break;
 			}
 
-#if 0 /* Later */
-			/* Binary */
-			case 'b':
-			{
-				int arg;
-				size_t i, max = 32;
-				u32b bitmask;
-				char out[32 + 1];
-
-				/* Get the next argument */
-				arg = va_arg(vp, int);
-
-				/* Check our aux string */
-				switch (aux[0])
-				{
-					case '1': max = 2;  break;
-					case '2': max = 4;  break;
-					case '3': max = 8;  break;
-					case '4': max = 16; break;
-					default: 
-					case '5': max = 32; break;
-				}
-				/* Format specially */
-				for (i = 1; i <= max; i++, bitmask *= 2)
-				{
-					if (arg & bitmask) out[max - i] = '1';
-					else out[max - i] = '0';
-				}
-
-				/* Terminate */
-				out[max] = '\0';
-
-				/* Append the argument */
-				my_strcpy(tmp, out, sizeof tmp);
-
-				/* Done */
-				break;
-			}
-#endif
-
 			/* Oops */
 			default:
 			{
@@ -617,9 +548,7 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 			}
 		}
 
-
-		/* Mega-Hack -- handle "capitalization" */
-		if (do_xtra)
+		if (titlecase)
 		{
 			for (q = 0; tmp[q]; q++)
 			{
@@ -659,7 +588,7 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 /*
  * Add a formatted string to the end of a string
  */
-void strnfcat(char *str, size_t max, size_t *end, cptr fmt, ...)
+void strnfcat(char *str, size_t max, size_t *end, const char *fmt, ...)
 {
 	size_t len;
 
@@ -690,7 +619,7 @@ static size_t format_len = 0;
  * Do a vstrnfmt (see above) into a (growable) static buffer.
  * This buffer is usable for very short term formatting of results.
  */
-char *vformat(cptr fmt, va_list vp)
+char *vformat(const char *fmt, va_list vp)
 {
 	/* Initial allocation */
 	if (!format_buf)
@@ -735,7 +664,7 @@ void vformat_kill(void)
 /*
  * Do a vstrnfmt (see above) into a buffer of a given size.
  */
-size_t strnfmt(char *buf, size_t max, cptr fmt, ...)
+size_t strnfmt(char *buf, size_t max, const char *fmt, ...)
 {
 	size_t len;
 
@@ -761,7 +690,7 @@ size_t strnfmt(char *buf, size_t max, cptr fmt, ...)
  * Note that the buffer is (technically) writable, but only up to
  * the length of the string contained inside it.
  */
-char *format(cptr fmt, ...)
+char *format(const char *fmt, ...)
 {
 	char *res;
 	va_list vp;
@@ -785,7 +714,7 @@ char *format(cptr fmt, ...)
 /*
  * Vararg interface to plog()
  */
-void plog_fmt(cptr fmt, ...)
+void plog_fmt(const char *fmt, ...)
 {
 	char *res;
 	va_list vp;
@@ -808,7 +737,7 @@ void plog_fmt(cptr fmt, ...)
 /*
  * Vararg interface to quit()
  */
-void quit_fmt(cptr fmt, ...)
+void quit_fmt(const char *fmt, ...)
 {
 	char *res;
 	va_list vp;
