@@ -75,93 +75,94 @@ enum
 	MSG_BLESSED       = 62,
 	MSG_HERO          = 63,
 	MSG_BERSERK       = 64,
-	MSG_PROT_EVIL     = 65,
-	MSG_INVULN        = 66,
-	MSG_SEE_INVIS     = 67,
-	MSG_INFRARED      = 68,
-	MSG_RES_ACID      = 69,
-	MSG_RES_ELEC      = 70,
-	MSG_RES_FIRE      = 71,
-	MSG_RES_COLD      = 72,
-	MSG_RES_POIS      = 73,
-	MSG_STUN          = 74,
-	MSG_CUT           = 75,
-	MSG_STAIRS_UP     = 76,
-	MSG_STORE_ENTER   = 77,
-	MSG_STORE_LEAVE   = 78,
-	MSG_STORE_HOME    = 79,
-	MSG_MONEY1        = 80,
-	MSG_MONEY2        = 81,
-	MSG_MONEY3        = 82,
-	MSG_SHOOT_HIT     = 83,
-	MSG_STORE5        = 84,
-	MSG_LOCKPICK      = 85,
-	MSG_DISARM        = 86,
-	MSG_IDENT_BAD     = 87,
-	MSG_IDENT_EGO     = 88,
-	MSG_IDENT_ART     = 89,
-	MSG_BR_ELEMENTS   = 90,
-	MSG_BR_FROST      = 91,
-	MSG_BR_ELEC       = 92,
-	MSG_BR_ACID       = 93,
-	MSG_BR_GAS        = 94,
-	MSG_BR_FIRE       = 95,
-	MSG_BR_CONF       = 96,
-	MSG_BR_DISENCHANT = 97,
-	MSG_BR_CHAOS      = 98,
-	MSG_BR_SHARDS     = 99,
-	MSG_BR_SOUND      = 100,
-	MSG_BR_LIGHT      = 101,
-	MSG_BR_DARK       = 102,
-	MSG_BR_NETHER     = 103,
-	MSG_BR_NEXUS      = 104,
-	MSG_BR_TIME       = 105,
-	MSG_BR_INERTIA    = 106,
-	MSG_BR_GRAVITY    = 107,
-	MSG_BR_PLASMA     = 108,
-	MSG_BR_FORCE      = 109,
-	MSG_SUM_MONSTER   = 110,
-	MSG_SUM_ANGEL     = 111,
-	MSG_SUM_UNDEAD    = 112,
-	MSG_SUM_ANIMAL    = 113,
-	MSG_SUM_SPIDER    = 114,
-	MSG_SUM_HOUND     = 115,
-	MSG_SUM_HYDRA     = 116,
-	MSG_SUM_DEMON     = 117,
-	MSG_SUM_DRAGON    = 118,
-	MSG_SUM_HI_UNDEAD = 119,
-	MSG_SUM_HI_DRAGON = 120,
-	MSG_SUM_HI_DEMON  = 121,
-	MSG_SUM_WRAITH    = 122,
-	MSG_SUM_UNIQUE    = 123,
-	MSG_WIELD         = 124,
-	MSG_CURSED        = 125,
-	MSG_PSEUDOID      = 126,
-	MSG_HUNGRY        = 127,
-	MSG_NOTICE        = 128,
-	MSG_AMBIENT_DAY   = 129,
-	MSG_AMBIENT_NITE  = 130,
-	MSG_AMBIENT_DNG1  = 131,
-	MSG_AMBIENT_DNG2  = 132,
-	MSG_AMBIENT_DNG3  = 133,
-	MSG_AMBIENT_DNG4  = 134,
-	MSG_AMBIENT_DNG5  = 135,
-	MSG_CREATE_TRAP   = 136,
-	MSG_SHRIEK        = 137,
-	MSG_CAST_FEAR     = 138,
-	MSG_HIT_GOOD      = 139,
-	MSG_HIT_GREAT     = 140,
-	MSG_HIT_SUPERB    = 141,
-	MSG_HIT_HI_GREAT  = 142,
-	MSG_HIT_HI_SUPERB = 143,
-	MSG_SPELL         = 144,
-	MSG_PRAYER        = 145,
-	MSG_KILL_UNIQUE   = 146,
-	MSG_KILL_KING     = 147,
-	MSG_DRAIN_STAT    = 148,
-	MSG_MULTIPLY      = 149,
+	MSG_BOLD          = 65,
+	MSG_PROT_EVIL     = 66,
+	MSG_INVULN        = 67,
+	MSG_SEE_INVIS     = 68,
+	MSG_INFRARED      = 69,
+	MSG_RES_ACID      = 70,
+	MSG_RES_ELEC      = 71,
+	MSG_RES_FIRE      = 72,
+	MSG_RES_COLD      = 73,
+	MSG_RES_POIS      = 74,
+	MSG_STUN          = 75,
+	MSG_CUT           = 76,
+	MSG_STAIRS_UP     = 77,
+	MSG_STORE_ENTER   = 78,
+	MSG_STORE_LEAVE   = 79,
+	MSG_STORE_HOME    = 80,
+	MSG_MONEY1        = 81,
+	MSG_MONEY2        = 82,
+	MSG_MONEY3        = 83,
+	MSG_SHOOT_HIT     = 84,
+	MSG_STORE5        = 85,
+	MSG_LOCKPICK      = 86,
+	MSG_DISARM        = 87,
+	MSG_IDENT_BAD     = 88,
+	MSG_IDENT_EGO     = 89,
+	MSG_IDENT_ART     = 90,
+	MSG_BR_ELEMENTS   = 91,
+	MSG_BR_FROST      = 92,
+	MSG_BR_ELEC       = 93,
+	MSG_BR_ACID       = 94,
+	MSG_BR_GAS        = 95,
+	MSG_BR_FIRE       = 96,
+	MSG_BR_CONF       = 97,
+	MSG_BR_DISEN 	  = 98,
+	MSG_BR_CHAOS      = 99,
+	MSG_BR_SHARDS     = 100,
+	MSG_BR_SOUND      = 101,
+	MSG_BR_LIGHT      = 102,
+	MSG_BR_DARK       = 103,
+	MSG_BR_NETHER     = 104,
+	MSG_BR_NEXUS      = 105,
+	MSG_BR_TIME       = 106,
+	MSG_BR_INERTIA    = 107,
+	MSG_BR_GRAVITY    = 108,
+	MSG_BR_PLASMA     = 109,
+	MSG_BR_FORCE      = 110,
+	MSG_SUM_MONSTER   = 111,
+	MSG_SUM_ANGEL     = 112,
+	MSG_SUM_UNDEAD    = 113,
+	MSG_SUM_ANIMAL    = 114,
+	MSG_SUM_SPIDER    = 115,
+	MSG_SUM_HOUND     = 116,
+	MSG_SUM_HYDRA     = 117,
+	MSG_SUM_DEMON     = 118,
+	MSG_SUM_DRAGON    = 119,
+	MSG_SUM_HI_UNDEAD = 120,
+	MSG_SUM_HI_DRAGON = 121,
+	MSG_SUM_HI_DEMON  = 122,
+	MSG_SUM_WRAITH    = 123,
+	MSG_SUM_UNIQUE    = 124,
+	MSG_WIELD         = 125,
+	MSG_CURSED        = 126,
+	MSG_PSEUDOID      = 127,
+	MSG_HUNGRY        = 128,
+	MSG_NOTICE        = 129,
+	MSG_AMBIENT_DAY   = 130,
+	MSG_AMBIENT_NITE  = 131,
+	MSG_AMBIENT_DNG1  = 132,
+	MSG_AMBIENT_DNG2  = 133,
+	MSG_AMBIENT_DNG3  = 134,
+	MSG_AMBIENT_DNG4  = 135,
+	MSG_AMBIENT_DNG5  = 136,
+	MSG_CREATE_TRAP   = 137,
+	MSG_SHRIEK        = 138,
+	MSG_CAST_FEAR     = 139,
+	MSG_HIT_GOOD      = 140,
+	MSG_HIT_GREAT     = 141,
+	MSG_HIT_SUPERB    = 142,
+	MSG_HIT_HI_GREAT  = 143,
+	MSG_HIT_HI_SUPERB = 144,
+	MSG_SPELL         = 145,
+	MSG_PRAYER        = 146,
+	MSG_KILL_UNIQUE   = 147,
+	MSG_KILL_KING     = 148,
+	MSG_DRAIN_STAT    = 149,
+	MSG_MULTIPLY      = 150,
 
-	MSG_MAX           = 150,
+	MSG_MAX           = 151,
 	SOUND_MAX         = MSG_MAX
 };
 
