@@ -16,6 +16,7 @@
  *    are included in all such copies.  Other copyrights may also apply.
  */
 #include "angband.h"
+#include "z-term.h"
 
 
 /*
@@ -185,7 +186,6 @@
  *
  *   Term->init_hook = Init the term
  *   Term->nuke_hook = Nuke the term
- *   Term->user_hook = Perform user actions
  *   Term->xtra_hook = Perform extra actions
  *   Term->curs_hook = Draw (or Move) the cursor
  *   Term->bigcurs_hook = Draw (or Move) the big cursor (bigtile mode)
@@ -193,10 +193,6 @@
  *   Term->text_hook = Draw some text in the window
  *   Term->pict_hook = Draw some attr/chars in the window
  *
- * The "Term->user_hook" hook provides a simple hook to an implementation
- * defined function, with application defined semantics.  It is available
- * to the program via the "Term_user()" function.
- *
  * The "Term->xtra_hook" hook provides a variety of different functions,
  * based on the first parameter (which should be taken from the various
  * TERM_XTRA_* defines) and the second parameter (which may make sense
@@ -260,12 +256,6 @@
  * files use "white space" ("attr 1" / "char 32") to "erase" or "clear"
  * any window, for efficiency.
  *
- * The game "Angband" uses the "Term_user" hook to allow any of the
- * "main-xxx.c" files to interact with the user, by calling this hook
- * whenever the user presses the "!" key when the game is waiting for
- * a new command.  This could be used, for example, to provide "unix
- * shell commands" to the Unix versions of the game.
- *
  * See "main-xxx.c" for a simple skeleton file which can be used to
  * create a "visual system" for a new platform when porting Angband.
  */
@@ -280,7 +270,9 @@
  */
 term *Term = NULL;
 
-
+/* grumbles */
+int log_i = 0;
+int log_size = 0;
 
 
 /*** Local routines ***/
@@ -397,18 +389,6 @@ static errr term_win_copy(term_win *s, term_win *f, int w, int h)
 /*** External hooks ***/
 
 
-/*
- * Execute the "Term->user_hook" hook, if available (see above).
- */
-errr Term_user(int n)
-{
-	/* Verify the hook */
-	if (!Term->user_hook) return (-1);
-
-	/* Call the hook */
-	return ((*Term->user_hook)(n));
-}
-
 /*
  * Execute the "Term->xtra_hook" hook, if available (see above).
  */
@@ -578,7 +558,7 @@ void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1
  * a valid location, so the first "n" characters of "s" can all be added
  * starting at (x,y) without causing any illegal operations.
  */
-void Term_queue_chars(int x, int y, int n, byte a, cptr s)
+void Term_queue_chars(int x, int y, int n, byte a, const char *s)
 {
 	int x1 = -1, x2 = -1;
 
@@ -694,7 +674,7 @@ static void Term_fresh_row_pict(int y, int x1, int x2)
 			{
 				/* Draw pending attr/char pairs */
 				(void)((*Term->pict_hook)(fx, y, fn, &scr_aa[fx], &scr_cc[fx],
-				                          &scr_taa[fx], &scr_tcc[fx]));
+							  &scr_taa[fx], &scr_tcc[fx]));
 
 				/* Forget */
 				fn = 0;
@@ -703,7 +683,7 @@ static void Term_fresh_row_pict(int y, int x1, int x2)
 			/* Skip */
 			continue;
 		}
-		
+
 		/* Save new contents */
 		old_aa[x] = na;
 		old_cc[x] = nc;
@@ -720,7 +700,7 @@ static void Term_fresh_row_pict(int y, int x1, int x2)
 	{
 		/* Draw pending attr/char pairs */
 		(void)((*Term->pict_hook)(fx, y, fn, &scr_aa[fx], &scr_cc[fx],
-		                          &scr_taa[fx], &scr_tcc[fx]));
+					  &scr_taa[fx], &scr_tcc[fx]));
 	}
 }
 
@@ -1020,6 +1000,9 @@ static void Term_fresh_row_text(int y, int x1, int x2)
 	}
 }
 
+byte tile_width = 1;            /* Tile width in units of font width */
+byte tile_height = 1;           /* Tile height in units of font height */
+
 /* Helper variables for large cursor */
 bool bigcurs = FALSE;
 bool smlcurs = TRUE;
@@ -1354,7 +1337,7 @@ errr Term_fresh(void)
 		/* Draw the cursor */
 		if (!scr->cu && scr->cv)
 		{
-		        if ((((tile_width > 1)||(tile_height > 1)) && 
+			if ((((tile_width > 1)||(tile_height > 1)) &&
 			     (!smlcurs) && (Term->saved == 0) && (scr->cy > 0))
 			    || bigcurs)
 			{
@@ -1553,7 +1536,7 @@ errr Term_addch(byte a, char c)
  * positive value, future calls to either function will
  * return negative ones.
  */
-errr Term_addstr(int n, byte a, cptr buf)
+errr Term_addstr(int n, byte a, const char *buf)
 {
 	int k;
 
@@ -1618,7 +1601,7 @@ errr Term_putch(int x, int y, byte a, char c)
  */
 void Term_big_putch(int x, int y, byte a, char c)
 {
-        int hor, vert;
+	int hor, vert;
 
 	/* Avoid warning */
 	(void)c;
@@ -1626,39 +1609,39 @@ void Term_big_putch(int x, int y, byte a, char c)
 	/* No tall skinny tiles */
 	if (tile_width > 1)
 	{
-	        /* Horizontal first */
-	        for (hor = 0; hor <= tile_width; hor++)
+		/* Horizontal first */
+		for (hor = 0; hor <= tile_width; hor++)
 		{
-		        /* Queue dummy character */
-		        if (hor != 0)
-			{	
-			        if (a & 0x80)
-				        Term_putch(x + hor, y, 255, -1);
+			/* Queue dummy character */
+			if (hor != 0)
+			{
+				if (a & 0x80)
+					Term_putch(x + hor, y, 255, -1);
 				else
-				        Term_putch(x + hor, y, TERM_WHITE, ' ');
+					Term_putch(x + hor, y, TERM_WHITE, ' ');
 			}
 
 			/* Now vertical */
 			for (vert = 1; vert <= tile_height; vert++)
 			{
-			        /* Queue dummy character */
-			        if (a & 0x80)
-				        Term_putch(x + hor, y + vert, 255, -1);
+				/* Queue dummy character */
+				if (a & 0x80)
+					Term_putch(x + hor, y + vert, 255, -1);
 				else
-				        Term_putch(x + hor, y + vert, TERM_WHITE, ' ');
+					Term_putch(x + hor, y + vert, TERM_WHITE, ' ');
 			}
 		}
 	}
 	else
 	{
-	        /* Only vertical */
-	        for (vert = 1; vert <= tile_height; vert++)
+		/* Only vertical */
+		for (vert = 1; vert <= tile_height; vert++)
 		{
-		        /* Queue dummy character */
-		        if (a & 0x80)
-			        Term_putch(x, y + vert, 255, -1);
+			/* Queue dummy character */
+			if (a & 0x80)
+				Term_putch(x, y + vert, 255, -1);
 			else
-			        Term_putch(x, y + vert, TERM_WHITE, ' ');
+				Term_putch(x, y + vert, TERM_WHITE, ' ');
 		}
 	}
 }
@@ -1667,10 +1650,13 @@ void Term_big_putch(int x, int y, byte a, char c)
 /*
  * Move to a location and, using an attr, add a string
  */
-errr Term_putstr(int x, int y, int n, byte a, cptr s)
+errr Term_putstr(int x, int y, int n, byte a, const char *s)
 {
 	errr res;
 
+	if (!Term)
+		return 0;
+
 	/* Move first */
 	if ((res = Term_gotoxy(x, y)) != 0) return (res);
 
@@ -1736,11 +1722,11 @@ errr Term_erase(int x, int y, int n)
 		/* Save the "literal" information */
 		scr_aa[x] = na;
 		scr_cc[x] = nc;
-		
+
 		scr_taa[x] = 0;
 		scr_tcc[x] = 0;
 
-		/* Track minumum changed column */
+		/* Track minimum changed column */
 		if (x1 < 0) x1 = x;
 
 		/* Track maximum changed column */
@@ -1840,7 +1826,7 @@ errr Term_redraw(void)
 
 
 /*
- * Redraw part of a widow.
+ * Redraw part of a window.
  */
 errr Term_redraw_section(int x1, int y1, int x2, int y2)
 {
@@ -1910,12 +1896,9 @@ errr Term_get_cursor(bool *v)
  */
 errr Term_get_size(int *w, int *h)
 {
-	/* Access the cursor */
-	(*w) = Term->wid;
-	(*h) = Term->hgt;
-
-	/* Success */
-	return (0);
+	*w = Term ? Term->wid : 80;
+	*h = Term ? Term->hgt : 24;
+	return 0;
 }
 
 
@@ -1968,6 +1951,9 @@ errr Term_what(int x, int y, byte *a, char *c)
  */
 errr Term_flush(void)
 {
+	if (!Term)
+		return 0;
+
 	/* Hack -- Flush all events */
 	Term_xtra(TERM_XTRA_FLUSH, 0);
 
@@ -1979,28 +1965,40 @@ errr Term_flush(void)
 }
 
 
+/* sketchy keylogging pt. 2 */
+static void log_keypress(ui_event e)
+{
+	if (e.type != EVT_KBRD) return;
+	if (!e.key.code) return;
+
+	keylog[log_i] = e.key;
+	if (log_size < KEYLOG_SIZE) log_size++;
+	log_i = (log_i + 1) % KEYLOG_SIZE;
+}
+
+
 /*
  * Add a keypress to the "queue"
  */
-errr Term_keypress(int k)
+errr Term_keypress(keycode_t k, byte mods)
 {
-  /* Hack -- Refuse to enqueue non-keys */
-  if (!k) return (-1);
-  
-  /* Store the char, advance the queue */
-  Term->key_queue[Term->key_head].key = k;
-  Term->key_queue[Term->key_head].index = 0;
-  Term->key_queue[Term->key_head].type = EVT_KBRD;
-  Term->key_head++;
-  
-  /* Circular queue, handle wrap */
-  if (Term->key_head == Term->key_size) Term->key_head = 0;
-  
-  /* Success (unless overflow) */
-  if (Term->key_head != Term->key_tail) return (0);
-  
-  /* Problem */
-  return (1);
+	/* Hack -- Refuse to enqueue non-keys */
+	if (!k) return (-1);
+
+	/* Store the char, advance the queue */
+	Term->key_queue[Term->key_head].type = EVT_KBRD;
+	Term->key_queue[Term->key_head].key.code = k;
+	Term->key_queue[Term->key_head].key.mods = mods;
+	Term->key_head++;
+
+	/* Circular queue, handle wrap */
+	if (Term->key_head == Term->key_size) Term->key_head = 0;
+
+	/* Success (unless overflow) */
+	if (Term->key_head != Term->key_tail) return (0);
+
+	/* Problem */
+	return (1);
 }
 
 /*
@@ -2009,11 +2007,10 @@ errr Term_keypress(int k)
 errr Term_mousepress(int x, int y, char button)
 {
   /* Store the char, advance the queue */
-  Term->key_queue[Term->key_head].key = 0;
-  Term->key_queue[Term->key_head].mousex = x;
-  Term->key_queue[Term->key_head].mousey = y;
-  Term->key_queue[Term->key_head].index = button;
   Term->key_queue[Term->key_head].type = EVT_MOUSE;
+  Term->key_queue[Term->key_head].mouse.x = x;
+  Term->key_queue[Term->key_head].mouse.y = y;
+  Term->key_queue[Term->key_head].mouse.button = button;
   Term->key_head++;
   
   /* Circular queue, handle wrap */
@@ -2037,18 +2034,18 @@ errr Term_mousepress(int x, int y, char button)
  */
 errr Term_key_push(int k)
 {
-	ui_event_data ke;
+	ui_event ke;
 
 	if (!k) return (-1);
 
 	ke.type = EVT_KBRD;
-	ke.index = 0;
-	ke.key = k;
+	ke.key.code = k;
+	ke.key.mods = 0;
 
 	return Term_event_push(&ke);
 }
 
-errr Term_event_push(const ui_event_data *ke)
+errr Term_event_push(const ui_event *ke)
 {
 	/* Hack -- Refuse to enqueue non-keys */
 	if (!ke) return (-1);
@@ -2086,10 +2083,10 @@ errr Term_event_push(const ui_event_data *ke)
  *
  * Remove the keypress if "take" is true.
  */
-errr Term_inkey(ui_event_data *ch, bool wait, bool take)
+errr Term_inkey(ui_event *ch, bool wait, bool take)
 {
 	/* Assume no key */
-	ch->type = ch->key = 0;
+	memset(ch, 0, sizeof *ch);
 
 	/* Hack -- get bored */
 	if (!Term->never_bored)
@@ -2126,6 +2123,9 @@ errr Term_inkey(ui_event_data *ch, bool wait, bool take)
 	/* Extract the next keypress */
 	(*ch) = Term->key_queue[Term->key_tail];
 
+	/* sketchy key loggin */
+	log_keypress(*ch);
+
 	/* If requested, advance the queue, wrap around if necessary */
 	if (take && (++Term->key_tail == Term->key_size)) Term->key_tail = 0;
 
@@ -2198,6 +2198,9 @@ errr Term_load(void)
 
 		/* Free the old window */
 		(void)term_win_nuke(tmp);
+
+		/* Kill it */
+		FREE(tmp);
 	}
 
 	/* Assume change */
@@ -2238,7 +2241,8 @@ errr Term_resize(int w, int h)
 	term_win *hold_mem;
 	term_win *hold_tmp;
 
-	ui_event_data evt = { EVT_RESIZE, 0, 0, 0, 0 };
+	ui_event evt = EVENT_EMPTY;
+	evt.type = EVT_RESIZE;
 
 
 	/* Resizing is forbidden */
@@ -2528,7 +2532,7 @@ errr term_init(term *t, int w, int h, int k)
 	t->key_size = k;
 
 	/* Allocate the input queue */
-	t->key_queue = C_ZNEW(t->key_size, ui_event_data);
+	t->key_queue = C_ZNEW(t->key_size, ui_event);
 
 
 	/* Save the size */
@@ -2580,3 +2584,14 @@ errr term_init(term *t, int w, int h, int k)
 	/* Success */
 	return (0);
 }
+
+bool panel_contains(unsigned int y, unsigned int x)
+{
+	unsigned int hgt;
+	unsigned int wid;
+	if (!Term)
+		return TRUE;
+	hgt = SCREEN_HGT;
+	wid = SCREEN_WID;
+	return (y - Term->offset_y) < hgt && (x - Term->offset_x) < wid;
+}
