@@ -1,6 +1,8 @@
 #ifndef INCLUDED_ZTYPE_H
 #define INCLUDED_ZTYPE_H
 
+#include "h-basic.h"
+
 typedef struct data_panel data_panel;
 typedef struct type_union type_union;
 
@@ -43,9 +45,25 @@ struct data_panel
 	type_union value[MAX_FMT];	/* (short) arugment list */
 };
 
-struct point {
+struct loc {
 	int x;
 	int y;
 };
 
+struct loc loc(int x, int y);
+
+/*
+ * A set of points that can be constructed to apply a set of changes to
+ */
+struct point_set {
+	int n;
+	int allocated;
+	struct loc *pts;
+};
+
+struct point_set *point_set_new(int initial_size);
+void point_set_dispose(struct point_set *ps);
+void add_to_point_set(struct point_set *ps, int y, int x);
+int point_set_size(struct point_set *ps);
+
 #endif /* !INCLUDED_ZTYPE_H */
